// FILE: xch.cpp                Copyright (c), Symmetric Research, 2003-2007
//
// Source file for PARxCH specific portion of the Scope program
//
//      CParXchApp
//      CParXchWin
//      CHardwareDlg
//      CChannelDlg
//      CAcquireDlg
//
//

#include "xch.h"

/* parxch.h is now included by xch.h above
extern "C" {
#include "parxch.h"
}
*/



#ifdef GPS_AVAILABLE
#define HDR_PRODUCT_STRING   "SR PARxCH with GPS"
#else
#define HDR_PRODUCT_STRING   "SR PARxCH"
#endif



// Global application object

CParXchApp APP;


// General pointer to it (defined in scope.cpp)

extern CScopeApp *pAPP;



//***********************************************************************
//
// CLASS CParXchApp
//
// This class, derived from CScopeApp, contains the PARxCH specific
// application portion of the Scope data acquisition program.  It works
// in conjunction with its parent class to deal with acquisition, files,
// and ini.  The CParXchWin class takes care of PARxCH specific main
// window functions.
//
//


BEGIN_MESSAGE_MAP(CParXchApp, CScopeApp)
        //{{AFX_MSG_MAP(CParXchApp)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()



CScopeWindow* CParXchApp::NewScopeWindow() {

        TRACE0("CParXchApp::NewScopeWindow\n");

        CScopeWindow* pWnd = (CScopeWindow*) new CParXchWin;

        return( pWnd );
}

int CParXchApp::ExitInstance() {

        TRACE0("CParXchApp::ExitInstance\n");

                
        // Clean up any existing data array

        if (pDataArray)
                { free(pDataArray); pDataArray = NULL; }

                                
        return( CScopeApp::ExitInstance() );
}

int CParXchApp::InitAppData() {
        
        TRACE0("CParXchApp::InitAppData\n");

        int nvmax, nvtyp;

                

        // Initialize common global constants

        CScopeApp::InitAppData();


        

        // Set specific global application pointer

        pAPP = this;







        // Initialize max and min values and allocate scan arrays

        AppTitle.LoadString( IDR_SCOPE );
        AppRev.LoadString(   IDR_REV );

        // Set number of maximum and typical values per channel in a buffer.
        // 512 and 128 are chosen to be similar to PAR24B FIFO sizes.
        
        nvmax       = 512;
        nvtyp       = 128;

        DataPtSize  = sizeof(long);
        ROnsamples  = TRUE;

        MinDigitalChannel   = 0;
        MinGpsChannel       = 0;
        MinCounterChannel   = 0;
        MinAnaPlotChannel   = 0;
        MinDigPlotChannel   = 0;
        MinGpsPlotChannel   = 0;
        MinCntPlotChannel   = 0;
        MinTotalPlotChannel = 0;

        // The following "MaxAlloc" channel counts are use for array
        // allocations and apply to any PARxCH device, so they are the
        // maximum possible values over all PARxCH devices.

        MaxAllocTotalChannel     = PARXCH_MAX_ANALOG_CHANNELS  +
                                   PARXCH_MAX_DIGITAL_CHANNELS +
                                   PARXCH_MAX_GPS_CHANNELS     +
                                   PARXCH_MAX_COUNTER_CHANNELS;
        MaxAllocTotalPlotChannel = PARXCH_MAX_ANAPLOT_CHANNELS +
                                   PARXCH_MAX_DIGPLOT_CHANNELS +
                                   PARXCH_MAX_GPSPLOT_CHANNELS +
                                   PARXCH_MAX_CNTPLOT_CHANNELS;
        MaxAllocDigPlotChannel   = PARXCH_MAX_DIGPLOT_CHANNELS;

        
        // The following channel counts apply for a specific PARxCH
        // device and will be reset later once it has been selected, for
        // now just use the maximum for all PARxCH devices.

        MinAnalogChannel    = PARXCH_MAX_ANALOG_CHANNELS;
        
        MaxAnalogChannel    = PARXCH_MAX_ANALOG_CHANNELS;
        MaxDigitalChannel   = PARXCH_MAX_DIGITAL_CHANNELS;
        MaxGpsChannel       = PARXCH_MAX_GPS_CHANNELS;
        MaxCounterChannel   = PARXCH_MAX_COUNTER_CHANNELS;
        
        MaxAnaPlotChannel   = PARXCH_MAX_ANAPLOT_CHANNELS;
        MaxDigPlotChannel   = PARXCH_MAX_DIGPLOT_CHANNELS;
        MaxGpsPlotChannel   = PARXCH_MAX_GPSPLOT_CHANNELS;
        MaxCntPlotChannel   = PARXCH_MAX_CNTPLOT_CHANNELS;


        MinTotalChannel     = MinAnalogChannel;
        MaxTotalChannel     = MaxAnalogChannel + MaxDigitalChannel +
                              MaxGpsChannel    + MaxCounterChannel;
        MaxTotalPlotChannel = MaxAnaPlotChannel + MaxDigPlotChannel +
                              MaxGpsPlotChannel + MaxCntPlotChannel;


        MaxSample   = nvmax;
        MinAmpShift = 0;
        MaxAmpShift = 24;
        MinGainLog  = PARXCH_GAIN_MIN;
        MaxGainLog  = PARXCH_GAIN_MAX;
        MinSPS      = (float)PARXCH_SPS_MIN;
        MaxSPS      = (float)PARXCH_SPS_MAX;
        MaxTriggerThreshold = 8388608L;         // half of full scale 24 bits
        MaxTriggerDigioLine = 3;                // I/O line 3

        DefaultColor     = CS_BROWN;
        DefaultNdisplay  = MaxAnaPlotChannel;
        DefaultNsamples  = nvtyp;
        DefaultNblock    = (DefaultScreenWidth - HTITLEMARGIN) / DefaultNsamples;
        DefaultThreshold = MaxTriggerThreshold / 2;

        Nplotbuffers     = DefaultNblock;




        // Initialize additional specific constants
        
        LedValue        = 0;
        ConvertToOffset = 0x00000000;
        pDataArray      = NULL;
        AtodHandle      = BAD_DEVHANDLE;

        
        // Parameters that can be set from the ini file
        // are initialized separately in IniSetDefaults.  This
        // is done later, after the scan arrays have been
        // allocated.
        

        return( 1 );
}


int CParXchApp::IniVerifyData() {

	TRACE0("CParXchApp::IniVerifyData\n");

	int  i, ichan;
	int  State, AnalogState, DigitalState, GpsState, CounterState;
	

        // Since the channel counts depend on which PARxCH device
        // we are using, we check all the hardware parameters first.

	// Hardware parameter verification
	
        if (Inixch.HardDriverName.GetLength() == 0)
                Inixch.HardDriverName = PARXCH_0;

	if ( (Inixch.HardXchModel != PARXCH_MODEL_PAR1CH) &&
             (Inixch.HardXchModel != PARXCH_MODEL_PAR4CH) &&
             (Inixch.HardXchModel != PARXCH_MODEL_PAR8CH)
           )
		Inixch.HardXchModel = PARXCH_MODEL_UNKNOWN;

	if ( (Inixch.HardPortMode != PARXCH_PORT_MODE_EPP)     &&
             (Inixch.HardPortMode != PARXCH_PORT_MODE_BPP)     &&
             (Inixch.HardPortMode != PARXCH_PORT_MODE_ECP_EPP) &&
             (Inixch.HardPortMode != PARXCH_PORT_MODE_ECP_BPP) 
           )
		Inixch.HardPortMode = PARXCH_PORT_MODE_BPP;


	    
        // Device id unknown or default requested, so use default
        
        if (Inixch.HardXchModel == PARXCH_MODEL_UNKNOWN) {
                Inixch.HardXchModel = ParXchGetDefaultModel(
                                            Inixch.HardDriverName.GetBuffer(0) );
                Inixch.HardDriverName.ReleaseBuffer();
                }


        // Set channel counts based on selected model

        SetChannelCounts( 0 );




        
	// Verify common data
	
	CScopeApp::IniVerifyData();



	// Additional channel parameter verification

        AnalogState  =   CHECKED;
        DigitalState = UNCHECKED;
        GpsState     = UNCHECKED;
        CounterState = UNCHECKED;
        
        if (Ini.DigitalEnable)
                DigitalState = CHECKED;

        if (Ini.GpsEnable)
                GpsState = CHECKED;
        
        if (Ini.CounterEnable)
                CounterState = CHECKED;
        
        
        // Set state depending on type

        ichan = 0;

	for ( i = 0 ; i < MaxTotalChannel ; i++ ) {

                switch ( Ini.pScanChType[i] ) {
                        case CH_TYPE_ANALOG:
                                State = AnalogState;
                                break;
                        case CH_TYPE_DIGITAL:
                                State = DigitalState;
                                break;
                        case CH_TYPE_GPSMARK:
                                State = GpsState;
                                break;
                        case CH_TYPE_GPSCOUNTER:
                                State = CounterState;
                                break;
                        default:
                                State = UNCHECKED;
                        }

		Ini.pScanActive[i] = State;
                if (Ini.pScanActive[i] == CHECKED) {
                        Ini.pScanOrder[ichan] = i;
                        ichan++;
                        }
		}

        Ini.Nchannels = ichan;
        NchannelsPlot = Ini.Nchannels;
        if (DigitalState)
                NchannelsPlot += (MaxDigPlotChannel - MaxDigitalChannel);
        if (GpsState)
                NchannelsPlot += (MaxGpsPlotChannel - MaxGpsChannel);
        if (CounterState)
                NchannelsPlot += (MaxCntPlotChannel - MaxCounterChannel);

        


	// Acquisition parameter verification

	if ( (Ini.AcqGainLog < MinGainLog) || (Ini.AcqGainLog > MaxGainLog) )
		Ini.AcqGainLog = PARXCH_GAIN_MIN;
        
        if ((Inixch.AcqTurboLog < PARXCH_TURBO_MIN) ||
            (Inixch.AcqTurboLog > PARXCH_TURBO_MAX) )
                Inixch.AcqTurboLog = PARXCH_TURBO_MAX;

        if ((Ini.AcqGainLog+Inixch.AcqTurboLog < PARXCH_GAINTURBO_MIN) ||
            (Ini.AcqGainLog+Inixch.AcqTurboLog > PARXCH_GAINTURBO_MAX) )
                Inixch.AcqTurboLog = PARXCH_GAINTURBO_MAX - Ini.AcqGainLog;

	if ((Ini.AcqSPS < MinSPS) || (Ini.AcqSPS > MaxSPS)) {
		Ini.AcqSPS = (float)PARXCH_SPS_DEFAULT;
                CorrectSps(
                           &Ini.AcqSPS,
                           &Inixch.AcqDecimation,
                           &Inixch.AcqExtraDecimation
                          );
                }

        if ((Inixch.AcqDataCoding != PARXCH_DF_SIGNED) &&
            (Inixch.AcqDataCoding != PARXCH_DF_OFFSET) ) {
                Inixch.AcqDataCoding = PARXCH_DF_OFFSET;
                SetConvertToOffset();
                }
        
        if ((Inixch.AcqToggleLed != TRUE) && (Inixch.AcqToggleLed != FALSE))
                Inixch.AcqToggleLed = TRUE;
        

	return( 1 );
}

int CParXchApp::IniSetDefaults() {
        
	TRACE0("CParXchApp::IniSetDefaults\n");


        // Set the hardware parameters and the channel counts which
        // depend on them first, even before the common defaults.


	// Hardware parameter defaults
	
        Inixch.HardDriverName = PARXCH_0;
        Inixch.HardXchModel   = PARXCH_MODEL_UNKNOWN;
        Inixch.HardPortMode   = PARXCH_PORT_MODE_BPP;

        SetChannelCounts( 1 ); // Use Max values
	    


	// Set common data defaults
	
	CScopeApp::IniSetDefaults();



	// Acquisition parameter defaults

        Ini.AcqGainLog       = PARXCH_GAIN_1;
        Ini.AcqSPS           = (float)PARXCH_SPS_DEFAULT;
        
        Inixch.AcqTurboLog   = PARXCH_GAINTURBO_MAX - Ini.AcqGainLog;
        Inixch.AcqDataCoding = PARXCH_DF_OFFSET;
        Inixch.AcqToggleLed  = TRUE;
        
	CorrectSps(
                   &Ini.AcqSPS,
                   &Inixch.AcqDecimation,
                   &Inixch.AcqExtraDecimation
                  );

        SetConvertToOffset( );

	return( 1 );
}


int CParXchApp::IniSaveCurrent() {
        
	TRACE0("CParXchApp::IniSaveCurrent\n");

        
	// Save common data
	
	CScopeApp::IniSaveCurrent();


	// Save hardware parameters
	
        InixchSave.HardDriverName = Inixch.HardDriverName;
        InixchSave.HardXchModel   = Inixch.HardXchModel;
        InixchSave.HardPortMode   = Inixch.HardPortMode;


	// Common acquisition parameters saved in the parent class...

        InixchSave.AcqDataCoding = Inixch.AcqDataCoding;
        InixchSave.AcqToggleLed  = Inixch.AcqToggleLed;

        
	return( 1 );
}


int CParXchApp::IniRestoreCurrent() {
        
	TRACE0("CParXchApp::IniRestoreCurrent\n");

        
	// Restore common data
	
	CScopeApp::IniRestoreCurrent();


	// Restore hardware parameters
	
        Inixch.HardDriverName = InixchSave.HardDriverName;
        Inixch.HardXchModel   = InixchSave.HardXchModel;
        Inixch.HardPortMode   = InixchSave.HardPortMode;


	// Common acquisition parameters already restored in parent class ...

        Inixch.AcqDataCoding = InixchSave.AcqDataCoding;
        Inixch.AcqToggleLed  = InixchSave.AcqToggleLed;


        // Restore dependent values

        CorrectSps(
                   &Ini.AcqSPS,
                   &Inixch.AcqDecimation,
                   &Inixch.AcqExtraDecimation
                  );

        SetConvertToOffset( );
        
	return( 1 );
}


int CParXchApp::IniIsChanged() {
        
	TRACE0("CParXchApp::IniIsChanged\n");

        // Compare saved and current ini.  Return 1 if changed, 0 if the same

        
	// Check common data
	
	if (CScopeApp::IniIsChanged())
                return( 1 );


	// Check hardware parameters
	
        if (InixchSave.HardDriverName != Inixch.HardDriverName)  return( 1 );
        if (InixchSave.HardXchModel   != Inixch.HardXchModel)    return( 1 );
        if (InixchSave.HardPortMode   != Inixch.HardPortMode)    return( 1 );


		
        // Common acquisition parameters checked in the parent class...

        if (InixchSave.AcqDataCoding != Inixch.AcqDataCoding)  return( 1 );
        if (InixchSave.AcqToggleLed  != Inixch.AcqToggleLed)  return( 1 );
        
	return( 0 );
}


int CParXchApp::IniReadKeyword( void ) {
        
	TRACE0("CParXchApp::IniReadKeyword\n");

        int           itmp, iok, GainLog, TokenType;
        double        dtmp;


        // Process common keywords in scope parent class

        if ( CScopeApp::IniReadKeyword( ) )
        
                return( 1 );


        // Check for hardware constants
	// HardDriverName, HardXchModel, HardPortMode

        else if ( parse.KeywordEquals( "AtodModelName" )  ) {

                parse.GetToken();

                if ( parse.Match( "PAR1CH" ) )

                        Inixch.HardXchModel = PARXCH_MODEL_PAR1CH;

                else if ( parse.Match( "PAR4CH" ) )

                        Inixch.HardXchModel = PARXCH_MODEL_PAR4CH;

                else if ( parse.Match( "PAR8CH" ) )

                        Inixch.HardXchModel = PARXCH_MODEL_PAR8CH;

                else if ( parse.Match( "DEFAULT" ) )

                        Inixch.HardXchModel = PARXCH_MODEL_UNKNOWN;

                else
                        parse.ErrorIni( "AtodModelName must = {PAR1CH,PAR4CH,PAR8CH,DEFAULT}" );
                }



        else if ( parse.KeywordEquals( "AtodDriverName" ) ||
                  parse.KeywordEquals( "PortName" )        ) {

                TokenType = parse.GetToken();
                
                if ( TokenType != TOKEN_STRING && TokenType != TOKEN_SYMBOL)
                        parse.ErrorIni( "bad AtodDriverName string");
                else
                        Inixch.HardDriverName = parse.TOKEN;
                }



        else if ( parse.KeywordEquals( "AtodPortMode" ) ||
                  parse.KeywordEquals( "PortMode" )      ) {

                parse.GetToken();

                if ( parse.Match( "BPP" ) )

                        Inixch.HardPortMode = PARXCH_PORT_MODE_BPP;

                else if ( parse.Match( "EPP" ) )

                        Inixch.HardPortMode = PARXCH_PORT_MODE_EPP;

                else if ( parse.Match( "ECP/BPP" ) )

                        Inixch.HardPortMode = PARXCH_PORT_MODE_ECP_BPP;

                else if ( parse.Match( "ECP/EPP" ) )

                        Inixch.HardPortMode = PARXCH_PORT_MODE_ECP_EPP;

                else
                        parse.ErrorIni( "PortMode must = {BPP,EPP,ECP/BPP,ECP/EPP}" );

                }





	// Initialize acquisition constants:
	// AcqGain, AcqSPS

        else if ( parse.KeywordEquals( "SamplingRate" ) ) {

                iok = parse.GetFloat( &dtmp, "SamplingRate must = float" );
                
                if ( iok && dtmp >= (double)MinSPS && dtmp <= (double)MaxSPS ) {
                        Ini.AcqSPS = (float)dtmp;
                	CorrectSps(
                                   &Ini.AcqSPS,
                                   &Inixch.AcqDecimation,
                                   &Inixch.AcqExtraDecimation
                                  );
                        }
                else {
                        sprintf( pAPP->szTemp,
                                 "SamplingRate must be %8.2f to %8.2f Hz",
                                 MinSPS, MaxSPS );
                        parse.ErrorIni( pAPP->szTemp );
                        }
                }



        else if ( parse.KeywordEquals( "Gain" ) ) {

                iok = parse.GetInt( &itmp, "illegal Gain integer value" );

                if (iok)
                        GainLog = ConvertGain( itmp );
                else
                        GainLog = -1;

                if ( GainLog == -1 )
                        parse.ErrorIni( "Gain must = {1,2,4,8,16}" );
                else {
                        Ini.AcqGainLog     = GainLog;
                        Inixch.AcqTurboLog = PARXCH_GAINTURBO_MAX -
                                             Ini.AcqGainLog;
                	CorrectSps(
                                   &Ini.AcqSPS,
                                   &Inixch.AcqDecimation,
                                   &Inixch.AcqExtraDecimation
                                  );
                        }
                }



        else if ( parse.KeywordEquals( "ToggleLed" ) ) {

                iok = parse.GetOnOff( &itmp, "ToggleLed must = {ON,OFF}" );

                if (iok)
                        Inixch.AcqToggleLed = itmp;
                }



        // Check for data coding constant

        else if ( parse.KeywordEquals( "DataCoding" ) ) {

                parse.GetToken();

                if ( parse.Match( "SIGNED" ) )
                        Inixch.AcqDataCoding = PARXCH_DF_SIGNED;

                else if ( parse.Match( "OFFSET" ) )
                        Inixch.AcqDataCoding = PARXCH_DF_OFFSET;

                else
                        parse.ErrorIni( "DataCoding must = {SIGNED,OFFSET}" );
                
                SetConvertToOffset( );
                }


        // Unused options
        // These options are not used in scope at this time, but they
        // are used by the simple acquisition program simp.exe and
        // we want scope to work with simp.ini files.
        
        
        else if ( parse.KeywordEquals( "Calibration" )     ||
                  parse.KeywordEquals( "Decimation" )      ||
                  parse.KeywordEquals( "ExtraDecimation" ) ) {

                parse.GetToken();  // Ignore them
                
                }


        // Unrecognized specific keyword
        
        else 
                return( 0 );


        return( 1 );
}

int CParXchApp::IniWriteKeyword( FILE *OutFp ) {
        
	TRACE0("CParXchApp::IniWriteKeyword\n");

        char *NameString, *ModeString, *DataCodingString, *ToggleLedString;


        // Set up output strings

        switch ( Inixch.HardXchModel ) {
                case PARXCH_MODEL_PAR1CH: NameString = "PAR1CH";   break;
                case PARXCH_MODEL_PAR4CH: NameString = "PAR4CH";   break;
                case PARXCH_MODEL_PAR8CH: NameString = "PAR8CH";   break;
                default:               NameString = "UNKNOWN";  break;
                }


        switch ( Inixch.HardPortMode ) {
                case PARXCH_PORT_MODE_BPP:     ModeString = "BPP";      break;
                case PARXCH_PORT_MODE_EPP:     ModeString = "EPP";      break;
                case PARXCH_PORT_MODE_ECP_BPP: ModeString = "ECP/BPP";  break;
                case PARXCH_PORT_MODE_ECP_EPP: ModeString = "ECP/EPP";  break;
                default:                       ModeString = "UNKNOWN";  break;
                }


        switch ( Inixch.AcqDataCoding ) {
                case PARXCH_DF_OFFSET: DataCodingString = "OFFSET";  break;
                case PARXCH_DF_SIGNED: DataCodingString = "SIGNED";  break;
                default:               DataCodingString = "UNKNOWN"; break;
                }

        ToggleLedString = ( Inixch.AcqToggleLed ? "ON" : "OFF" );

        

        // Print Hardware keywords
        
        fprintf( OutFp, "         AtodDriverName = %s\n", Inixch.HardDriverName );
        fprintf( OutFp, "          AtodModelName = %s\n", NameString );
        fprintf( OutFp, "           AtodPortMode = %s\n", ModeString );
        fprintf( OutFp, "\n" );


        // Print Acquisition keywords (common ones printed by parent class)

        fprintf( OutFp, "             DataCoding = %s\n",     DataCodingString );
        fprintf( OutFp, "              ToggleLed = %s\n",     ToggleLedString );
        fprintf( OutFp, "\n" );

        

        // Print common keywords
        
        CScopeApp::IniWriteKeyword( OutFp );

        

        
        return( 1 );
}



int CParXchApp::ConvertGain( int value ) {
        
	TRACE0("CParXchApp::ConvertGain\n");
        
        int lnvalue, gtable[] = { 1, 2, 4, 8, 16 };


        // Look up the log of the gain value.

        for ( lnvalue = PARXCH_GAIN_MIN ; lnvalue <= PARXCH_GAIN_MAX ; lnvalue++ ) {

                if ( gtable[ lnvalue ] == value )

                        return( lnvalue );
                }

        return( -1 );
}

int CParXchApp::CorrectSps( float *sps, int *decimation, int *extradecimation ) {

        TRACE0("CParXchApp::CorrectSps\n");
        
        int    tb1, tb2, dec1, dec2, extdec1, extdec2, dummy;
        double sps0, sps1, sps2, diff1, diff2;
        
        sps0 = (double)*sps;

        ParXchSpsGainToTde( sps0, Ini.AcqGainLog, &tb1, &dec1, &extdec1 );
        
        tb2     = tb1;
        dec2    = dec1 + 1;
        extdec2 = extdec1;
        
        ParXchTdeToSpsGain( tb1, dec1, extdec1, &sps1, &dummy );
        ParXchTdeToSpsGain( tb2, dec2, extdec2, &sps2, &dummy );

        diff1 = sps1 - sps0;
        diff2 = sps0 - sps2;

        if (diff1 < diff2) {
                *sps = (float)sps1;
                *decimation = dec1;
                *extradecimation = extdec1;
                return(dec1);
                }
        else {
                *sps = (float)sps2;
                *decimation = dec2;
                *extradecimation = extdec2;
                return(dec2);
                }

}

int CParXchApp::SetConvertToOffset( void ) {

        TRACE0("CParXchApp::SetConvertToOffset\n");
        
        // Set xor constant that will correct data to offset format
        
        if (Inixch.AcqDataCoding == PARXCH_DF_SIGNED)
                ConvertToOffset = 0x00800000;
        
        else // (Inixch.AcqDataCoding == PARXCH_DF_OFFSET)
                ConvertToOffset = 0x00000000;

        return( 1 );
}

int CParXchApp::SetChannelCounts( int UseMax ) {

        TRACE0("CParXchApp::SetChannelCounts\n");

        int i, j, XchModel;


        // Choose which model to use, maximum or normal
        
        if (UseMax)
                XchModel = PARXCH_MODEL_UNKNOWN;
        else
                XchModel = Inixch.HardXchModel;



        // Set channel counts based on specific PARxCH model

        switch( XchModel ) {
                case PARXCH_MODEL_PAR1CH:
                        MinAnalogChannel  = PAR1CH_ANALOG_CHANNELS;
                        MaxAnalogChannel  = PAR1CH_ANALOG_CHANNELS;
                        MaxDigitalChannel = PAR1CH_DIGITAL_CHANNELS;
                        MaxGpsChannel     = PAR1CH_GPS_CHANNELS;
                        MaxCounterChannel = PAR1CH_COUNTER_CHANNELS;
                        MaxAnaPlotChannel = PAR1CH_ANAPLOT_CHANNELS;
                        MaxDigPlotChannel = PAR1CH_DIGPLOT_CHANNELS;
                        MaxGpsPlotChannel = PAR1CH_GPSPLOT_CHANNELS;
                        MaxCntPlotChannel = PAR1CH_CNTPLOT_CHANNELS;
                        AtodModelName     = PAR1CH_MODELNAME;
                        break;

                case PARXCH_MODEL_PAR4CH:
                        MinAnalogChannel  = PAR4CH_ANALOG_CHANNELS;
                        MaxAnalogChannel  = PAR4CH_ANALOG_CHANNELS;
                        MaxDigitalChannel = PAR4CH_DIGITAL_CHANNELS;
                        MaxGpsChannel     = PAR4CH_GPS_CHANNELS;
                        MaxCounterChannel = PAR4CH_COUNTER_CHANNELS;
                        MaxAnaPlotChannel = PAR4CH_ANAPLOT_CHANNELS;
                        MaxDigPlotChannel = PAR4CH_DIGPLOT_CHANNELS;
                        MaxGpsPlotChannel = PAR4CH_GPSPLOT_CHANNELS;
                        MaxCntPlotChannel = PAR4CH_CNTPLOT_CHANNELS;
                        AtodModelName     = PAR4CH_MODELNAME;
                        break;
                        
                case PARXCH_MODEL_PAR8CH:
                        MinAnalogChannel  = PAR8CH_ANALOG_CHANNELS;
                        MaxAnalogChannel  = PAR8CH_ANALOG_CHANNELS;
                        MaxDigitalChannel = PAR8CH_DIGITAL_CHANNELS;
                        MaxGpsChannel     = PAR8CH_GPS_CHANNELS;
                        MaxCounterChannel = PAR8CH_COUNTER_CHANNELS;
                        MaxAnaPlotChannel = PAR8CH_ANAPLOT_CHANNELS;
                        MaxDigPlotChannel = PAR8CH_DIGPLOT_CHANNELS;
                        MaxGpsPlotChannel = PAR8CH_GPSPLOT_CHANNELS;
                        MaxCntPlotChannel = PAR8CH_CNTPLOT_CHANNELS;
                        AtodModelName     = PAR8CH_MODELNAME;
                        break;
                        
                default:
                        MinAnalogChannel  = PARXCH_MAX_ANALOG_CHANNELS;
                        MaxAnalogChannel  = PARXCH_MAX_ANALOG_CHANNELS;
                        MaxDigitalChannel = PARXCH_MAX_DIGITAL_CHANNELS;
                        MaxGpsChannel     = PARXCH_MAX_GPS_CHANNELS;
                        MaxCounterChannel = PARXCH_MAX_COUNTER_CHANNELS;
                        MaxAnaPlotChannel = PARXCH_MAX_ANAPLOT_CHANNELS;
                        MaxDigPlotChannel = PARXCH_MAX_DIGPLOT_CHANNELS;
                        MaxGpsPlotChannel = PARXCH_MAX_GPSPLOT_CHANNELS;
                        MaxCntPlotChannel = PARXCH_MAX_CNTPLOT_CHANNELS;
                        AtodModelName     = PARXCH_MODELNAME;
                        break;
                }

        MinTotalChannel     = MinAnalogChannel;
        MaxTotalChannel     = MaxAnalogChannel + MaxDigitalChannel +
                              MaxGpsChannel    + MaxCounterChannel;
        MaxTotalPlotChannel = MaxAnaPlotChannel + MaxDigPlotChannel +
                              MaxGpsPlotChannel + MaxCntPlotChannel;
        
        DefaultNdisplay     = MaxAnaPlotChannel;



        // Reset defaults based on absoluted PARxCH maximums

        j = MaxAllocTotalChannel;
        
        for ( i = 0 ; i < j ; i++ )
                Ini.pScanChType[i] = CH_TYPE_NONE;



        // Update channel types based on new counts

        for ( i = 0 ; i < MaxAnalogChannel ; i++ )
                Ini.pScanChType[i] = CH_TYPE_ANALOG;
        
        j = MaxAnalogChannel;
        for ( i = j ; i < j + MaxDigitalChannel ; i++ )
                Ini.pScanChType[i] = CH_TYPE_DIGITAL;
        
        j = MaxAnalogChannel + MaxDigitalChannel;
        for ( i = j ; i < j + MaxGpsChannel ; i++ )
                Ini.pScanChType[i] = CH_TYPE_GPSMARK;

        j = MaxAnalogChannel + MaxDigitalChannel + MaxGpsChannel;
        for ( i = j ; i < j + MaxCounterChannel ; i++ )
                Ini.pScanChType[i] = CH_TYPE_GPSCOUNTER;



        // Keep CounterEnable in synch with GpsEnable until there
        // is a dialog button for it
        
        if (MaxCounterChannel > 0 && Ini.GpsEnable)
                Ini.CounterEnable = ON;
        else
                Ini.CounterEnable = OFF;
        
        return( 1 );
}

int CParXchApp::AtodMeasure( int *ival ) {

        TRACE0("CParXchApp::AtodMeasure\n");
        return( 0 );
}

int CParXchApp::AtodInit() {
        
        TRACE0("CParXchApp::AtodInit\n");

        int  errorcode;
        long CtrlReg;


        // Skip opening the PARxCH and GPS if the user has indicated
        // the A/D is not available
        
        if ( !Ini.AtodEnable ) {
                AtodHandle = BAD_DEVHANDLE;
                GpsHandle  = BAD_DEVHANDLE;
                ErrString  = "PARxCH open skipped, AtodEnable = OFF.";
                LogMessage( MSGLEVEL_MSG, MSGDLG_SHOW, ERROR_NOT_FATAL, NULL );
                return( 0 );
                }


        

        // Init A/D

        CorrectSps(
                   &Ini.AcqSPS, 
                   &Inixch.AcqDecimation,
                   &Inixch.AcqExtraDecimation
                  );

        SetConvertToOffset( );
        
        AtodHandle = ParXchFullOpen(
                                     Inixch.HardDriverName.GetBuffer(0),
                                     Inixch.HardXchModel,
                                     Inixch.HardPortMode, 
                                     Inixch.AcqDataCoding,
                                     Ini.AcqGainLog,
                                     Inixch.AcqTurboLog,
                                     Inixch.AcqDecimation,
                                     Inixch.AcqExtraDecimation,
                                     PARXCH_UNUSED,
                                     &CtrlReg,
                                     &errorcode
                                   );
        Inixch.HardDriverName.ReleaseBuffer();
        


        // Set specific error information

        if ( (errorcode >= PARXCH_ERROR_DATA_FORMAT) &&
             (errorcode <= PARXCH_ERROR_DECIMATION)  )
                  
                TmpString = "Invalid parameter (df,gain,turbo,decim).\n";
                
        else if (errorcode == PARXCH_ERROR_DRIVER_NOT_OPEN)
        
                TmpString = "Check that the device driver is installed.\n";
                
        else if (errorcode != PARXCH_ERROR_NONE)
        
                TmpString = "Check power, cable, driver name, Xch model, and port mode.\n";
                
        else
                TmpString = "";

        sprintf( szTemp, "(%s)", PARXCH_ERROR_MSG[errorcode] );


        // Set general error info, close if needed, and return failure
                                                        
        if (AtodHandle == BAD_DEVHANDLE) {
                ErrString  = "Could not open and initialize the ";
                ErrString += AtodModelName;
                ErrString += ".  ";
                ErrString += szTemp;
                return( 0 );
                }
        if (errorcode != PARXCH_ERROR_NONE) {
                ErrString  = "Could not initialize the ";
                ErrString += AtodModelName;
                ErrString += ".  ";
                ErrString += szTemp;
                ParXchClose( AtodHandle );
                AtodHandle = BAD_DEVHANDLE;
                return( 0 );
                }


        // Set user led
        
        LedValue = 0;
        ParXchUserLed( AtodHandle, LedValue );



        // Initialize GPS (if requested)
        
        if ( !GpsTimingInit() ) {
                ParXchClose( AtodHandle );
                AtodHandle = BAD_DEVHANDLE;
                return( 0 );
                }



        // Return success
        
        return( 1 );
}


int CParXchApp::AtodStart() {
        
        TRACE0("CParXchApp::AtodStart\n");

        // Initialize current position counter.  Used for gathering a
        // whole buffer of ready data before indicating ready.
        
        Nextpt = 0;

        
        // Don't start anything if the A/D is not available
        
        if ( !Ini.AtodEnable )
                return( 1 );



        // Start the GPS (if requested), then the A/D

        GpsTimingStart( );
        ParXchStart( AtodHandle );


        // Write start message to log, but don't show dialog

        ErrString = "Starting acquisition.";
        LogMessage( MSGLEVEL_MSG, MSGDLG_NO_SHOW, ERROR_NOT_FATAL, NULL );

        return( 1 );
}


int CParXchApp::AtodStop() {
        
        TRACE0("CParXchApp::AtodStop\n");

        ParXchStop( AtodHandle );       // Stop A/D execution
        GpsTimingStop();                // Stop + close the GPS (if requested)
        ParXchClose( AtodHandle );      // Close the A/D driver
        AtodHandle = BAD_DEVHANDLE;


        // Write stop message to log, but don't show dialog

        ErrString = "Stopping acquisition.";
        LogMessage( MSGLEVEL_MSG, MSGDLG_NO_SHOW, ERROR_NOT_FATAL, NULL );

        return( 1 );
}


int CParXchApp::AtodAttachGpsTiming() {

        int errorcode;
        
        TRACE0("CParXchApp::AtodAttachGpsTiming\n");

        // Attach the PARGPS driver to the PARxCH driver.

        if ( !ParXchAttachGps( AtodHandle, GpsHandle, &errorcode ) ) {
                ErrString.Format( "PARxCH could not attach PARGPS (%s).",
                                  PARXCH_ERROR_MSG[errorcode] );
                return( 0 );
                }


        // Check that an interrupt number is actually assigned to the driver.
        // If not, give a warning since the user MAY really want to do this.

        if ( ParXchInterruptGetNumber( AtodHandle ) == 0 ) {
                ErrString.Format( "No IRQ assigned.  Reinstall PARxCH driver if IRQ required." );
                LogMessage( MSGLEVEL_WARN, MSGDLG_SHOW, ERROR_NOT_FATAL, NULL );
                }

        return( 1 );
}


int CParXchApp::AtodReleaseGpsTiming() {
        
        TRACE0("CParXchApp::AtodReleaseGpsTiming\n");

        ParXchReleaseGps( AtodHandle  );
        
        return( 1 );
}


int CParXchApp::AtodCheckReady() {

        int Npts, ErrorCode, ReadRequest;
        
        TRACE0("CParXchApp::AtodCheckReady\n");


#define NREMAIN_DATA ((unsigned int)(Nbufferpts-Nextpt))

        // Indicate what to read

        ReadRequest = PARXCH_REQUEST_ANALOG;
        if (Ini.DigitalEnable)
                ReadRequest |= PARXCH_REQUEST_DIGITAL;
        if (Ini.GpsEnable)
                ReadRequest |= PARXCH_REQUEST_GPS;
        if (Ini.CounterEnable)
                ReadRequest |= PARXCH_REQUEST_COUNTER;

        
        
        // Try reading data, if none is ready, Npts will be 0

        Npts = ParXchFullReadData(
                                  AtodHandle,
                                  &pDataArray[Nextpt],
                                  NREMAIN_DATA,
                                  ReadRequest,
                                  &ErrorCode
                                 );
        Nextpt += Npts;


        // Check for an error during read

        if (ErrorCode != PARXCH_ERROR_NONE) {
                ErrString.Format(
                                 "ReadData failed with error # %d: %s.",
                                 ErrorCode,
                                 (LPCTSTR) PARXCH_ERROR_MSG[ErrorCode]
                                );
                TmpString  = "\nThis error may occur if the wrong XchModel is used,\n";
                TmpString += "if the parallel cable is disconnected or if the PARxCH\n";
                TmpString += "lost power.  The XchModel can be set from the hardware\n";
                TmpString += "dialog.  Run diags\\diag std to verify the board and find\n";
                TmpString += "the correct XchModel settings.";
                LogMessage( MSGLEVEL_ERR, MSGDLG_SHOW, ERROR_MAYBE_FATAL, &TmpString );
                return( -1 );
                }





        // Read any ready GPS PPS and serial data (if requested)

        if (Ini.GpsEnable)
                if (!GpsTimingGetReadyData(  ) ) 
                        return( -1 );


        
        
        // If 1 analog buffer is full
        
        if (NREMAIN_DATA == 0) {

                // Save number of gps data read while 1 analog buffer filled
                
                Npps    = NextptPps;
                Nserial = NextptSerial;


                // Reset current position counters for next buffer
                
                Nextpt       = 0;
                NextptPps    = 0;
                NextptSerial = 0;
                
                return( 1 );  // Indicate buffer ready
                }
        else
                return( 0 );  // Buffer not ready yet
}


int CParXchApp::AtodCheckOverflow() {
        
        TRACE0("CParXchApp::AtodCheckOverflow\n");

        return( ParXchOverflow( AtodHandle ) );
}


int CParXchApp::AtodOverflowRecovery() {
        TRACE0("CParXchApp::AtodOverflowRecovery\n");

        // After an overflow has been detected, the PARxCH must be stopped
        // and restarted.  Some users might want to read out the valid data
        // data currently saved in the fifo first.  If so, put that code
        // here ...
        



        // Perform orderly shutdown and restart after overflow.

        CScopeWindow *pWnd = (CScopeWindow*)m_pMainWnd;

        if (!pWnd) {
                ErrString = "No scope window available for overflow recovery.";
                LogMessage( MSGLEVEL_ERR, MSGDLG_SHOW, ERROR_FATAL, NULL );
                return( 0 );
                }


        // Shutdown

        pWnd->OnStopProgram();


        // Data is collected in a temporary file until the full
        // requested amount has been acquired.  When an error occurs,
        // add an end of file records, compute a new name ending in .FIX,
        // and rename the partial file of data collected up to the time
        // of the error.  Then we increment the sequential start number
        // so scope will leave a gap in the normal .DAT file numbers at
        // time of the error.

        if ( Ini.OutFileFormat == FILEFMT_DAT ) {
                TmpFileName.Format( "%s\\SCOPE.TMP", (LPCTSTR)Ini.OutDir );
                TmpFileOpen( &TmpFile );
                SrDatRecordWrite( TmpFile.m_pStream, SRDAT_TAGID_EOF, NULL, 0 );
                TmpFileClose( &TmpFile );
                }

        FinalFileName.Format( "%s\\%08ld.FIX", (LPCTSTR)Ini.OutDir, Ini.OutSeqStart );
        TmpFileRename();
        Ini.OutSeqStart++;


        // Restart

        pWnd->OnRunProgram();


        return( 1 );
}


int CParXchApp::AtodGetData() {
        
        TRACE0("CParXchApp::AtodGetData\n");

        int npts;

                
        // Twiddle the yellow LED to show it's working

        if (Inixch.AcqToggleLed == TRUE) {
                
                LedValue ^= 1;
                ParXchUserLed( AtodHandle, LedValue );
                }


        
        // Data is actually read incrementally in AtodCheckReady
        // which also calls GpsTimingGetReadyData to incrementally read
        // GPS data 
        
        npts = Nbufferpts;


        
        return( npts );
}

#define CLEARTOPBIT  0x7FFFFFFF
#define CLEARTOPBYTE 0x00FFFFFF

int CParXchApp::AtodDemuxData() {

        TRACE0("CParXchApp::AtodDemuxData\n");

        int i, jc, jp, jscan, k, na, nc, nd, ns, np, kmux, kdmx, startpt;
        long pt;

        // Demux raw data values into the plottable demuxed array
        // The raw data can be in either signed or offset format.
        // But, we always want the demuxed data to be in offset 
        // format to easy plotting computations.  We do this conversion
        // by and-ing out the top byte and xor-ing the 24th bit.
        // The xor constant is chosen elsewhere based on the data
        // format so that no change occurs when the input data is
        // already in offset format.

        // Note that word offset is used with two different meanings.
        // One is DC offset meaning the background level of a channel
        // and the other is offset format meaning data that goes from
        // 0x00000000 to 0x00FFFFFF.  The DC offset below is computed
        // based on the demuxed, offset format, data.


        startpt = Iplotbuff * Ini.Nsamples;


        na = Ini.Nsamples;
        nc = Ini.Nchannels;
        ns = NsamplesPlot;
        np = NchannelsPlot;
        nd = MaxDigPlotChannel;


        jp = 0;
        for ( jc = 0 ; jc < nc ; jc++ ) {

                jscan = Ini.pScanOrder[jc];
                
                // Standard processing for analog channels

                if (Ini.pScanChType[jscan] == CH_TYPE_ANALOG) {

                        pTMPoffset[jp] = 0.0;
                        for ( i = 0 ; i < na ; i++ ) {
                                kmux = i*nc+jc;
                                kdmx = jp*ns+i+startpt;
                                pDemuxArray[kdmx] = (pDataArray[kmux]
                                                     & CLEARTOPBYTE)
                                                     ^ ConvertToOffset;
                                pTMPoffset[jp] += (double)pDemuxArray[kdmx];
                                } // end for i
                        jp++;

                        } // end analog processing



                // Special processing for GPS Mark channel (no offset convert)
                
                else if (Ini.pScanChType[jscan] == CH_TYPE_GPSMARK) {

                        for ( i = 0 ; i < na ; i++ ) {
                                kmux = i*nc+jc;
                                kdmx = jp*ns+i+startpt;
                                pDemuxArray[kdmx] = pDataArray[kmux];
                                }
                        pTMPoffset[jp] = 0;
                        jp++;
                        } // end gps processing


                // Special processing for GPS Counter channel (do not plot)

                else if (Ini.pScanChType[jscan] == CH_TYPE_GPSCOUNTER) {

                        } // end GPS counter processing



                // Special processing for Digital channel - split into
                // multiple parts for plotting
                
                else { // if (Ini.pScanChType[jscan] == CH_TYPE_DIGITAL) {

                        for ( k = 0 ; k < nd ; k++ ) {
                                for ( i = 0 ; i < na ; i++ ) {
                                        kmux = i*nc+jc;
                                        kdmx = jp*ns+i+startpt;
                                        pt = pDataArray[kmux];
                                        pDemuxArray[kdmx] = ((pt >> k) & 0x01);
                                        }
                                pTMPoffset[jp] = 0;
                                jp++;
                                }
                        
                        } // end digital processing
                
                } // end for jc



        // Use the offset computed from the first AD buffer for
        // all parts of the plot buffer
                
        if (ResetDC == TRUE) {

                // If AutoDC is TRUE, then always do ResetDC
                
                if (Ini.PltAutoDC == FALSE)
                        ResetDC = FALSE;
                        
                for ( jp = 0 ; jp < np ; jp++ )
                        pDCoffset[jp] = (long)(pTMPoffset[jp] / (double)na);

                }

        
        return( 1 );
}

int CParXchApp::AllocateDataArray( void ) {

        TRACE0("CParXchApp::AllocateDataArray\n");

        // Clean up any previous array and allocate new one
        
        if (pDataArray)
                { free(pDataArray); pDataArray = NULL; }

        pDataArray = (long*) malloc(Nbufferbytes);

        

        if ( pDataArray )
                return( 1 );
        else
                return( 0 );
}


int CParXchApp::TriggerCondition( void ) {

        TRACE0("CParXchApp::TriggerCondition\n");

        // This function is called by the OnTimer function to decide
        // whether or not the currently acquired buffer of data should
        // be written out.
        //

        // Currently all processing is done by the parent class
        
        return( CScopeApp::TriggerCondition( ) );



        
        // If you want to do any XCH specific processing, remove the
        // return from above and do it here.
        //
        // Because this function is a member of the CParXchApp class which
        // is derived from the CScopeApp class you have access to all the
        // member variables of both classes.  In particular, you may wish
        // to use
        //
        //      pDataArray      a pointer to the raw data
        //      Ini.Nchannels   the number of channels
        //      Ini.Nsamples    the number of samples per channel

}

int CParXchApp::TriggerAction( void ) {

        TRACE0("CParXchApp::TriggerAction\n");


        // This function is called by the OnTimer function to perform
        // some action when a trigger has occured.
        //

        // The XCH specific processing, is done here.
        //
        // Because this function is a member of the CParXchApp class which
        // is derived from the CScopeApp class you have access to all the
        // member variables of both classes.  In particular, you may wish
        // to use
        //
        //      pDataArray      a pointer to the raw data
        //      Ini.Nchannels   the number of channels
        //      Ini.Nsamples    the number of samples per channel



        // If this action chosen for true triggering,
        // turn on digital output bit

        // WCT: Consider keeping a global UserIo value and or-ing in signal
        //      so we only change the one signal line

        if ( Ini.TrgType != TRIG_NONE  &&  Ini.TrgActionSignal == TRUE )
                ParXchUserIoWr( AtodHandle, ( 0x01 << Ini.TrgLineToSignal ) );



        // Now perform the default parent class action too.
        
        return( CScopeApp::TriggerAction( ));
}


int CParXchApp::TriggerDone( void ) {

        TRACE0("CParXchApp::TriggerDone\n");

        // Perform PARxCH specific processing here.

        // Turn off digitial output bit if it was turned on
        // and the trigger state is not being latched

        if ( Ini.TrgType != TRIG_NONE     &&
             Ini.TrgActionSignal == TRUE  &&
             Ini.TrgLatchLine == FALSE    )
                TriggerClearLatch( );


        // Now perform the default the parent class trigger cleanup too.
        
        return( CScopeApp::TriggerDone( ));
}

int CParXchApp::TriggerClearLatch( void ) {

        TRACE0("CParXchApp::TriggerClearLatch\n");

        // Perform PARxCH specific processing here.

        // Turn off digitial output bit to clear

        // WCT: Consider keeping a global UserIo value and and-ing in signal
        //      so we only change the one signal line

        ParXchUserIoWr( AtodHandle, (0x00 << Ini.TrgLineToSignal) );


        // Now perform the default the parent class trigger latch clearing too.
        
        return( CScopeApp::TriggerClearLatch( ));
}

int CParXchApp::TriggerThresholdSetRange( int Chan, long *ThMin, long *ThMax ) {

        TRACE0("CParXchApp::TriggerThresholdSetRange\n");

        // This function sets the min and max threshold values.
        // DCoffset was computed elsewhere with data in offset format.

        *ThMin = pDCoffset[Chan] - Ini.TrgThreshold;
        if (*ThMin < 0L)
                *ThMin = 0L;
        
        *ThMax = pDCoffset[Chan] + Ini.TrgThreshold;
        if (*ThMax > 0x00FFFFFFL)
                *ThMax = 0x00FFFFFFL;

        return( 1 );
}
int CParXchApp::TriggerDigitalCompare( int Pt, long TriggerMask ) {

        TRACE0("CParXchApp::TriggerDigitalCompare\n");


        // This function compares the data point pt with the mask and
        // returns 1 if any of the corresponding bits are set.  Since
        // each digital channel is packed into a single bit in a single
        // long, the mask should have a 1 in the bit position for each
        // digital channel that is participating in the trigger check.

        if (pDataArray[Pt] & TriggerMask)
                return( 1 );
        else
                return( 0 );
        
}
int CParXchApp::TriggerThresholdCompare( int Pt, long ThMin, long ThMax ) {

        TRACE0("CParXchApp::TriggerThresholdCompare\n");

        long OffsetPt;

        // This function converts the data point pt to offset format and
        // then compares it with the supplied threshold.  It returns 1
        // if the amplitude of the point exceeds the DCoffset corrected
        // threshold in either direction (min or max).

        OffsetPt = (pDataArray[Pt] & CLEARTOPBYTE) ^ ConvertToOffset;
        
        if ( OffsetPt >= ThMax )
                return( 1 );
        else if ( OffsetPt <= ThMin )
                return( 1 );
        else
                return( 0 );
        
}
int CParXchApp::TriggerDigioCompare( int Line, int ActiveState ) {

        TRACE0("CParXchApp::TriggerDigioCompare\n");

        int SignalState, DigitalIoNibble;

        // This function compares the signal on the specified digital
        // I/O line and returns 1 if the signal matches the specified
        // state.

        ParXchUserIoRd( AtodHandle, &DigitalIoNibble );

        SignalState = (DigitalIoNibble >> Line) & 0x1;

        if ( SignalState == ActiveState )
                return( 1 );
        else
                return( 0 );
}
        
int CParXchApp::HdrFill() {


        // Fill in general header info

        CScopeApp::HdrFill();

        


        // Fill in xch specific header info

        switch ( Ini.OutFileFormat ) {
                
          case FILEFMT_OUT:
                    
            SrStringCopy( OutHdr.Id, HDR_PRODUCT_STRING, 32 );

            if (Inixch.AcqDataCoding == PARXCH_DF_OFFSET)
                    OutHdr.DataCoding = 0;
            else
                    OutHdr.DataCoding = 1;
        
            OutHdr.Gain = (double)(1<<Ini.AcqGainLog); 

            break;
            

          case FILEFMT_DAT:
                    
            SrStringCopy( DatHdr.DatProduct,     HDR_PRODUCT_STRING, SRDAT_DIM_STRING );
            SrStringCopy( DatHdr.DatApplication, "SR SCOPEXCH",      SRDAT_DIM_STRING );

            DatHdr.DatAtodModel = Inixch.HardXchModel;

            if (Inixch.AcqDataCoding == PARXCH_DF_OFFSET)
                    DatHdr.AtodDataCoding = SRDAT_DATACODING_OFFSET;
            else
                    DatHdr.AtodDataCoding = SRDAT_DATACODING_SIGNED;
        
            DatHdr.AtodGain = (double)(1<<Ini.AcqGainLog); 

            break;


          default:
            // No header requested
            break;
            
          } // end switch
        
        return( 1 );
}


int CParXchApp::WriteBytes( CStdioFile *TmpFile ) {

        TRACE0("CParXchApp::WriteBytes\n");


        // Ensure valid file pointer
        
        if (!TmpFile) {
                ErrString = "Invalid TmpFile.";
                LogMessage( MSGLEVEL_ERR, MSGDLG_SHOW, ERROR_MAYBE_FATAL, NULL );
                return( 0 );
                }


        // Write out XCH specific data bytes in requested format.
        
        if (Ini.OutFileFormat == FILEFMT_OUT) {
                
                TmpFile->Write( pDataArray, Nbufferbytes );
                }

        
        else if (Ini.OutFileFormat == FILEFMT_DAT) {

                if ( !SrDatRecordWrite( TmpFile->m_pStream,
                                        SRDAT_TAGID_ANALOG24,
                                        pDataArray,
                                        Nbufferbytes
                                       )  ) {
                        ErrString = "Could not write .DAT analog data.";
                        LogMessage( MSGLEVEL_ERR, MSGDLG_SHOW, ERROR_MAYBE_FATAL, NULL );
                        return( 0 );
                        }
                }
        
        return( 1 );
}




//***********************************************************************
//
// CLASS CParXchWin
//
// This class, derived from CScopeWindow, contains the PARxCH specific
// main window portion of the Scope data acquisition program.  It primarily
// controls dialogs specific to the PARxCH while its parent class handles
// the bulk of the graphics drawing, menu operations, and main window
// functions.
//


BEGIN_MESSAGE_MAP( CParXchWin, CScopeWindow )
        //{{AFX_MSG_MAP(CParXchWin)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()



CParXchWin::CParXchWin()
        :CScopeWindow() {  // parent constructor

        TRACE0("CParXchWin::CParXchWin\n");
}

CParXchWin::~CParXchWin() {

        TRACE0("CParXchWin::~CParXchWin\n");
}


void CParXchWin::PostNcDestroy() {

        TRACE0("CParXchWin::PostNcDestroy\n");

        // call grandparent function to delete "this"

        CFrameWnd::PostNcDestroy();
}


void CParXchWin::OnHardwareDialog() {

        TRACE0("CParXchWin::OnHardwareDialog\n");
        
        int oldXchModel;

        CHardwareDlg dlg(this);

        oldXchModel = APP.Inixch.HardXchModel;

        if (dlg.DoModal() == IDOK) {


                // If model has changed, we must recompute number
                // of channels, buffers, plot positions, and etc

                if (oldXchModel != APP.Inixch.HardXchModel) {

                        APP.SetChannelCounts( 0 );
                        pAPP->UpdateActiveChannels();
                        pAPP->UpdatePlotTraceInfo();
                        pAPP->IsAllocated = FALSE;
                        if (!pAPP->AllocateBuffers())
                                return;
                        if (!ComputePlotPositions())
                                return;
                                
                        SetScrollBar( SB_HORZ );
                        
                        pAPP->ResetPaint = TRUE;
                        Invalidate();
                        }

                }
                
        controlkey = FALSE;

}


void CParXchWin::OnChannelDialog() {

        TRACE0("CParXchWin::OnChannelDialog\n");

        int oldNdisplay, oldNchannels, oldDigEnable;
        int oldAllPlotChannels, maxFirst;
                
        CChannelDlg dlg(this);

        oldNdisplay  = pAPP->Ini.PltNdisplay;
        oldNchannels = pAPP->Ini.Nchannels;
        oldDigEnable = pAPP->Ini.DigitalEnable;

        if (dlg.DoModal() == IDOK) {

                if (oldDigEnable != pAPP->Ini.DigitalEnable) {

                        pAPP->GpsMarkIndex = pAPP->MaxAnalogChannel;
                        oldAllPlotChannels = pAPP->MaxAnaPlotChannel;
                        if (pAPP->Ini.GpsEnable) 
                                oldAllPlotChannels += pAPP->MaxGpsPlotChannel;

                        
                        // Digital channels have been turned on, 
                        // so there are now more channels
                        
                        if (pAPP->Ini.DigitalEnable) {
                                pAPP->GpsMarkIndex += pAPP->MaxDigitalChannel;
                                if (pAPP->Ini.PltNdisplay == oldAllPlotChannels)
                                        pAPP->Ini.PltNdisplay += pAPP->MaxDigPlotChannel;
                                
                                // This calls ComputeBufferSizes and re-allocates
                                // the data and demux arrays if necessary
                                if (!pAPP->AllocateBuffers())
                                        return;
                                }

                        // Digital channels have been turned off,
                        // so there are now fewer channels.
                        
                        else { // if (!pAPP->Ini.DigitalEnable) {
                                if (!pAPP->ComputeBufferSizes())
                                        return;
                                }

                        }
                        
                
                // Update first channel if needed

                maxFirst = pAPP->NchannelsPlot - pAPP->Ini.PltNdisplay;
                if (pAPP->Ini.PltFirstTrace > maxFirst)
                        pAPP->Ini.PltFirstTrace = maxFirst;

                pAPP->UpdatePlotTraceInfo();
                 
                CClientDC dc(this);
                ComputeMaxTitleLength( &dc );
                if (!ComputePlotPositions())
                        return;
                
                SetScrollBar( SB_BOTH );
                
                pAPP->ResetPaint = TRUE;
                Invalidate();

        
                }
        controlkey = FALSE;
}

void CParXchWin::OnAcquireDialog() {

        TRACE0("CParXchWin::OnAcquireDialog\n");

        CAcquireDlg dlg(this);

        if (dlg.DoModal() == IDOK) {

                }
        
                
        controlkey = FALSE;
}







//***********************************************************************
//
// CLASS CHardwareDlg
//
// This class, derived from CScopeDialog, presents and controls the PARxCH
// specific Hardware dialog.  This dialog allows the user to specify the 
// parallel port mode and driver name.
//


BEGIN_MESSAGE_MAP(CHardwareDlg, CScopeDialog)
        //{{AFX_MSG_MAP(CHardwareDlg)
        ON_MESSAGE(WM_EDITLOSTFOCUS,           OnEditLostFocus)
        ON_BN_CLICKED(IDC_BTN_W_HELP,          OnHelp)
        ON_BN_CLICKED(IDC_BTN_W_OK,            OnOK)
        ON_BN_CLICKED(IDC_BTN_W_CANCEL,        OnCancel)
        ON_CBN_SELCHANGE(IDC_CBO_W_DRIVERNAME, OnSelchangeName)
        ON_CBN_KILLFOCUS(IDC_CBO_W_DRIVERNAME, OnKillfocusName)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()



CHardwareDlg::CHardwareDlg(CWnd* pParent /*=NULL*/)
            : CScopeDialog(CHardwareDlg::IDD, pParent) {

        TRACE0("CHardwareDlg::CHardwareDlg\n");

        //{{AFX_DATA_INIT(CHardwareDlg)
                // NOTE: the ClassWizard will add member initialization here
        //}}AFX_DATA_INIT

        m_Index = 0;
        m_Name  = PARXCH_0;
}


void CHardwareDlg::DoDataExchange(CDataExchange* pDX) {

        TRACE0("CHardwareDlg::DoDataExchange\n");

        CScopeDialog::DoDataExchange(pDX);
        //{{AFX_DATA_MAP(CHardwareDlg)
        DDX_Control(pDX, IDC_CBO_W_DRIVERNAME, m_CBO_DriverName);
        //}}AFX_DATA_MAP
}


BOOL CHardwareDlg::OnInitDialog() {

        TRACE0("CHardwareDlg::OnInitDialog\n");

        int ibtn;

        
        CScopeDialog::OnInitDialog( IDC_BTN_W_OK );
        


        // Init Model Buttons
        
        if (APP.Inixch.HardXchModel == PARXCH_MODEL_PAR1CH)
                
                ibtn = IDC_OPT_W_PAR1CH;
        
        else if (APP.Inixch.HardXchModel == PARXCH_MODEL_PAR4CH)
                
                ibtn = IDC_OPT_W_PAR4CH;
        
        else if (APP.Inixch.HardXchModel == PARXCH_MODEL_PAR8CH)
                
                ibtn = IDC_OPT_W_PAR8CH;
        
        else // (APP.Inixch.HardXchModel == PARXCH_MODEL_UNKNOWN)
                
                ibtn = IDC_OPT_W_PAR4CH;


        CheckRadioButton( IDC_OPT_W_PAR1CH, IDC_OPT_W_PAR8CH, ibtn );



                
        // Init Port Mode Radio Buttons
        
        if (APP.Inixch.HardPortMode == PARXCH_PORT_MODE_BPP)
                
                ibtn = IDC_OPT_W_MODEBPP;
        
        else if (APP.Inixch.HardPortMode == PARXCH_PORT_MODE_EPP)
                
                ibtn = IDC_OPT_W_MODEEPP;
        
        else if (APP.Inixch.HardPortMode == PARXCH_PORT_MODE_ECP_BPP)
                
                ibtn = IDC_OPT_W_MODEECPB;
        
        else // (APP.Inixch.HardPortMode == PARXCH_PORT_MODE_ECP_EPP)
                
                ibtn = IDC_OPT_W_MODEECPE;


        CheckRadioButton( IDC_OPT_W_MODEBPP, IDC_OPT_W_MODEECPE, ibtn );



                
        // Init Driver Name Combo Box

        m_Name = APP.Inixch.HardDriverName;

        SetComboName();




                
        // Disable OK button if not in interactive mode

        if ( !pAPP->Ini.DoInteractive ) {
                CWnd *pWnd = GetDlgItem( IDC_BTN_W_OK );
                if (pWnd)
                        pWnd->EnableWindow( FALSE );
                }

        
        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CHardwareDlg::OnOK() {

        TRACE0("CHardwareDlg::OnOK\n");

        int ibtn;


        if (!CheckFocus())
                return;



        CScopeDialog::OnOK();


        
        // Update Driver Name

        APP.Inixch.HardDriverName = m_Name;

        
        // Update Port Mode

        ibtn = GetCheckedRadioButton( IDC_OPT_W_MODEBPP, IDC_OPT_W_MODEECPE );

        if (ibtn == IDC_OPT_W_MODEBPP)
                
                APP.Inixch.HardPortMode = PARXCH_PORT_MODE_BPP;

        else if (ibtn == IDC_OPT_W_MODEEPP)
                
                APP.Inixch.HardPortMode = PARXCH_PORT_MODE_EPP;
        
        else if (ibtn == IDC_OPT_W_MODEECPB)
                
                APP.Inixch.HardPortMode = PARXCH_PORT_MODE_ECP_BPP;
        
        else // (ibtn == IDC_OPT_W_MODEECPE)
                
                APP.Inixch.HardPortMode = PARXCH_PORT_MODE_ECP_EPP;


        // Update Model Id

        ibtn = GetCheckedRadioButton( IDC_OPT_W_PAR1CH, IDC_OPT_W_PAR8CH );

        if (ibtn == IDC_OPT_W_PAR1CH)
                
                APP.Inixch.HardXchModel = PARXCH_MODEL_PAR1CH;

        else if (ibtn == IDC_OPT_W_PAR4CH)
                
                APP.Inixch.HardXchModel = PARXCH_MODEL_PAR4CH;
        
        else // (ibtn == IDC_OPT_W_PAR8CH)
                
                APP.Inixch.HardXchModel = PARXCH_MODEL_PAR8CH;
        
}

void CHardwareDlg::OnCancel() {

        TRACE0("CHardwareDlg::OnCancel\n");

        CScopeDialog::OnCancel();
}

void CHardwareDlg::OnHelp() {

        TRACE0("CHardwareDlg::OnHelp\n");

        
        if (!CheckFocus())
                return;

        
        pAPP->ShowHelp(IDR_HELP_HARD_TITLE, IDR_HELP_HARD_TEXT, this);
}

LONG CHardwareDlg::OnEditLostFocus( UINT nID, LONG xxx ) {

        TRACE1("CHardwareDlg::OnEditLostFocus (%u)\n", nID);

        CString tempstr;
        int     temp;
        BOOL    good, good1, good2;
        

        if (!DoValidate) {
                return( 1 );
                }


        // Don't allow validation when already doing validation

        DoValidate = FALSE;


                
        // Validate various edit controls
        
        switch ( nID ) {
                             // Just fall through, with no break since
                             // AFX_IDC_FONTNAMES somehow has focus
                case 1001:   // after enter from typing in a DropDown
                             // CBO.  So validate them both to be safe.
                                
                        good1 = TRUE;

                        // break;
                        
                case IDC_CBO_W_DRIVERNAME:

                        GetDlgItemText( IDC_CBO_W_DRIVERNAME, pAPP->szTemp, MAXSTR );
                        tempstr = pAPP->szTemp;

                        if ( sscanf((LPCTSTR)tempstr,"SrParXch%3x",&temp) == 1 ) {
                                good2 = TRUE;
                                m_Name = tempstr;
                                }
                        else
                                good2 = FALSE;

                        SetComboName();
                        
                        good = (good1 && good2);
                        
                        break;



                default:
                        good = TRUE;
                        break;
                                                
                        
                }


        // Re-allow validation

        DoValidate = TRUE;


        if (good)
                return( 1 );
        else
                return( 0 );
}


void CHardwareDlg::OnSelchangeName() {

        TRACE0("CHardwareDlg::OnSelchangeName\n");

        
        m_Index = m_CBO_DriverName.GetCurSel();

        if ( (m_Index == CB_ERR) || (m_Index == -1) ) {

                GetDlgItemText( IDC_CBO_W_DRIVERNAME, pAPP->szTemp, MAXSTR );
                m_Name = pAPP->szTemp;
                }

        else {
                m_CBO_DriverName.GetLBText( m_Index, pAPP->szTemp );
                m_Name = pAPP->szTemp;
                }
}

void CHardwareDlg::OnKillfocusName() {

        TRACE0("CHardwareDlg::OnKillfocusName\n");

        PostMessage( WM_EDITLOSTFOCUS, IDC_CBO_W_DRIVERNAME, 0 );
}

void CHardwareDlg::SetComboName() {

        TRACE1("CHardwareDlg::SetComboName to %s\n", (LPCTSTR)m_Name);


        // Find string in list
                
        m_Index = m_CBO_DriverName.FindString( -1, m_Name );

        

        if ( (m_Index == CB_ERR) || (m_Index == -1) )
         
                SetDlgItemText( IDC_CBO_W_DRIVERNAME, m_Name );
        else
                m_CBO_DriverName.SetCurSel( m_Index );
}


//***********************************************************************
//
// CLASS CChannelDlg
//
// This class, derived from CScopeDialog, presents and controls the PARxCH
// specific Channel dialog.  This dialog allows the user to specify
// the titles and activation status of the channels.
//


BEGIN_MESSAGE_MAP(CChannelDlg, CScopeDialog)
        //{{AFX_MSG_MAP(CChannelDlg)
        ON_MESSAGE(WM_EDITLOSTFOCUS,     OnEditLostFocus)
        ON_BN_CLICKED(IDC_BTN_C_OK,      OnOK)
        ON_BN_CLICKED(IDC_BTN_C_CANCEL,  OnCancel)
        ON_BN_CLICKED(IDC_BTN_C_HELP,    OnHelp)
        ON_BN_CLICKED(IDC_BTN_C_BACK,    OnBtnBack)
        ON_BN_CLICKED(IDC_BTN_C_NEXT,    OnBtnNext)
        ON_BN_CLICKED(IDC_CHK_C_DIGCHAN, OnChkDigitalChannels)
        ON_EN_CHANGE(IDC_TXT_C_CURR,     OnChangeTxtCurr)
        ON_EN_KILLFOCUS(IDC_TXT_C_CURR,  OnKillfocusTxtCurr)
        ON_EN_KILLFOCUS(IDC_TXT_C_TITLE, OnKillfocusTxtTitle)
        ON_LBN_SELCHANGE(IDC_LST_C_SCAN, OnSelchangeListScan)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()


CChannelDlg::CChannelDlg(CWnd* pParent /*=NULL*/)
           : CScopeDialog(CChannelDlg::IDD, pParent) {

        TRACE0("CChannelDlg::CChannelDlg\n");

        //{{AFX_DATA_INIT(CChannelDlg)
        m_CurrChan    = 0;
        m_ShowDigital = TRUE;
	m_ShowGps     = FALSE;
        //}}AFX_DATA_INIT


        pTempActive = NULL;
        pTempTitle = NULL;

}

CChannelDlg::~CChannelDlg() {

        TRACE0("CChannelDlg::~CChannelDlg\n");

        // Clean up dialog title buffers
        
        if (pTempActive)
                { free(pTempActive); pTempActive = NULL; }
        
        if (pTempTitle)
                { free(pTempTitle); pTempTitle = NULL; }

}

void CChannelDlg::DoDataExchange(CDataExchange* pDX) {

        TRACE0("CChannelDlg::DoDataExchange\n");

        CScopeDialog::DoDataExchange(pDX);
        //{{AFX_DATA_MAP(CChannelDlg)
        DDX_Control(pDX, IDC_LST_C_SCAN, m_LSTscan);
        DDX_Text(pDX, IDC_TXT_C_CURR, m_CurrChan);
        //}}AFX_DATA_MAP
}



BOOL CChannelDlg::OnInitDialog() {

        TRACE0("CChannelDlg::OnInitDialog\n");
        
        int i, j, k;
        CButton *pChk;



        // Allocate dialog buffers

        pTempActive = (int*) malloc( pAPP->MaxTotalPlotChannel * sizeof(int) );
        pTempTitle  = (char*)malloc( pAPP->MaxTotalPlotChannel * MAXTITLE *sizeof(char));


        if (!pTempActive || !pTempTitle) {
                pAPP->ErrString = "Could not allocate dialog buffers.";
                pAPP->LogMessage( MSGLEVEL_ERR, MSGDLG_SHOW, ERROR_FATAL, NULL );
                EndDialog( -1 );
                return( FALSE );
                }




        
        // Initialize all member variables from application globals


        CScopeDialog::OnInitDialog( IDC_BTN_C_OK );


        m_ShowDigital = pAPP->Ini.DigitalEnable;
        m_ShowGps     = pAPP->Ini.GpsEnable;


        // The pScan arrays are indexed by i upto MaxTotalChannel, but
        // the pTemp arrays are indexed by j upto MaxTotalPlotChannel.

        j = 0;
        for ( i = 0 ; i < pAPP->MaxTotalChannel ; i++ ) {

                if ( pAPP->Ini.pScanChType[i] == CH_TYPE_DIGITAL) {
                        for ( k = 0 ; k < pAPP->MaxDigPlotChannel ; k++ ) {
                                pTempActive[j] = pAPP->Ini.pScanActive[i];
                                SrStringCopy(
                                             &pTempTitle[j*MAXTITLE],
                                             &(pAPP->Ini.pDigTitle[k*MAXTITLE]),
                                             MAXTITLE
                                            );
                                j++;
                                } // end for k
                        } // end CH_TYPE_DIGITAL

                else if ( pAPP->Ini.pScanChType[i] == CH_TYPE_GPSCOUNTER) {
                        ; // Do not plot or display on list
                        } // end CH_TYPE_GPSCOUNTER

                else {
                        pTempActive[j] = pAPP->Ini.pScanActive[i];
                        SrStringCopy(
                                     &pTempTitle[j*MAXTITLE],
                                     &(pAPP->Ini.pScanTitle[i*MAXTITLE]),
                                     MAXTITLE
                                    );
                        j++;
                        }

                } // end for i

        UpdateScanList();
        UpdateChannelData(0);


        // Set state of show digital check button

        pChk = (CButton*)GetDlgItem( IDC_CHK_C_DIGCHAN );
        if (pChk) {
                pChk->SetCheck( m_ShowDigital );
                if (pAPP->MaxDigitalChannel <= 0) {
                        pChk->EnableWindow( FALSE );
                        GetDlgItem( IDC_FRA_C_SETDIGINPUT )->EnableWindow( FALSE );
                        }
                }

        // Disable OK button if not in interactive mode

        if ( !pAPP->Ini.DoInteractive ) {
                CWnd *pWnd = GetDlgItem( IDC_BTN_C_OK );
                if (pWnd)
                        pWnd->EnableWindow( FALSE );
                }


        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CChannelDlg::OnHelp() {

        TRACE0("CChannelDlg::OnHelp\n");
        
        if (!CheckFocus())
                return;

                        
        pAPP->ShowHelp(IDR_HELP_CHAN_TITLE, IDR_HELP_CHAN_TEXT, this);
}

void CChannelDlg::OnCancel() {
        
        TRACE0("CChannelDlg::OnCancel\n");
        
        CScopeDialog::OnCancel();
}

void CChannelDlg::OnOK() {

        TRACE0("CChannelDlg::OnOK\n");
        
        int ichan, i, j, k;
        CButton* pChk;


        if (!CheckFocus())
                return;

        
        CScopeDialog::OnOK();


        // Retrieve state of check button

        pChk = (CButton*)GetDlgItem( IDC_CHK_C_DIGCHAN );
        m_ShowDigital = pChk->GetCheck();


        // Update globals

        pAPP->Ini.DigitalEnable = m_ShowDigital;

        // The pScan arrays are indexed by i upto MaxTotalChannel, but
        // the pTemp arrays are indexed by j upto MaxTotalPlotChannel.

        ichan = 0;
        j     = 0;

        for ( i = 0 ; i < pAPP->MaxTotalChannel ; i++ ) {

                if (j < pAPP->MaxTotalPlotChannel) // GPS_COUNTER not plotted
                        pAPP->Ini.pScanActive[i] = pTempActive[j];

                if (pAPP->Ini.pScanChType[i] == CH_TYPE_DIGITAL) {

                        SrStringCopy( &(pAPP->Ini.pScanTitle[i*MAXTITLE]),
                                      "DGTL", MAXTITLE );

                        for ( k = 0 ; k < pAPP->MaxDigPlotChannel ; k++ ) {
                                SrStringCopy(
                                             &(pAPP->Ini.pDigTitle[k*MAXTITLE]),
                                             &pTempTitle[j*MAXTITLE],
                                             MAXTITLE
                                            );
                                j++;
                                } // end for k

                        } // end if CH_TYPE_DIGITAL

                else if (pAPP->Ini.pScanChType[i] == CH_TYPE_GPSCOUNTER) {
                        // Do not plot or show in list
                        } // end if CH_TYPE_GPSCOUNTER

                else {
                        SrStringCopy(
                                     &(pAPP->Ini.pScanTitle[i*MAXTITLE]),
                                     &pTempTitle[j*MAXTITLE],
                                     MAXTITLE
                                    );
                        j++;
                        }
                
                if (pAPP->Ini.pScanActive[i] == CHECKED) {
                        pAPP->Ini.pScanOrder[ichan] = i;
                        ichan++;
                        }
                }

        pAPP->Ini.Nchannels = ichan;
        pAPP->NchannelsPlot = pAPP->Ini.Nchannels;
        if (pAPP->Ini.DigitalEnable)
                pAPP->NchannelsPlot += (pAPP->MaxDigPlotChannel - pAPP->MaxDigitalChannel);
        if (pAPP->Ini.GpsEnable)
                pAPP->NchannelsPlot += (pAPP->MaxGpsPlotChannel - pAPP->MaxGpsChannel);
        if (pAPP->Ini.CounterEnable)
                pAPP->NchannelsPlot += (pAPP->MaxCntPlotChannel - pAPP->MaxCounterChannel);
        if (pAPP->Ini.PltNdisplay > pAPP->NchannelsPlot)
                pAPP->Ini.PltNdisplay = pAPP->NchannelsPlot;

}

LONG CChannelDlg::OnEditLostFocus( UINT nID, LONG xxx ) {

        TRACE1("CChannelDlg::OnEditLostFocus (%u)\n", nID);

        int  ipos, icurr;
        BOOL good;
        

        if (!DoValidate) {
                return( 1 );
                }


        // Don't allow validation when already doing validation

        DoValidate = FALSE;
        
        // Validate various edit controls
        
        switch ( nID ) {

                case IDC_TXT_C_CURR:

                        icurr = m_CurrChan;

                        good = ValidateAsInt(IDC_TXT_C_CURR,
                                                0, 
                                                pAPP->MaxTotalPlotChannel - 1, 
                                                &icurr, 
                                                "Current channel");

                        // Always update channel since icurr
                        // will be new or reset from bad

                        UpdateChannelData(icurr);
                        
                        break;




                case IDC_TXT_C_TITLE:
                        
                        GetDlgItemText(
                                        IDC_TXT_C_TITLE,
                                        &pTempTitle[m_CurrChan*MAXTITLE],
                                        MAXTITLE
                                       ); 
        
                        ipos = FindPos();
                        m_LSTscan.DeleteString(ipos);
                        AddItem( ipos, m_CurrChan );
                        ipos = FindPos();
                        m_LSTscan.SetCurSel( ipos );

                        good = TRUE;
                        
                        break;


                        

                default:
                        good = TRUE;
                        break;
                                                
                        
                }


        // Re-allow validation

        DoValidate = TRUE;


        TRACE0("CChannelDlg::OnEditLostFocus - END\n");

        if (good)
                return( 1 );
        else
                return( 0 );
}

void CChannelDlg::OnBtnBack() {

        TRACE0("CChannelDlg::OnBtnBack\n");
        

        int icurr, maxchan;
        
        if (!CheckFocus())
                return;


        maxchan = m_LSTscan.GetCount() - 1;
        icurr   = m_CurrChan - 1;
        if (icurr < 0)
                icurr = maxchan;
        
        UpdateChannelData( icurr );
        
}

void CChannelDlg::OnBtnNext() {
        
        TRACE0("CChannelDlg::OnBtnNext\n");
        
        int icurr, maxchan;

        if (!CheckFocus())
                return;

                
        maxchan = m_LSTscan.GetCount() - 1;
        icurr = m_CurrChan + 1;
        if (icurr > maxchan)
                icurr = 0;
        
        UpdateChannelData( icurr );
        
}

void CChannelDlg::OnChkDigitalChannels() {
        TRACE0("CChannelDlg::OnChkDigitalChannels\n");

        int i, j, k, icurr, DigitalState, maxchan;

        if (!CheckFocus())
                return;

        // Toggle state

        m_ShowDigital = !m_ShowDigital;

        
        // Update for state of digital channels

        if (m_ShowDigital)
                DigitalState = CHECKED;
        else
                DigitalState = UNCHECKED;

        j = 0;
        for ( i = 0 ; i < pAPP->MaxTotalChannel ; i++ ) {

                if ( pAPP->Ini.pScanChType[i] == CH_TYPE_DIGITAL) {

                        for ( k = 0 ; k < pAPP->MaxDigPlotChannel ; k++ ) {
                                pTempActive[j] = DigitalState;
                                j++;
                                } // end for k

                        } // end if CH_TYPE_DIGITAL

                else if ( pAPP->Ini.pScanChType[i] == CH_TYPE_GPSCOUNTER) {
                        ; // do not plot or show in list
                        } // end if CH_TYPE_GPSCOUNTER

                else // !CH_TYPE_DIGITAL
                        j++;

                } // end for i

        UpdateScanList();

        
        // Ensure current channel is still valid

        maxchan = m_LSTscan.GetCount() - 1;
        icurr   = m_CurrChan;
        if (icurr > maxchan)
                icurr = maxchan;
        
        UpdateChannelData( icurr );
        
}

void CChannelDlg::OnChangeTxtCurr() {

        TRACE0("CChannelDlg::OnChangeTxtCurr\n");
        
        CWnd *pWnd = GetFocus();


        if (pWnd && (pWnd->GetDlgCtrlID() == IDC_TXT_C_CURR) ) 
                GetDlgItem( IDC_LAB_C_CURR )->EnableWindow( FALSE );
}


void CChannelDlg::OnKillfocusTxtCurr() {

        TRACE0("CChannelDlg::OnKillfocusTxtCurr\n");
        
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_C_CURR, 0 );
}

void CChannelDlg::OnKillfocusTxtTitle() {

        TRACE0("CChannelDlg::OnKillfocusTxtTitle\n");
        
        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_C_TITLE, 0 );
}

void CChannelDlg::OnSelchangeListScan() {

        TRACE0("CChannelDlg::OnSelchangeListScan\n");
        
        int icurr, ipos;
        
        ipos = m_LSTscan.GetCurSel();
                icurr = m_LSTscan.GetItemData(ipos);
        if (icurr != m_CurrChan)
            UpdateChannelData(icurr);
        
        TRACE0("CChannelDlg::OnSelchangeListScan - END\n");
}

void CChannelDlg::UpdateChannelData( int icurr ) {
        
        TRACE1("CChannelDlg::UpdateChannelData (%d)\n", icurr);
        
        int ipos;
    
        m_CurrChan = icurr;
        
        SetDlgItemInt(  IDC_TXT_C_CURR,  m_CurrChan );
        SetDlgItemText( IDC_TXT_C_TITLE, &pTempTitle[m_CurrChan*MAXTITLE] );

        ipos = FindPos();
        m_LSTscan.SetCurSel( ipos );

        GetDlgItem( IDC_LAB_C_CURR )->EnableWindow( TRUE );

}

void CChannelDlg::UpdateScanList( void ) {
        
        TRACE0("CChannelDlg::UpdateScanList\n");

        int j, ichan;

        m_LSTscan.ResetContent();

        ichan = 0;
        for ( j = 0 ; j < pAPP->MaxTotalPlotChannel ; j++ ) {
                if (pTempActive[j] == CHECKED) {
                        AddItem( ichan, j );
                        ichan++;
                        }
                }
}

void CChannelDlg::AddItem( int ichan, int ikey ) {
        
        TRACE0("CChannelDlg::AddItem\n");
        
        CString tmpString;
        int     ipos;

        tmpString.Format("Ch%02d - %s", ichan, &pTempTitle[ikey*MAXTITLE] );
        ipos = m_LSTscan.AddString(tmpString);
        m_LSTscan.SetItemData(ipos,ikey);
}

int CChannelDlg::FindPos() {
        
        TRACE0("CChannelDlg::FindPos\n");
        
        int ipos, ncount;


        ncount = m_LSTscan.GetCount();

        for ( ipos = 0 ; ipos < ncount ; ipos++ ) {
                if ((int)m_LSTscan.GetItemData(ipos) == m_CurrChan)
                        return( ipos );
                }
        
        return( -1 );
}





//***********************************************************************
//
// CLASS CAcquireDlg
//
// This class, derived from CScopeDialog, presents and controls the PARxCH
// specific Acquisition dialog.  This dialog allows the user to specify
// the sampling rate and the gain.
//

BEGIN_MESSAGE_MAP(CAcquireDlg, CScopeDialog)
        //{{AFX_MSG_MAP(CAcquireDlg)
        ON_MESSAGE(WM_EDITLOSTFOCUS,       OnEditLostFocus)
        ON_BN_CLICKED(IDC_BTN_A_OK,        OnOK)
        ON_BN_CLICKED(IDC_BTN_A_CANCEL,    OnCancel)
        ON_BN_CLICKED(IDC_BTN_A_HELP,      OnHelp)
        ON_EN_KILLFOCUS(IDC_TXT_A_SPS,     OnKillfocusSps)
        ON_BN_CLICKED(IDC_CHK_A_TOGGLELED, OnChkToggleLed)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()



CAcquireDlg::CAcquireDlg(CWnd* pParent /* = NULL */ )
        : CScopeDialog(CAcquireDlg::IDD, pParent)      {

        TRACE0("CAcquireDlg::CAcquireDlg\n");

        m_SPS       = (float)PARXCH_SPS_DEFAULT;
        m_ToggleLed = CHECKED;

}

BOOL CAcquireDlg::OnInitDialog() {

        TRACE0("CAcquireDlg::OnInitDialog\n");

        int ibtn;
        
        CScopeDialog::OnInitDialog( IDC_BTN_A_OK );


        // Set SPS

        m_SPS = pAPP->Ini.AcqSPS;
        
        SetDlgItemFloat( IDC_TXT_A_SPS, m_SPS );


        // Set Gain
        
        CComboBox *pCbo = (CComboBox*)GetDlgItem( IDC_CBO_A_GAIN );
        pCbo->SetCurSel(pAPP->Ini.AcqGainLog);



        // Set Data Format (Data Coding)
        
        if (APP.Inixch.AcqDataCoding == PARXCH_DF_SIGNED)
                
                ibtn = IDC_OPT_A_DCSIGNED;
        
        else // (APP.Inixch.AcqDataCoding == PARXCH_DF_OFFSET)
                
                ibtn = IDC_OPT_A_DCOFFSET;


        CheckRadioButton( IDC_OPT_A_DCSIGNED, IDC_OPT_A_DCOFFSET, ibtn );



        // Set Toggle LED

        if (APP.Inixch.AcqToggleLed == TRUE)
                m_ToggleLed = CHECKED;
        else
                m_ToggleLed = UNCHECKED;
        
        CButton *pChk = (CButton*)GetDlgItem( IDC_CHK_A_TOGGLELED );
        pChk->SetCheck(m_ToggleLed);



        // Disable OK button if not in interactive mode

        if ( !pAPP->Ini.DoInteractive ) {
                CWnd *pWnd = GetDlgItem( IDC_BTN_A_OK );
                if (pWnd)
                        pWnd->EnableWindow( FALSE );
                }

        
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CAcquireDlg::OnOK() {

        TRACE0("CAcquireDlg::OnOK\n");

        int ibtn;

        if (!CheckFocus())
                return;


        
        // Don't close if SPS changed due to error
        
        if (!VerifySps())
                return;

                
        CScopeDialog::OnOK();


        // Get SPS
                                        
        pAPP->Ini.AcqSPS = m_SPS;


        // Get Gain
        
        CComboBox *pCbo = (CComboBox*)GetDlgItem( IDC_CBO_A_GAIN );
        pAPP->Ini.AcqGainLog = pCbo->GetCurSel();
        
        APP.Inixch.AcqTurboLog = PARXCH_GAINTURBO_MAX - pAPP->Ini.AcqGainLog;
        APP.CorrectSps(
                       &pAPP->Ini.AcqSPS,
                       &APP.Inixch.AcqDecimation,
                       &APP.Inixch.AcqExtraDecimation
                      );


        // Get Data Format (Data Coding) 
        
        ibtn = GetCheckedRadioButton( IDC_OPT_A_DCSIGNED, IDC_OPT_A_DCOFFSET );

        if (ibtn == IDC_OPT_A_DCSIGNED)
                
                APP.Inixch.AcqDataCoding = PARXCH_DF_SIGNED;
        
        else // (ibtn == IDC_OPT_A_DCOFFSET)
                
                APP.Inixch.AcqDataCoding = PARXCH_DF_OFFSET;

        APP.SetConvertToOffset( );



        // Get Toggle LED
        
        CButton *pChk = (CButton*)GetDlgItem( IDC_CHK_A_TOGGLELED );
        m_ToggleLed = pChk->GetCheck();
        if (m_ToggleLed == CHECKED)
                APP.Inixch.AcqToggleLed = TRUE;
        else
                APP.Inixch.AcqToggleLed = FALSE;
        
}


void CAcquireDlg::OnCancel() {

        TRACE0("CAcquireDlg::OnCancel\n");


        CScopeDialog::OnCancel();
}

void CAcquireDlg::OnHelp() {

        TRACE0("CAcquireDlg::OnHelp\n");

        if (!CheckFocus())
                return;

        pAPP->ShowHelp(IDR_HELP_ACQ_TITLE, IDR_HELP_ACQ_TEXT, this);
}

LONG CAcquireDlg::OnEditLostFocus( UINT nID, LONG xxx ) {

        TRACE1("CAcquireDlg::OnEditLostFocus (%u)\n", nID);

        BOOL   good;
        int    decim, extradecim;
        

        if (!DoValidate) {
                return( 1 );
                }


        // Don't allow validation when already doing validation

        DoValidate = FALSE;
        
        // Validate various edit controls
        
        switch ( nID ) {

                case IDC_TXT_A_SPS:

                        good = ValidateAsFloat(
                                                IDC_TXT_A_SPS,
                                                pAPP->MinSPS,
                                                pAPP->MaxSPS,
                                                &m_SPS,
                                                "Samples per second"
                                               );

                        APP.CorrectSps( &m_SPS, &decim, &extradecim );
        
                        SetDlgItemFloat( IDC_TXT_A_SPS, m_SPS );


                        // Check that chosen SPS won't overflow GPS buffer

                        if (!pAPP->GpsTimingCheckBufferSize(
                                                        pAPP->Ini.GpsEnable,
                                                        pAPP->Ini.Nsamples,
                                                        m_SPS ) ) {

                                pAPP->ErrString.Format("Sampling rate %f is too low, a GPS buffer would overflow.", m_SPS);                                
                                AfxMessageBox( pAPP->ErrString );
                                m_SPS = pAPP->Ini.AcqSPS;
                                SetDlgItemFloat( IDC_TXT_A_SPS, m_SPS );
                                good = FALSE;
                                }

                        break;
                        

                default:
                        good = TRUE;
                        break;
                                                
                        
                }


        // Re-allow validation

        DoValidate = TRUE;


        if (good)
                return( 1 );
        else
                return( 0 );
}

void CAcquireDlg::OnKillfocusSps() {

        TRACE0("CAcquireDlg::OnKillfocusSps\n");

        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_A_SPS, 0 );
}

void CAcquireDlg::OnChkToggleLed(  ) {

        m_ToggleLed = !m_ToggleLed;
}

int CAcquireDlg::VerifySps() {

        TRACE0("CAcquireDlg::VerifySps\n");
        
        float spsvalue;

        spsvalue = GetDlgItemFloat( IDC_TXT_A_SPS );
        
        // Check it is not less than the minimum
        
        if (spsvalue < pAPP->MinSPS) {
                SetDlgItemFloat( IDC_TXT_A_SPS, pAPP->MinSPS );
                ::MessageBeep(0);
                return( 0 );
                }

        // Check it is not greater than the maximum
        
        else if (spsvalue > pAPP->MaxSPS) {
                SetDlgItemFloat( IDC_TXT_A_SPS, pAPP->MaxSPS );
                ::MessageBeep(0);
                return( 0 );
                }

        // Check that it is not so small a GPS buffer would overflow

        else if (!pAPP->GpsTimingCheckBufferSize( pAPP->Ini.GpsEnable,
                                                  pAPP->Ini.Nsamples,
                                                  spsvalue ) ) {

                SetDlgItemFloat( IDC_TXT_A_SPS, pAPP->Ini.AcqSPS );
                ::MessageBeep(0);
                return( 0 );
                }

        // Then it is OK

        else
                return( 1 );
                
}
