// FILE: xch.h                Copyright (c), Symmetric Research, 2003-2004
//
// Header file for the PARxCH specific portion of the Scope program includes
//
//      CParXchApp      main app    PARxCH specific version of CScopeApp
//      CParXchWin      main window PARxCH specific version of CScopeWindow
//      CHardwareDlg    dialog for  PARxCH hardware options
//      CChannelDlg     dialog for  PARxCH channel options
//      CAcquireDlg     dialog for  PARxCH acquisition options
//
//



// SPECIFIC RESOURCE CONSTANTS

#include "xch.rch"



// SPECIFIC ATOD TYPE DEFINES

extern "C" {
#include "parxch.h"
}



// GENERAL APPLICATION CONSTANTS

#include "scope.h"


// PARxCH SPECIFIC INI STRUCTURE

struct INI_DATAXCH {
        
        CString HardDriverName;
        int     HardXchModel;
        int     HardPortMode;

        int     AcqDataCoding;
        int     AcqTurboLog;
        int     AcqDecimation;        // These two are not actually used 
        int     AcqExtraDecimation;   // or accessable from the GUI yet
        int     AcqToggleLed;
        
        };





//***********************************************************************


class CParXchApp : public CScopeApp {
        
public:

        // Data

        struct INI_DATAXCH Inixch;
        struct INI_DATAXCH InixchSave;

        int       Nextpt;     // Current data position counter.
        int       LedValue;
        long      ConvertToOffset;
        long     *pDataArray;
        DEVHANDLE AtodHandle;
        CString   AtodModelName;




        // Methods

        //{{AFX_VIRTUAL(CParXchApp)
        public:
        virtual int  ExitInstance();
        //}}AFX_VIRTUAL

        int ConvertGain( int value );
        int CorrectSps( float *sps,int *decimation,int *extradecimation );
        int SetConvertToOffset( void );
        int SetChannelCounts( int UseMax );

        virtual CScopeWindow* NewScopeWindow();
        virtual int InitAppData( void );
        virtual int IniVerifyData( void );
        virtual int IniSetDefaults( void );
        virtual int IniSaveCurrent( void );
        virtual int IniRestoreCurrent( void );
        virtual int IniIsChanged( void );
        virtual int IniReadKeyword( void );
        virtual int IniWriteKeyword( FILE *OutFp );
        virtual int AllocateDataArray( void );
        virtual int TriggerCondition( void );
        virtual int TriggerAction( void );
        virtual int TriggerDone( void );
        virtual int TriggerClearLatch( void );
        virtual int TriggerThresholdSetRange( int chan, long *thmin, long *thmax );
        virtual int TriggerDigitalCompare( int pt, long mask );
        virtual int TriggerThresholdCompare( int pt, long thmin, long thmax );
        virtual int TriggerDigioCompare( int line, int activestate );
        virtual int HdrFill(  void );
        virtual int WriteBytes( CStdioFile *TmpFile );

        virtual int AtodMeasure( int *ival );
        virtual int AtodInit();
        virtual int AtodStart();
        virtual int AtodStop();
        virtual int AtodAttachGpsTiming();
        virtual int AtodReleaseGpsTiming();
        virtual int AtodCheckReady();
        virtual int AtodCheckOverflow();
        virtual int AtodOverflowRecovery();
        virtual int AtodGetData();
        virtual int AtodDemuxData();

        
        //{{AFX_MSG(CParXchApp)
        //}}AFX_MSG

        DECLARE_MESSAGE_MAP()
};





//***********************************************************************

class CParXchWin : public CScopeWindow {

public:

        // Methods

        CParXchWin();
        ~CParXchWin();


        //{{AFX_VIRTUAL(CParXchWin)
        virtual void PostNcDestroy();
        //}}AFX_VIRTUAL

        
        //{{AFX_MSG(CParXchWin)
        virtual afx_msg void OnHardwareDialog();
        virtual afx_msg void OnChannelDialog();
        virtual afx_msg void OnAcquireDialog();
        //}}AFX_MSG
        
        DECLARE_MESSAGE_MAP()
};





//***********************************************************************

class CHardwareDlg : public CScopeDialog {


public:
        CHardwareDlg(CWnd* pParent = NULL);   // standard constructor

        
        // Dialog Data
        
        //{{AFX_DATA(CHardwareDlg)
        enum { IDD = IDD_HARDWARE };
        CComboBox       m_CBO_DriverName;
        //}}AFX_DATA

        CString         m_Name;
        int             m_Index;

        // ClassWizard generated virtual function overrides
        
        //{{AFX_VIRTUAL(CHardwareDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL

protected:

        void SetComboName(void);
        

        // Generated message map functions
        
        //{{AFX_MSG(CHardwareDlg)
        virtual BOOL OnInitDialog();
        virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
        afx_msg void OnOK();
        afx_msg void OnCancel();
        afx_msg void OnHelp();
        afx_msg void OnSelchangeName();
        afx_msg void OnKillfocusName();
        //}}AFX_MSG
        
        DECLARE_MESSAGE_MAP()
};






//***********************************************************************


class CChannelDlg : public CScopeDialog {
        

public:
        CChannelDlg(CWnd* pParent = NULL);   // standard constructor
        ~CChannelDlg();                      // destructor

        
        // Data
        
        int  *pTempActive;
        char *pTempTitle;
        
        //{{AFX_DATA(CChannelDlg)
        enum { IDD = IDD_CHANNEL };
        CListBox  m_LSTscan;
        int       m_CurrChan;
        BOOL      m_ShowDigital;
	BOOL      m_ShowGps;
        //}}AFX_DATA


        // ClassWizard generated virtual function overrides
        
        //{{AFX_VIRTUAL(CChannelDlg)
        protected:
        virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
        //}}AFX_VIRTUAL


protected:

        void UpdateChannelData( int icurr );
        void UpdateScanList();
        void AddItem( int ichan, int ikey );
        int  FindPos();
        

        // Generated message map functions
        
        //{{AFX_MSG(CChannelDlg)
        virtual BOOL OnInitDialog();
        virtual void OnOK();
        virtual void OnCancel();
        virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
        afx_msg void OnHelp();
        afx_msg void OnBtnBack();
        afx_msg void OnBtnNext();
        afx_msg void OnChkDigitalChannels( );
        afx_msg void OnChangeTxtCurr();
        afx_msg void OnKillfocusTxtCurr();
        afx_msg void OnKillfocusTxtTitle();
        afx_msg void OnSelchangeListScan();
        //}}AFX_MSG
        
        DECLARE_MESSAGE_MAP()
};




//***********************************************************************


class CAcquireDlg : public CScopeDialog {
        
public:
        CAcquireDlg(CWnd* pParent = NULL);

        // Data
        
        //{{AFX_DATA(CAcquireDlg)
        enum { IDD = IDD_ACQUIRE };
        //}}AFX_DATA

        float m_SPS;
        int   m_ToggleLed;
        int   m_DataCoding;


        
        // Methods
        
        int  VerifySps();

        //{{AFX_MSG(CAcquireDlg)
        virtual BOOL OnInitDialog();
        virtual void OnOK();
        virtual void OnCancel();
        virtual void OnHelp();
        virtual LONG OnEditLostFocus( UINT nID, LONG xxx );
        virtual void OnKillfocusSps();
        afx_msg void OnChkToggleLed( );
        //}}AFX_MSG

        DECLARE_MESSAGE_MAP()
};

