// FILE: view.cpp             Copyright (c), Symmetric Research, 2003-2008
//
// Source file for generic portion of the View program includes
//
//      CViewApp
//      CViewWindow
//      CViewDialog    should be virtual (ie must declare derived class)
//      CDisplayDlg
//      CSeqNameDlg
//      CGotoPtDlg
//      CContentsDlg
//      CTopicDlg
//
//

#include <direct.h>     // needed for _getcwd (GET Current Working Directory)

#include "view.h"





// Global application object and pointer

CViewApp APP;
CViewApp *pAPP;



//***********************************************************************
//
// CLASS CViewApp
//
// This class, derived from CWinApp, contains the application portion of
// the View display program.  This portion contains code dealing with
// global variables, files, and the acquired data.  CViewWindow takes
// care of the graphics and main window functions.
//
//

BEGIN_MESSAGE_MAP(CViewApp, CWinApp)
        //{{AFX_MSG_MAP(CViewApp)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()



BOOL CViewApp::InitInstance() {

        TRACE0("CViewApp::InitInstance\n");

        RECT screenrect;


        // Call just in case this isn't a no-op

        CWinApp::InitInstance();



        // Get full screen size (could also have used GetSystemMetrics)

        SystemParametersInfo(SPI_GETWORKAREA,0,&screenrect,0);

        DefaultScreenWidth  = screenrect.right  - screenrect.left;
        DefaultScreenHeight = screenrect.bottom - screenrect.top;



        // Set initial data

        if (!InitAppData())
                return( FALSE );


        if ( !ProcessCommandLine( ) )
                return( FALSE );






        // Create main window and check validity.
        //
        // If invalid, delete it and NULL out the pointer so
        // DestroyWindow is not called when InitInstance fails.
        // The window class is already gone because of failing
        // during its construction phase.


        m_pMainWnd = (CWnd*)NewViewWindow();

        if ( ! ::IsWindow(m_pMainWnd->m_hWnd) ) {

                delete m_pMainWnd;
                m_pMainWnd = NULL;

                return( FALSE );
                }



        // Show main window

        m_pMainWnd->ShowWindow(m_nCmdShow);
        m_pMainWnd->UpdateWindow();



        return( TRUE );

}


int CViewApp::ExitInstance() {

        TRACE0("CViewApp::ExitInstance\n");


        // Clean up data arrays

        if (pDataArray)
                { free(pDataArray); pDataArray = NULL; }

        if (pDemuxArray)
                 { free(pDemuxArray); pDemuxArray = NULL; }


        // Clean up channel arrays

        if (pScanActive)
                { free(pScanActive);  pScanActive = NULL; }

        if (pScanChType)
                { free(pScanChType);  pScanChType = NULL; }

        if (pScanOrder)
                { free(pScanOrder);   pScanOrder  = NULL; }

        if (pScanTitle)
                { free(pScanTitle);   pScanTitle  = NULL; }

        if (pDigTitle)
                { free(pDigTitle);    pDigTitle = NULL; }


        if (pTMPoffset)
                { free(pTMPoffset);  pTMPoffset = NULL; }

        if (pDCoffset)
                { free(pDCoffset);   pDCoffset  = NULL; }

        if (pPlotChType)
                { free(pPlotChType); pPlotChType  = NULL; }

        if (pPlotTitle)
                { free(pPlotTitle);  pPlotTitle  = NULL; }


        return( CWinApp::ExitInstance() );
}

int CViewApp::ProcessCommandLine( ) {

        TRACE0("CViewApp::ProcessCommandLine\n");

        int     i, j, datafmt, len1;
        CString CmdLine, Arg1, Arg2, FileExt;
        CCommandLineInfo CmdLineInfo;


        // Quick return if command line is empty

        if (m_lpCmdLine[0] == '\0')
                return( 1 );

        // Check command line for filenames or empty

        CmdLine = m_lpCmdLine;
        CmdLine.TrimLeft();
        CmdLine.TrimRight();


        if ( CmdLine.Compare( "/?" ) == 0 ||
             CmdLine.Compare( "?" )  == 0 ) {
                ErrString  = "View can be started with zero, one, or two arguments.\n";
                ErrString += "If one argument is provided, it must be the name\n";
                ErrString += "of a valid .dat file.  If two arguments are given\n";
                ErrString += "they must be the names of a valid .out file and its\n";
                ErrString += "accompanying .hdr header file.  If no arguments\n";
                ErrString += "are given, you must use the File Menu Open Dialog\n";
                ErrString += "to specify the desired filenames.\n";
                AfxMessageBox( ErrString, MB_OK );
                ErrCode = 1;
                return( FALSE );
                }



        // Extract the arguments


        ParseCommandLine( CmdLineInfo );

        Arg1 = CmdLineInfo.m_strFileName;
        len1 = Arg1.GetLength();

        j = 0;
        for ( i = len1; i > 0 ; i-- ) {
                if (Arg1[i-1] == '.')
                        break;
                j++;
                }
        FileExt = Arg1.Right(j);

        if ( FileExt.CompareNoCase( "DAT" ) == 0 )
                datafmt = FILEFMT_DAT;
        else if ( FileExt.CompareNoCase( "OUT" ) == 0 )
                datafmt = FILEFMT_OUT;
        else
                datafmt = FILEFMT_OTHER;


        // No header file taken from command line yet

        Arg2.Empty();



        // Check that the specified files exist with read permission

        if ( SrFileReadable( Arg1 ) ) {
                if ( !SetFileInfo( datafmt, Arg1, Arg2 ) ) {
                        ShowError();
                        return( 0 );
                        }
                }
        else {
                ErrString.Format( "%s\n", (LPCTSTR)CmdLine );
                ErrString += "Command line ignored: .dat or .out file must exist and be readable\n";
                ErrString += "Please use the File > Open menu option to select a valid data file.\n";
                ShowError();
                }



        return( 1 );
}

char *GuessHdr[] = { "SCOPE", "SIMP", "SIMP1CH", "SIMP4CH", "SIMP8CH", "DEMO", NULL };

int CViewApp::SetFileInfo( int DataFmt, CString &Name1, CString &Name2 ) {

        TRACE0("CViewApp::SetFileInfo\n");

        int            FoundHdr, SeqStartDec, SeqStartHex, SeqType;
        CFileException Except;
        CFileStatus    Status;
        CStdioFile     DataFile;
        char           **Guess;
        char path[_MAX_PATH], name[_MAX_PATH];
        char drive[_MAX_DRIVE], dir[_MAX_DIR], prefix[_MAX_FNAME], ext[_MAX_EXT];


        // Open data file and get its name.

        if ( DataFile.Open( Name1, (CFile::modeRead|CFile::typeBinary), &Except ) ) {
                DataFullName = DataFile.GetFilePath();

                // If file format was not determined by extension, check
                // contents for dat format and set format flag accordingly.

                if (DataFmt == FILEFMT_OTHER) {
                        if ( SrDatHdrRead( DataFile.m_pStream, &DatHdr ) &&
                             SrDatHdrValidate( &DatHdr )                 )
                                DataFmt = FILEFMT_DAT;
                        else
                                DataFmt = FILEFMT_OUT;
                        }

                DataFile.Close();
                }
        else {
                ErrString.Format("Could not open data file %s.\n", Name1 );
                AppendFileExceptionCause( &Except );
                ErrString += "  Please cancel or choose again.\n";
                return( 0 );
                }


        // Set data format

        DataFormat = DataFmt;


        // Compute additional data file info: path, prefix, extension

        _splitpath( DataFullName, drive, dir, prefix, ext );
        _makepath( path, drive, dir, NULL, NULL );
        _makepath( name, NULL, NULL, prefix, ext );
        DataPrefix = prefix;
        DataPath   = path;
        DataName   = name;
        DataExt    = ext;

        OpenDir = DataPath; // Initial dir for the open FileDialog



        // For .out files, get status and full name of header file

        if (DataFmt == FILEFMT_OUT) {

                // If no header name is given, then guess one

                if ( Name2.IsEmpty() ) {
                        FoundHdr = 0;
                        Guess    = GuessHdr;
                        while ( !FoundHdr && (Guess != NULL) ) {
                                Name2.Format( "%s%s.HDR", DataPath, *Guess );
                                if ( SrFileReadable( Name2 ) )
                                        FoundHdr = 1;
                                Guess++;
                                }
                        if (!FoundHdr) {
                                ErrString.Format("Could not guess header file name.\n" );
                                ErrString += "  Please cancel or choose again.\n";
                                return( 0 );
                                }
                        } // end is empty

                if (CFile::GetStatus( Name2, Status ) ) {
                        HdrFullName = Status.m_szFullName;
                        _splitpath( HdrFullName, NULL, NULL, prefix, ext );
                        _makepath( name, NULL, NULL, prefix, ext );
                        HdrName = name;
                        }
                else {
                        ErrString.Format("Could not access header file %s.\n", Name2 );
                        ErrString += "  Please cancel or choose again.\n";
                        return( 0 );
                        }
                }

        // No header file for .dat files

        else {
                HdrFullName.Empty();
                HdrName.Empty();
                }



        // Allow sequential files if the file prefix is numeric

        if (IsFilePrefixNumeric( (LPCTSTR)DataPrefix,
                                 &SeqStartDec, &SeqStartHex, &SeqType ) )
                DataDoSeq = TRUE;
        else
                DataDoSeq = FALSE;



        // Now set additional values from the header info

        if ( !ReadHdrFromFile( SeqStartDec, SeqStartHex, SeqType ) ) {
                ErrString = "Failed to read header\n";
                return( 0 );
                }


        // Allocate data buffers

        ResetDC = TRUE;
        ComputeBufferSizes();
        UpdatePlotTraceInfo();

        if ( ! AllocateBuffers() ) {
                ErrString = "Failed to allocate demux buffers\n";
                return( 0 );
                }


        return( 1 );
}

int CViewApp::IsFilePrefixNumeric( const char *FilePrefix,
                                   int *NumberDec, int *NumberHex, int *Type ) {

        TRACE0("CViewApp::IsFilePrefixNumeric\n");

        int nscandec, nscanhex, numdec, numhex, c, seqtype;
        const char *fp;

        // Check to see if there are any non-(hex)numeric characters
        // in the given filename

        // Set value in case of early exit

        if (NumberDec)  *NumberDec = 0;
        if (NumberHex)  *NumberHex = 0;
        if (Type)       *Type      = SEQTYPE_NONE;


        // Initialize sequence type.  If a hex alpha digit is found,
        // then we know we're in hex mode.  If all the digits are
        // purely numeric, we're still not sure.

        seqtype = SEQTYPE_NONE;


        // Loop for all characters in file prefix

        fp = FilePrefix;

        while ( *fp != '\0' ) {

                // Get next possible digit

                c = (int)(*fp);


                // If not a hex digit, we're done

                if ( !isxdigit( c ) )
                        return( 0 );


                // If not a decimal digit, we must be in hex mode

                if ( !isdigit( c ) )
                        seqtype = SEQTYPE_HEX;

                fp++;

                } // end while fp


        // They were all valid hex digits, so scan in the entire number
        // Read it as hex if a hex character was found or decimal otherwise

        nscandec = sscanf( FilePrefix, "%ld", &numdec );
        nscanhex = sscanf( FilePrefix, "%lx", &numhex );

        if (NumberDec)  *NumberDec = numdec;
        if (NumberHex)  *NumberHex = numhex;
        if (Type)       *Type      = seqtype;

        if (nscandec == 1 && nscanhex == 1)
                return( 1 );
        else
                return( 0 ); // Failed to scan the number (should never happen)
}

CViewWindow* CViewApp::NewViewWindow() {

        TRACE0("CViewApp::NewViewWindow\n");

        CViewWindow* pWnd = new CViewWindow;

        return( pWnd );
}

int CViewApp::InitAppData() {

        TRACE0("CViewApp::InitAppData\n");

        int  i, j;
        char ChanString[MAXSTR];



        // Set general global application pointer

        pAPP = this;



        // Initialize common global constants

        ErrCode              = 0;
        LastNbufferbytes     = 0;
        LastNbufferbytesPlot = 0;

        IsAllocated          = FALSE;
        ResetPaint           = TRUE;
        ResetDC              = TRUE;

        pDemuxArray          = NULL;
        pDataArray           = NULL;

        hWndPrev             = NULL;

        ErrString            = "Error:";
        szTemp[0]            = '\0';

        GpsMarkChannel       = SRDAT_INDEXCHANNEL_NONE;



        // Get current working directory
        // This is curr dir from Dos prompt, and exe dir from Win Run...

        if (_getcwd( szTemp, MAXSTR ) != NULL)
                CurrDir = szTemp;
        else
                CurrDir = ".";



        AppTitle.LoadString( IDR_VIEW );
        AppRev.LoadString(   IDR_REV );



        // Initialize min and max values

        MinDigitalChannel   = 0;
        MinGpsChannel       = 0;
        MinCounterChannel   = 0;
        MinAnaPlotChannel   = 0;
        MinDigPlotChannel   = 0;
        MinGpsPlotChannel   = 0;
        MinCntPlotChannel   = 0;
        MinTotalPlotChannel = 0;


        // These channel counts use the maximum for all PARxCH devices

        MinAnalogChannel    = PARXCH_MAX_ANALOG_CHANNELS;

        MaxAnalogChannel    = PARXCH_MAX_ANALOG_CHANNELS;
        MaxDigitalChannel   = PARXCH_MAX_DIGITAL_CHANNELS;
        MaxGpsChannel       = PARXCH_MAX_GPS_CHANNELS;
        MaxCounterChannel   = PARXCH_MAX_COUNTER_CHANNELS;

        MaxAnaPlotChannel   = PARXCH_MAX_ANAPLOT_CHANNELS;
        MaxDigPlotChannel   = PARXCH_MAX_DIGPLOT_CHANNELS;
        MaxGpsPlotChannel   = PARXCH_MAX_GPSPLOT_CHANNELS;
        MaxCntPlotChannel   = PARXCH_MAX_CNTPLOT_CHANNELS;


        MinTotalChannel     = MinAnalogChannel;
        MaxTotalChannel     = MaxAnalogChannel + MaxDigitalChannel +
                              MaxGpsChannel    + MaxCounterChannel;
        MaxTotalPlotChannel = MaxAnaPlotChannel + MaxDigPlotChannel +
                              MaxGpsPlotChannel + MaxCntPlotChannel;


        MinAmpShift = 0;
        MaxAmpShift = 24;

        MaxSample       = 512;  // max num values/chan/buffer
        DefaultNsamples = 128;  // typical num values/chan/buffer

        Nplotbuffers = (DefaultScreenWidth - HTITLEMARGIN) / DefaultNsamples;




        // Allocate new channel arrays

        pTMPoffset  = (double*) malloc( MaxTotalPlotChannel * sizeof(double) );
        pDCoffset   = (long*)   malloc( MaxTotalPlotChannel * sizeof(long) );
        pPlotChType = (int*)    malloc( MaxTotalPlotChannel * sizeof(int) );
        pPlotTitle  = (char*)   malloc( MaxTotalPlotChannel * sizeof(char) * MAXTITLE );

        pScanActive  = (long*) malloc( MaxTotalChannel * sizeof(long) );
        pScanChType  = (int*)  malloc( MaxTotalChannel * sizeof(int) );
        pScanOrder   = (int*)  malloc( MaxTotalChannel * sizeof(int) );
        pScanTitle   = (char*) malloc( MaxTotalChannel * sizeof(char) * MAXTITLE );

        pDigTitle   = (char*) malloc( MaxDigPlotChannel * sizeof(char) * MAXTITLE );


        if (!pScanActive  || !pScanOrder   || !pScanChType  ||
            !pScanTitle   || !pDigTitle    ||
            !pTMPoffset || !pDCoffset || !pPlotChType || !pPlotTitle ) {
                ErrString = "Could not allocate scan buffers or offset";
                ShowError();
                return( FALSE );
                }



        // Initialize offsets

        for ( j = 0 ; j < MaxTotalPlotChannel ; j++ ) {

                pTMPoffset[j]  = 0.0;
                pDCoffset[j]   = 0;
                pPlotChType[j] = CH_TYPE_ANALOG;
                strcpy( &pPlotTitle[j*MAXTITLE], "" );
                }



        // Program setting parameter defaults

        DoStatusBar    = TRUE;


        // Acquisition parameter defaults

        AcqSPS         = (float)PARXCH_SPS_DEFAULT;



        // Display parameter defaults

        PltClipping    = FALSE;
        PltAutoDC      = TRUE;
        PltShiftBy     = 8;  // was MaxAmpShift/2;
        PltNdisplay    = MaxTotalPlotChannel;
        PltFirstSample = 0;
        PltFirstTrace  = 0;
        PltColorScheme = CS_GREEN;


        ConvertToOffset = CONVERT_OFFSET;



        // Sequential File and Naming defaults

        DataNaming      = FILE_NONE;
        DataSeqType     = SEQTYPE_NONE;
        DataSeqInc      = 1;



        // Set channel defaults assuming PAR8CH with digital and GPS

        ClearChannels();


        XchModel     = PARXCH_MODEL_PAR8CH;
        Nchannels    = MaxTotalChannel;
        NsamplesBuff = DefaultNsamples;

        DigitalEnable = TRUE;
        GpsEnable     = TRUE;
        CounterEnable = TRUE;

        for ( i = 0 ; i < Nchannels ; i++ ) {

                sprintf( ChanString, "CH%02d", i );
                SrStringCopy( &pScanTitle[i*MAXTITLE], ChanString, MAXTITLE );
                pScanActive[i]  = CHECKED;
                pScanChType[i]  = CH_TYPE_ANALOG;
                pScanOrder[i]   = i;
                }



        i = MaxAnalogChannel;
        SrStringCopy( &pScanTitle[i*MAXTITLE], "DGTL", MAXTITLE );
        pScanChType[i]  = CH_TYPE_DIGITAL;

        i++;
        SrStringCopy( &pScanTitle[i*MAXTITLE], "MARK", MAXTITLE );
        pScanChType[i]  = CH_TYPE_GPSMARK;

        i++;
        SrStringCopy( &pScanTitle[i*MAXTITLE], "CNTR", MAXTITLE );
        pScanChType[i]  = CH_TYPE_GPSCOUNTER;


        for ( i = 0 ; i < MaxDigPlotChannel ; i++ ) {
                sprintf( ChanString, "DIG%01d", i );
                SrStringCopy( &pDigTitle[i*MAXTITLE], ChanString, MAXTITLE );
                }

        ComputeNchannelsPlot();



        // Compute dependent values

        ComputeBufferSizes();
        UpdatePlotTraceInfo();



        // Data view parameters

        DataFormat = FILEFMT_NONE;
        DataExt    = ".DAT";
        DataPrefix = "00000000";
        DataName.Format( "%s%s", DataPrefix, DataExt );
        DataPath.Format( "%s\\", CurrDir );
        DataFullName.Format( "%s%s", DataPath, DataName );
        HdrName    = "SCOPE.HDR";
        HdrFullName.Format( "%s%s", DataPath, HdrName );

        DataDoSeq       = TRUE;
        DataSeqHome     = 0;
        DataPtsPerBuff  = NsamplesBuff;
        DataBuffPerFile = Nplotbuffers;
        DataPtsPerFile  = DataPtsPerBuff * DataBuffPerFile;
        DataBytesPerPt  = Nchannels * DATA_PT_SIZE;
        DataCurrentPt   = 0;

        ComputeBoundingPts();


        // Default initial directory for the open FileDialog

        OpenDir = DataPath;

        return( 1 );
}

int CViewApp::AllocateBuffers( void ) {

        TRACE0("CViewApp::AllocateBuffers\n");

        int   i, j;
        long *pData, joff;


        ComputeBufferSizes();


        // Only re-dim if arrays needs to get bigger

        if (LastNbufferbytes < Nbufferbytes) {

                LastNbufferbytes = Nbufferbytes;


                // Clean up any previous raw data array and allocate new one

                if (pDataArray)
                        { free(pDataArray); pDataArray = NULL; }

                pDataArray = (long*) malloc(Nbufferbytes);

                }


        if (LastNbufferbytesPlot < NbufferbytesPlot) {

                LastNbufferbytesPlot = NbufferbytesPlot;


                // Clean up any previous demux array and allocate new one

                if (pDemuxArray)
                        { free(pDemuxArray); pDemuxArray = NULL; }

                pDemuxArray = (long*) malloc(NbufferbytesPlot);

                }


        // Check for successful allocation

        if (!pDemuxArray || !pDataArray) {

                IsAllocated = FALSE;

                ErrString = "Can not allocate data buffers";
                ShowError( );

                }

        else
                IsAllocated = TRUE;




        // When allocated, clear array (even if remains allocated from before)

        if (IsAllocated == TRUE) {

                pData = pDemuxArray;

                for ( j = 0 ; j < NchannelsPlot ; j++ ) {

                        joff = pDCoffset[j];

                        for ( i = 0 ; i < NsamplesPlot ; i++ )

                                *pData++ = joff;

                        }

                }


        return( IsAllocated );
}


// Call this function if Nchannels, NsamplesBuff, or Nplotbuffers changes

int CViewApp::ComputeBufferSizes( void ) {

        TRACE0("CViewApp::ComputeBufferSizes\n");

        NsamplesPlot     = Nplotbuffers   * NsamplesBuff;
        Nbufferpts       = NsamplesPlot   * Nchannels;
        NbufferptsPlot   = NsamplesPlot   * NchannelsPlot;
        Nbufferbytes     = Nbufferpts     * sizeof( long );
        NbufferbytesPlot = NbufferptsPlot * sizeof( long );

        return( 1 );
}


// Call this function if Nchannels, GpsEnable, DigitalEnable, or CounterEnable changes

int CViewApp::ComputeNchannelsPlot( ) {

        TRACE0("CViewApp::ComputeNchannelsPlot\n");

        // Update NchannelsPlot after changes to Nchannels

        NchannelsPlot = Nchannels;
        if (GpsEnable)
                NchannelsPlot += (MaxGpsPlotChannel-MaxGpsChannel);
        if (DigitalEnable)
                NchannelsPlot += (MaxDigPlotChannel-MaxDigitalChannel);
        if (CounterEnable)
                NchannelsPlot += (MaxCntPlotChannel-MaxCounterChannel);


        // Display all channels

        PltNdisplay   = NchannelsPlot;
        PltFirstTrace = 0;


        return( 1 );
}


int CViewApp::UpdatePlotTraceInfo() {

        TRACE0("CViewApp::UpdatePlotTraceInfo\n");

        int i, j, k, ichan;

        // First, clear off any existing plot titles

        for ( j = 0 ; j < MaxTotalPlotChannel ; j++ )
                strcpy( &pPlotTitle[j*MAXTITLE], "" );


        // Then update arrays for plot titles and trace type info
        // based on scan info.

        // Trace info, indexed by j, goes to NchannelsPlot and
        // scan order info, indexed by i, goes to Nchannels and
        // selects the regular scan info indexed by ichan which
        // goes to MaxTotalChannel.

        j = 0;

        for ( i = 0 ; i < Nchannels ; i++ ) {

                ichan = pScanOrder[i];

                if (pScanChType[ichan] == CH_TYPE_DIGITAL) {
                        for ( k = 0 ; k < MaxDigPlotChannel ; k++ ) {
                                pPlotChType[j] = pScanChType[ichan];
                                SrStringCopy( &pPlotTitle[j*MAXTITLE],
                                              &pDigTitle[k*MAXTITLE],
                                              MAXTITLE );
                                j++;
                                }
                        }
                else if (pScanChType[ichan] == CH_TYPE_GPSCOUNTER) {
                        ; // do not plot or show in list
                        }
                else { // CH_TYPE_ANALOG or CH_TYPE_GPS
                        pPlotChType[j] = pScanChType[ichan];
                        SrStringCopy( &pPlotTitle[j*MAXTITLE],
                                      &pScanTitle[ichan*MAXTITLE],
                                       MAXTITLE );
                        j++;
                        }

                }

        return( 1 );
}

int CViewApp::ClearChannels() {

        TRACE0("CViewApp::ClearChannels\n");

        int  i, j;
        char ChanString[MAXSTR];

        // Clear all channel info

        XchModel = PARXCH_MODEL_UNKNOWN;

        DigitalEnable = FALSE;
        GpsEnable     = FALSE;
        CounterEnable = FALSE;

        Nchannels    = 1;
        NsamplesBuff = DefaultNsamples;

        ComputeNchannelsPlot();


        for ( i = 0 ;  i < MaxTotalChannel ; i++ ) {

                sprintf( ChanString, "CH%02d", i );
                SrStringCopy( &pScanTitle[i*MAXTITLE], ChanString, MAXTITLE );
                pScanActive[i]  = UNCHECKED;
                pScanChType[i]  = CH_TYPE_NONE;
                pScanOrder[i]   = i;
                }
        pScanActive[0] = CHECKED; // make one active

        for ( j = 0 ; j < MaxDigPlotChannel ; j++ ) {
                sprintf( ChanString, "DIG%1d", j );
                SrStringCopy( &pDigTitle[j*MAXTITLE], ChanString, MAXTITLE );
                }



        return( 1 );
}

int CViewApp::ReadHdrFromFile( int SeqStartDec, int SeqStartHex, int SeqType ) {

        TRACE0("CViewApp::ReadHdrFromFile\n");

        CStdioFile     HdrFile;
        CFileStatus    status;
        CFileException except;
        int            i;


        // Quick exit if no header file to read

        if (DataFormat == FILEFMT_NONE)
                return( 0 );


        // Out style header

        if (DataFormat == FILEFMT_OUT) {

                if (!HdrFile.Open( HdrFullName, (CFile::modeRead|CFile::typeBinary), &except ) ) {

                        ErrString.Format("Could not read header file %s.\n",
                                                                (LPCTSTR) HdrFullName);
                        AppendFileExceptionCause( &except );
                        ShowError();
                        return( 0 );
                        }

                if (!HdrFile.GetStatus( status )) {
                        ErrString.Format("Could not get status of header file %s.",
                                                                (LPCTSTR) HdrFullName);
                        ShowError();
                        return( 0 );
                        }

                HdrFile.Read( &OutHdr, (UINT)status.m_size );
                HdrFile.Close();

                } // end of FILEFMT_OUT


        // Dat style header

        else if (DataFormat == FILEFMT_DAT) {

                if (!HdrFile.Open( DataFullName, (CFile::modeRead|CFile::typeBinary), &except ) ) {

                        ErrString.Format("Could not read .dat file %s.\n",
                                                 (LPCTSTR) DataFullName);
                        AppendFileExceptionCause( &except );
                        ShowError();
                        return( 0 );
                        }


                if ( !SrDatHdrRead( HdrFile.m_pStream, &DatHdr ) ) {
                        HdrFile.Close();
                        return( 0 );
                        }

                HdrFile.Close();



                // Check for validity

                if ( !SrDatHdrValidate( &DatHdr ) ) {
                        ErrString.Format( "This is not a valid .DAT file." );
                        ShowError();
                        return( 0 );
                        }

                } // end of FILEFMT_DAT


        else {
                ErrString.Format("Unknown header format requested %d.", DataFormat);
                ShowError();
                return( 0 );
                }




        // Update variables from header

        ClearChannels();


        // Set Nchannels and NsamplesBuff
        // Plus pScanOrder, pScanActive, and pScanTitle

        if (DataFormat == FILEFMT_OUT) {  // values from separate .hdr file

                Nchannels       = OutHdr.Channels;
                NsamplesBuff    = DefaultNsamples;
                DataPtsPerBuff  = DefaultNsamples;
                DataPtsPerFile  = OutHdr.RecordPts;
                DataBuffPerFile = DataPtsPerFile / DataPtsPerBuff;
                AcqSPS          = (float)OutHdr.SampleRate;

                for ( i = 0 ; i < Nchannels ; i++ ) {
                    pScanOrder[i]  = i;
                    pScanActive[i] = CHECKED;
                    strcpy( &pScanTitle[i*MAXTITLE], OutHdr.ChannelList[i].title );
                    }

                if (OutHdr.DataCoding == 0)
                        ConvertToOffset = CONVERT_OFFSET;
                else
                        ConvertToOffset = CONVERT_SIGNED;



                // Set seq type and starting value

                if (DataSeqType == SEQTYPE_NONE) {
                        if (SeqType == SEQTYPE_DEC)
                                DataSeqType = SEQTYPE_DEC;
                        else if (SeqType == SEQTYPE_HEX)
                                DataSeqType = SEQTYPE_HEX;
                        else // default to decimal
                                DataSeqType = SEQTYPE_DEC;
                        }

                if (DataSeqType == SEQTYPE_DEC) {
                        DataSeqHome = SeqStartDec;
                        if (SeqType == SEQTYPE_HEX) {
                                ErrString  = "Decimal naming selected, but file name is in hex.\n";
                                ErrString += "Switching to hex naming.\n";
                                AfxMessageBox( ErrString, MB_OK );
                                DataSeqType = SEQTYPE_HEX;
                                DataSeqHome = SeqStartHex;
                                }
                        }
                else // (DataSeqType == SEQTYPE_HEX)
                        DataSeqHome = SeqStartHex;

                // No way to tell about naming style, so just hardwire something

                DataNaming = FILE_SEQDEC;

                } // end FILEFMT_OUT




        else { // (DataFormat == FILEFMT_DAT)  values from header in .dat file

                Nchannels       = DatHdr.Channels;
                NsamplesBuff    = DatHdr.ChannelPtsPerRecord;
                DataPtsPerBuff  = DatHdr.ChannelPtsPerRecord;
                DataPtsPerFile  = DatHdr.ChannelPtsPerFile;
                DataBuffPerFile = DataPtsPerFile / DataPtsPerBuff;
                AcqSPS          = (float)DatHdr.AtodSamplingRate;

                if (DatHdr.FileNameType == SRDAT_FILENAMETYPE_SEQDECIMAL) {
                        DataNaming  = FILE_SEQDEC;
                        DataSeqInc  = DatHdr.FileNameInc;
                        DataSeqHome = SeqStartDec;
                        if (DataSeqType == SEQTYPE_NONE)
                                DataSeqType = SEQTYPE_DEC;
                        else if (DataSeqType == SEQTYPE_HEX) {
                                ErrString = "Hex naming selected, but file used decimal naming.\n";
                                ErrString += "Switching to decimal naming.\n";
                                AfxMessageBox( ErrString, MB_OK );
                                DataSeqType = SEQTYPE_DEC;
                                }
                        }
                else if (DatHdr.FileNameType == SRDAT_FILENAMETYPE_SEQHEX) {
                        DataNaming  = FILE_SEQHEX;
                        DataSeqInc  = DatHdr.FileNameInc;
                        DataSeqHome = SeqStartHex;
                        if (DataSeqType == SEQTYPE_NONE)
                                DataSeqType = SEQTYPE_HEX;
                        else if (DataSeqType == SEQTYPE_DEC) {
                                ErrString = "Decimal naming selected, but file used hex naming.\n";
                                ErrString += "Switching to hex naming.\n";
                                AfxMessageBox( ErrString, MB_OK );
                                DataSeqType = SEQTYPE_HEX;
                                }
                        }
                else if (DatHdr.FileNameType == SRDAT_FILENAMETYPE_TIME) {
                        DataDoSeq   = FALSE; // Don't allow sequencing through
                                             // time based names until we can
                                             // do it reliably
                        DataNaming  = FILE_TIME;
                        DataSeqInc  = DatHdr.FileNameInc;
                        DataSeqHome = SeqStartHex;
                        if (DataSeqType == SEQTYPE_NONE)
                                DataSeqType = SEQTYPE_HEX;
                        else if (DataSeqType == SEQTYPE_DEC) {
                                ErrString = "Decimal naming selected, but file used hex time naming.\n";
                                ErrString += "Switching to hex naming.\n";
                                AfxMessageBox( ErrString, MB_OK );
                                DataSeqType = SEQTYPE_HEX;
                                }
                        }
                else if (SeqType == SEQTYPE_HEX) {
                        DataNaming  = FILE_SEQHEX;
                        DataSeqInc  = 1;
                        DataSeqHome = SeqStartHex;
                        if (DataSeqType == SEQTYPE_NONE)
                                DataSeqType = SEQTYPE_HEX;
                        else if (DataSeqType == SEQTYPE_DEC) {
                                ErrString = "Decimal naming selected, but file name is in hex.\n";
                                ErrString += "Switching to hex naming.\n";
                                AfxMessageBox( ErrString, MB_OK );
                                DataSeqType = SEQTYPE_HEX;
                                }
                        }
                else if (DatHdr.FileNameType == SRDAT_FILENAMETYPE_UNKNOWN) {
                        DataNaming  = FILE_SEQHEX;
                        DataSeqInc  = 1;
                        DataSeqHome = SeqStartHex;
                        if (DataSeqType == SEQTYPE_NONE)
                                DataSeqType = SEQTYPE_HEX;
                        if (DataSeqType == SEQTYPE_DEC)
                                DataSeqHome = SeqStartDec;
                        }
                else { // (DatHdr.FileNameType == SRDAT_FILENAMETYPE_NONE or _SINGLE)
                        DataNaming  = FILE_ONE;
                        DataSeqInc  = 1;
                        DataSeqHome = SeqStartDec;
                        if (DataSeqType == SEQTYPE_NONE)
                                DataSeqType = SEQTYPE_DEC;
                        if (DataSeqType == SEQTYPE_HEX)
                                DataSeqHome = SeqStartHex;
                        }


                for ( i = 0 ; i < Nchannels ; i++ ) {
                    pScanOrder[i]  = i;
                    pScanActive[i] = CHECKED;
                    strcpy( &pScanTitle[i*MAXTITLE], DatHdr.ChannelList[i].Title );
                    }

                for ( i = 0 ; i < DatHdr.DigitalPerChan ; i++ )
                    strcpy( &pDigTitle[i*MAXTITLE], DatHdr.DigitalList[i].Title );

                if (DatHdr.AtodDataCoding == SRDAT_DATACODING_OFFSET)
                        ConvertToOffset = CONVERT_OFFSET;
                else
                        ConvertToOffset = CONVERT_SIGNED;

                } // end FILEFMT_DAT



        // From the number of channels we can guess which PARxCH was used

        if (Nchannels < 3)
                XchModel = PARXCH_MODEL_PAR1CH;
        else if (Nchannels == 4 || Nchannels == 5)
                XchModel = PARXCH_MODEL_PAR4CH;
        else if (Nchannels > 7 && Nchannels < 12)
                XchModel = PARXCH_MODEL_PAR8CH;
        else
                XchModel = PARXCH_MODEL_UNKNOWN;





        // Determine the channel type and what is enabled ...

        DigitalEnable = FALSE;
        GpsEnable     = FALSE;
        CounterEnable = FALSE;


        // ... using recent .dat header info

        if ( (DataFormat == FILEFMT_DAT) && (DatHdr.DatRev > 105) ) {

                for ( i = 0 ; i < Nchannels ; i++ ) {
                        if (DatHdr.ChannelDigIndex == i) {
                                pScanChType[i] = CH_TYPE_DIGITAL;
                                DigitalEnable  = TRUE;
                                }
                        else if (DatHdr.ChannelMarkIndex == i) {
                                pScanChType[i] = CH_TYPE_GPSMARK;
                                GpsEnable      = TRUE;
                                }
                        else if (DatHdr.ChannelObcIndex == i) {
                                pScanChType[i] = CH_TYPE_GPSCOUNTER;
                                CounterEnable  = TRUE;
                                }
                        else
                                pScanChType[i] = CH_TYPE_ANALOG;
                        }
                } // end if DataFormat && DatRev


        // ... using their names

        else {

                for ( i = 0 ; i < Nchannels ; i++ ) {
                        if ( strcmp( &pScanTitle[i*MAXTITLE], "DGTL" ) == 0 ) {
                                pScanChType[i] = CH_TYPE_DIGITAL;
                                DigitalEnable  = TRUE;
                                }
                        else if ( strcmp( &pScanTitle[i*MAXTITLE], "MARK" ) == 0 ) {
                                pScanChType[i] = CH_TYPE_GPSMARK;
                                GpsEnable      = TRUE;
                                }
                        else if ( strcmp( &pScanTitle[i*MAXTITLE], "CNTR" ) == 0 ) {
                                pScanChType[i] = CH_TYPE_GPSCOUNTER;
                                CounterEnable  = TRUE;
                                }
                        else
                                pScanChType[i] = CH_TYPE_ANALOG;
                        }

                } // end else not DataFormat && DatRev




        // Compute some dependent values

        ComputeNchannelsPlot();
        ComputeBufferSizes();

        DataCurrentPt  = 0;
        DataBytesPerPt = Nchannels * DATA_PT_SIZE;

        ComputeBoundingPts();


        // Consider some additional checks for dat files
        // since we know that
        // pScanChType[DatHdr.ChannelMarkIndex] == CH_TYPE_GPSMARK

        return( 1 );
}

int CViewApp::ComputeBoundingPts( ) {

        TRACE0("CViewApp::ComputeBoundingPts\n");

        // Compute bounding start points

        DataMinPt = 0;
        DataMaxPt = DataPtsPerFile  - NsamplesPlot;
        if (DataMaxPt < DataMinPt)
                DataMaxPt = DataMinPt;

        return( 1 );
}

int CViewApp::GetFilePosForPt( int Pt, int *FileSeq, int *FileRec, int *FilePt ) {

        TRACE0("CViewApp::GetFilePosForPt\n");

        int fp, fr, fs, fx, corrsign;


        // Compute location of Pt in terms of the file sequence, record,
        // and point.

        if (Pt < 0)
                corrsign = 1;
        else
                corrsign = 0;

        fs = (int)((Pt+corrsign) / DataPtsPerFile ) - corrsign;
        fx = Pt - (fs * DataPtsPerFile);
        fr = (int)(fx / DataPtsPerBuff);
        fp = fx - (fr * DataPtsPerBuff);

        *FileSeq = (fs * DataSeqInc) + DataSeqHome;
        *FileRec = fr;
        *FilePt  = fp;

        // The file sequence number is only valid when using sequential files.

        if (!DataDoSeq)
                *FileSeq = -1;

        return( 1 );
}

int CViewApp::ReadDataFromFile( int FileSeq, int FileRec, int FilePt, int NumPts, int FirstPt ) {

        TRACE0("CViewApp::ReadDataFromFile\n");

        int            skip, npts, FilePos;
        unsigned int   Nbytes, NbData, nb;
        CString        CurrFullName;
        CStdioFile     DataFile;
        CFileStatus    status;
        CFileException except;
        long           TagType, nfill;
        SRDAT_TAG      Tag;

        // This function fills the pDataArray


        // Generate the name of the file to read from

        if (DataDoSeq) {
                if ( DataSeqType == SEQTYPE_DEC)
                        CurrFullName.Format( "%s%08d%s", DataPath, FileSeq, DataExt );
                else // ( DataSeqType == SEQTYPE_HEX)
                        CurrFullName.Format( "%s%08X%s", DataPath, FileSeq, DataExt );
                }
        else
                CurrFullName = DataFullName;

        Nbytes = NumPts * DataBytesPerPt; // bytes/pt includes ALL Nchannels


        // Open file

        if (!DataFile.Open( CurrFullName, (CFile::modeRead|CFile::typeBinary), &except ) ) {
                ErrString.Format("Could not read data file %s.\n",
                                 (LPCTSTR)CurrFullName);
                AppendFileExceptionCause( &except );
                ShowError();
                return( 0 );
                }


        // Compute proper start position

        if (DataFormat == FILEFMT_OUT) {

                // For .out files, the data records are all contiguous
                // so we can seek to the exact point all at once.

                FilePos  = (FileRec * DataPtsPerBuff) + FilePt;
                FilePos *= DataBytesPerPt; // convert to bytes
                }

        else {
                // For .dat files, we will just seek past the header and
                // then start skipping records until we get to the one
                // we want.

                FilePos = SRDAT_HDR_TOTALSIZE; // in bytes
                }


        // Seek to proper start position in bytes

        TRY {
                DataFile.Seek( (long)FilePos, CFile::begin );
                }
        CATCH( CFileException, e ) {
                ErrString.Format("Could not seek to byte %d in file %s.\n",
                                 FilePos,(LPCTSTR)CurrFullName);
                AppendFileExceptionCause( e );
                ShowError( );
                DataFile.Close();
                return( 0 );
                }
        END_CATCH



        // Read data

        if (DataFormat == FILEFMT_OUT) { // from .out file

                TRY {
                        nb = 0;
                        nb = DataFile.Read( &pDataArray[FirstPt*Nchannels], Nbytes );
                        }
                CATCH( CFileException, e ) {
                        ErrString.Format("Could not read data from %s (npts=%d).\n",
                                (LPCTSTR)CurrFullName, nb/DataBytesPerPt );
                        AppendFileExceptionCause( e );
                        ShowError( );
                        DataFile.Close();
                        return( 0 );
                        }
                END_CATCH

                } // end FILEFMT_OUT




        else { // (DataFormat == FILEFMT_DAT)  from .dat file

                TagType = SRDAT_TAGID_ANALOG24;
                nb      = 0;


                while (TagType != SRDAT_TAGID_EOF) {

                        if ( !SrDatTagRead( DataFile.m_pStream, &Tag ) ) {
                                Tag.TagId  = SRDAT_TAGID_EOF;
                                Tag.Nbytes = 0;
                                break;
                                }
                        TagType = Tag.TagId;
                        NbData  = Tag.Nbytes;

                        // Decide what to do with this record

                        if (TagType != SRDAT_TAGID_ANALOG24) // Not data record
                                skip = 1;

                        else if (FileRec > 0) {              // Not right one
                                FileRec--;
                                skip = 1;
                                }
                        else                                 // Read this one!
                                skip = 0;


                        // Now do it

                        if (skip) {

                                if ( !SrDatDataSkip( DataFile.m_pStream, NbData ) ) {
                                        DataFile.Close();
                                        return( 0 );
                                        }
                                } // end if skip

                        else { // read

                                nfill = (FirstPt * DataBytesPerPt) + NbData;
                                if (nfill > Nbufferbytes) {
                                        ErrString.Format( ".DAT analog data buffer too small need %ld, have %ld",
                                                nfill, Nbufferbytes);
                                        ShowError();
                                        DataFile.Close();
                                        return( 0 );
                                        }

                                if ( !SrDatDataRead( DataFile.m_pStream,&pDataArray[FirstPt*Nchannels],NbData ) ) {
                                        DataFile.Close();
                                        return( 0 );
                                        }

                                nb      += NbData;
                                FirstPt += (NbData/DataBytesPerPt); // pts for 1 ch


                                // Stop if we've read enough

                                if (nb >= Nbytes)
                                        break;

                                } // end else read

                        } // end while TagType

                } // end FILEFMT_DAT


        DataFile.Close();
        npts = nb/DataBytesPerPt;

        return( npts );
}

void CViewApp::AppendFileExceptionCause( CFileException *e ) {

        if (e->m_cause == CFileException::accessDenied)
                ErrString += "Access denied.";
        else if (e->m_cause == CFileException::fileNotFound)
                ErrString += "File not found.";
        else if (e->m_cause == CFileException::badPath)
                ErrString += "Bad path.";
        else if (e->m_cause == CFileException::invalidFile)
                ErrString += "Invalid File.";
        else if (e->m_cause == CFileException::endOfFile)
                ErrString += "End of File.";
        else {
                sprintf( szTemp, "%d", e->m_cause );
                ErrString += "Unknown cause ";
                ErrString += szTemp;
                }
}

#define CLEARTOPBIT  0x7FFFFFFF
#define CLEARTOPBYTE 0x00FFFFFF

int CViewApp::DemuxData() {

        TRACE0("CViewApp::DemuxData\n");

        int i, jc, jp, jscan, k, na, nc, nd, ns, np, kmux, kdmx, startpt;
        long pt;

        // Demux raw data values into the plottable demuxed array
        // The raw data can be in either signed or offset format.
        // But, we always want the demuxed data to be in offset
        // format to easy plotting computations.  We do this conversion
        // by and-ing out the top byte and xor-ing the 24th bit.
        // The xor constant is chosen elsewhere based on the data
        // format so that no change occurs when the input data is
        // already in offset format.

        // Note that word offset is used with two different meanings.
        // One is DC offset meaning the background level of a channel
        // and the other is offset format meaning data that goes from
        // 0x00000000 to 0x00FFFFFF.  The DC offset below is computed
        // based on the demuxed, offset format, data.


        startpt = 0;


        na = NsamplesPlot;
        nc = Nchannels;
        ns = NsamplesPlot;
        np = NchannelsPlot;
        nd = MaxDigPlotChannel;


        jp = 0;
        for ( jc = 0 ; jc < nc ; jc++ ) {

                jscan = pScanOrder[jc];

                // Standard processing for analog channels

                if (pScanChType[jscan] == CH_TYPE_ANALOG) {

                        pTMPoffset[jp] = 0.0;
                        for ( i = 0 ; i < na ; i++ ) {
                                kmux = i*nc+jc;
                                kdmx = jp*ns+i+startpt;
                                pDemuxArray[kdmx] = (pDataArray[kmux]
                                                     & CLEARTOPBYTE)
                                                     ^ ConvertToOffset;
                                pTMPoffset[jp] += (double)pDemuxArray[kdmx];
                                } // end for i
                        jp++;

                        } // end analog processing



                // Special processing for GPS Mark channel (no offset convert)

                else if (pScanChType[jscan] == CH_TYPE_GPSMARK) {

                        for ( i = 0 ; i < na ; i++ ) {
                                kmux = i*nc+jc;
                                kdmx = jp*ns+i+startpt;
                                pDemuxArray[kdmx] = pDataArray[kmux];
                                }
                        pTMPoffset[jp] = 0;
                        jp++;
                        } // end gps processing


                // Special processing for GPS Counter channel (do not plot)

                else if (pScanChType[jscan] == CH_TYPE_GPSCOUNTER) {

                        } // end GPS counter processing



                // Special processing for Digital channel - split into
                // multiple parts for plotting

                else { // if (pScanChType[jscan] == CH_TYPE_DIGITAL) {

                        for ( k = 0 ; k < nd ; k++ ) {
                                for ( i = 0 ; i < na ; i++ ) {
                                        kmux = i*nc+jc;
                                        kdmx = jp*ns+i+startpt;
                                        pt = pDataArray[kmux];
                                        pDemuxArray[kdmx] = ((pt >> k) & 0x01);
                                        }
                                pTMPoffset[jp] = 0;
                                jp++;
                                }

                        } // end digital processing

                } // end for jc



        // Use the offset computed from the first AD buffer for
        // all parts of the plot buffer

        if (ResetDC == TRUE) {

                // If AutoDC is TRUE, then always do ResetDC

                if (PltAutoDC == FALSE)
                        ResetDC = FALSE;

                for ( jp = 0 ; jp < np ; jp++ )
                        pDCoffset[jp] = (long)(pTMPoffset[jp] / (double)na);

                }


        return( 1 );
}


int CViewApp::ShowHelp( int titleID, int textID, CWnd *caller ) {

        int iret;

        CTopicDlg dlg(titleID, textID, m_pMainWnd);
        iret = dlg.DoModal();

        if (caller)
                caller->SetFocus();

        return( iret );
}

int CViewApp::LoadHelpTitles( CListBox *lbx ) {

        int pos;

        // Load general help titles at default location

        pos = AddResourceString( lbx, IDR_HELP_MAIN_TITLE, IDR_HELP_MAIN_TEXT );
        pos = AddResourceString( lbx, IDR_HELP_VIEW_TITLE, IDR_HELP_VIEW_TEXT );
        pos = AddResourceString( lbx, IDR_HELP_DISP_TITLE, IDR_HELP_DISP_TEXT );
        pos = AddResourceString( lbx, IDR_HELP_SEQN_TITLE, IDR_HELP_SEQN_TEXT );
        pos = AddResourceString( lbx, IDR_HELP_GOTO_TITLE, IDR_HELP_GOTO_TEXT );
//        pos = AddResourceString( lbx, IDR_HELP_HELP_TITLE, IDR_HELP_HELP_TEXT );

        return( pos );
}

int CViewApp::AddResourceString( CListBox *listbox,
                                  int titleID, int textID, int pos /*=-1*/ ) {

        CString htitle;

        htitle = (LPCSTR)::LockResource(
                         ::LoadResource( NULL,
                         ::FindResource( NULL,
                            (LPCTSTR)titleID, RT_RCDATA)) );

        if (pos == -1)
                pos = listbox->AddString( htitle );
        else
                pos = listbox->InsertString( pos, htitle );

        listbox->SetItemData( pos, MAKELONG(titleID,textID) );

        return( pos );
}

int CViewApp::ShowMsg( const char *Message, const char *Title ) {

        int iret;

        CMsgDlg dlg( Message, Title, m_pMainWnd );

        iret = dlg.DoModal();

        return( iret );
}

void CViewApp::ShowError( void ) {

        char *datestr, dbuffer[16];
        char *timestr, tbuffer[16];
        time_t     current_time;
        struct tm *local_time;
        CString    MsgString;


        CViewWindow *pWnd = (CViewWindow*)m_pMainWnd;


        // Get the date and time.

        time( &current_time );
        local_time = localtime( &current_time );

        strftime( dbuffer, 16, "%m/%d/%y", local_time );
        strftime( tbuffer, 16, "%H:%M:%S", local_time );

        datestr = dbuffer;
        timestr = tbuffer;

        MsgString.Format( "Error occurred on %s at %s\n\n%s\n",
                          datestr, timestr, ErrString );
        AfxMessageBox( (LPCTSTR)MsgString );

        if (pWnd)
                pWnd->Invalidate();
}

void CViewApp::ExitError( void ) {

        TRACE0("CViewApp::ExitError\n");

        ErrCode = 1;

        AfxMessageBox( (LPCTSTR)ErrString );


        if (m_pMainWnd != NULL)
                m_pMainWnd->DestroyWindow();

        else
                AfxPostQuitMessage( 0 );


        TRACE0("CViewApp::ExitError - END\n");
}



//***********************************************************************
//
// CLASS CViewWindow
//
// This class, derived from CFrameWnd, contains the main window portion
// of the View display program.  It provides the graphics drawing, menu
// operations, and other main window functions for the CViewApp class
// which deals with the acquired data.
//

BEGIN_MESSAGE_MAP( CViewWindow, CFrameWnd )
        //{{AFX_MSG_MAP(CViewWindow)
        ON_WM_CREATE()
        ON_WM_CLOSE()
        ON_WM_SIZE()
        ON_WM_PAINT()
        ON_WM_HSCROLL()
        ON_WM_VSCROLL()
        ON_WM_DESTROY()
        ON_WM_KEYDOWN()
        ON_WM_KEYUP()
        ON_WM_MOUSEMOVE()
        ON_COMMAND(IDM_FILE_DATAFILE,       OnOpenFileDialog)
        ON_COMMAND(IDM_FILE_EXIT,           OnClose)
        ON_COMMAND(IDM_OPTIONS_DISPLAY,     OnDisplayDialog)
        ON_COMMAND(IDM_OPTIONS_SEQNAME,     OnSeqNameDialog)
        ON_COMMAND(IDM_OPTIONS_CLIP,        OnSetClip)
        ON_COMMAND(IDM_OPTIONS_AUTODC,      OnSetAutoDC)
        ON_COMMAND(IDM_OPTIONS_UPDATEDC,    OnSetDC)
        ON_COMMAND(IDM_VIEW_NEXTBUFF,       OnShowNextBuffer)
        ON_COMMAND(IDM_VIEW_PREVBUFF,       OnShowPrevBuffer)
        ON_COMMAND(IDM_VIEW_GOTOBUFF,       OnShowGotoBuffer)
        ON_COMMAND( ID_VIEW_STATUS_BAR,     OnSetStatusBar)
        ON_COMMAND(IDM_HELP_CONTENTS,       OnContentsDialog)
        ON_COMMAND(IDM_HELP_ABOUT,          OnAboutDialog)
        ON_COMMAND(IDM_HELP_DEBUG,          OnDebug)
        ON_UPDATE_COMMAND_UI(IDM_OPTIONS_CLIP,   OnUpdateClipCheck)
        ON_UPDATE_COMMAND_UI(IDM_OPTIONS_AUTODC, OnUpdateAutoDCCheck)
        //}}AFX_MSG_MAP
        ON_MESSAGE(WM_ENTERSIZEMOVE, OnEnterSizeMove)
        ON_MESSAGE(WM_EXITSIZEMOVE,  OnExitSizeMove)
END_MESSAGE_MAP()

// status line indicator
static UINT indicators[] = { ID_SEPARATOR,       // status line indicator
                             IDS_PANE_COORDS,    // for mouse coords
                             IDS_PANE_INFO       // for sps, bits, nsat, etc
                           };


CViewWindow::CViewWindow() {


        TRACE0("CViewWindow::CViewWindow\n");


        // Initialize some variables

        pYclipmin  = pYclipmax = pYlast = pYaxis = NULL;
        pPtArray   = pOldArray = NULL;
        CurrFont   = NULL;
        PenBack    = PenAxis = PenTitle = PenTrace = NULL;
        IsDragging = FALSE;


        // Initialize dialog positions

        InitDialogPositions();


        // Create and load icon, menus, accelerators, etc

        LoadFrame( IDR_VIEW, WS_OVERLAPPEDWINDOW | WS_VSCROLL | WS_HSCROLL );


        TRACE0("CViewWindow::CViewWindow - END\n");
}

CViewWindow::~CViewWindow() {

        TRACE0("CViewWindow::~CViewWindow\n");
}

BOOL CViewWindow::PreCreateWindow(CREATESTRUCT& cs) {

        TRACE0("CViewWindow::PreCreateWindow\n");

        // Modify default size

        cs.x  = GUTTER;
        cs.y  = GUTTER;
        cs.cx = pAPP->DefaultScreenWidth  - GUTTER - cs.x;
        cs.cy = pAPP->DefaultScreenHeight - GUTTER - cs.y - HSTARTBAR;

        return( CFrameWnd::PreCreateWindow(cs) );

}

void CViewWindow::PostNcDestroy() {

        TRACE0("CViewWindow::PostNcDestroy\n");

        // call parent to delete "this"

        CFrameWnd::PostNcDestroy();

}

int CViewWindow::OnCreate(LPCREATESTRUCT lpCreateStruct) {

        TRACE0("CViewWindow::OnCreate\n");

        int nsize, yppi, apoint, PaneSize;
        UINT PaneId, PaneStyle;


        // Create parent class

        if (CFrameWnd::OnCreate(lpCreateStruct) == -1) {
                pAPP->ErrString = "CFrameWnd creation failed.\n";
                pAPP->ExitError();
                return -1;
                }

        // Add status bar

        if (!m_wndStatusBar.Create(this) ||
                !m_wndStatusBar.SetIndicators(indicators,
                  sizeof(indicators)/sizeof(UINT))) {

                pAPP->ErrString = "Status bar creation failed.\n";
                pAPP->ExitError();
                return -1;      // fail to create
                }

        // Make first pane a fixed size

        m_wndStatusBar.GetPaneInfo( PANE_MESSAGE, PaneId, PaneStyle, PaneSize );
        PaneStyle = (PaneStyle & ~SBPS_STRETCH);
        m_wndStatusBar.SetPaneInfo( PANE_MESSAGE, PaneId, PaneStyle, PaneSize );


        // Make second pane a fixed size too

        PaneStyle = SBPS_NORMAL;
        m_wndStatusBar.SetPaneInfo( PANE_COORDS, IDS_PANE_COORDS, PaneStyle, 120 );
        m_wndStatusBar.SetPaneText( PANE_COORDS, "( , )", TRUE );


        // Make last pane stretchy

        PaneStyle = SBPS_NORMAL | SBPS_STRETCH;
        m_wndStatusBar.SetPaneInfo( PANE_INFO, IDS_PANE_INFO, PaneStyle, 400 );
        m_wndStatusBar.SetPaneText( PANE_INFO, "View info", TRUE );


        // Set default colors, create pens

        SetColors();

        CreatePens();



        // Compute some important point sizes (2,8,20) as pixels

        CClientDC dc(this);

        yppi = dc.GetDeviceCaps( LOGPIXELSY );  // Y Pixels Per logical Inch

        minPoint = (yppi *  2) / 72;
        midPoint = (yppi *  8) / 72;
        maxPoint = (yppi * 20) / 72;

        apoint   = (yppi * 16) / 72 - 1;




        // Set up a font

        font.lfHeight         = 0;      // apoint;
        font.lfWidth          = 0;
        font.lfEscapement     = 0;
        font.lfOrientation    = 0;
        font.lfWeight         = FW_NORMAL; // tried FW_BOLD;
        font.lfItalic         = FALSE;
        font.lfUnderline      = FALSE;
        font.lfStrikeOut      = FALSE;
        font.lfCharSet        = ANSI_CHARSET;
        font.lfOutPrecision   = OUT_DEFAULT_PRECIS;
        font.lfClipPrecision  = CLIP_DEFAULT_PRECIS;
        font.lfQuality        = DEFAULT_QUALITY;
        font.lfPitchAndFamily = FIXED_PITCH | FF_MODERN; // was VARIABLE_PITCH | FF_SWISS;

        SrStringCopy(font.lfFaceName, "CourierNew", LF_FACESIZE); // was Arial





        // Initialize plot arrays and variables

        LastPtsPlot = 0;
        pPtArray    = NULL;
        pOldArray   = NULL;




        // Allocate vertical plot position buffers

        nsize = pAPP->MaxTotalPlotChannel * sizeof( int );

        pYlast    = (int*) malloc( nsize );
        pYaxis    = (int*) malloc( nsize );
        pYclipmin = (int*) malloc( nsize );
        pYclipmax = (int*) malloc( nsize );


        if (!pYlast || !pYaxis || !pYclipmin || !pYclipmax) {

                pAPP->ErrString = "Could not allocate vertical position buffers.";
                pAPP->ExitError();
                return -1;  // failed create
                }




        // If status bar is to be off, call OnBarCheck to toggle it
        // since the status bar starts off visible.
        // (See UpdateStatusBar for a way to set it rather than toggle it)

        if (!pAPP->DoStatusBar)
                CFrameWnd::OnBarCheck( AFX_IDW_STATUS_BAR );



        // Compute plot positions (and allocate plot buffers if needed)

        if (!ComputePlotPositions())
                return -1;      // failed create



        // Set scroll bar ranges

        SetScrollBar( SB_BOTH );

        controlkey = FALSE;



        // Set title and pane

        SetWindowTitle( );
        SetPaneString( );


        // If data is already available, then display it

        if (pAPP->DataFormat != FILEFMT_NONE) {
                pAPP->DataCurrentPt = 0;
                UpdateData();
                }

        return 0;
}

void CViewWindow::OnSize(UINT nType, int cx, int cy) {

        TRACE0("CViewWindow::OnSize\n");

        CFrameWnd::OnSize(nType, cx, cy);

        if (!ComputePlotPositions())
                return;

}

void CViewWindow::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) {

        TRACE0("CViewWindow::OnHScroll\n");

        int first, line, page, maxstart;

        first    = pAPP->PltFirstSample;
        maxstart = pAPP->NsamplesPlot - TraceWidth - 1;
        page     = TraceWidth;
        line     = maxstart / 20;

        if (line < 5)
                line = 5;
        if (maxstart < 0)
                maxstart = 0;


        switch (nSBCode) {

                case SB_PAGEUP:
                        first -= page;
                        break;

                case SB_PAGEDOWN:
                        first += page;
                        break;

                case SB_LINEUP:
                        first -= line;
                        break;

                case SB_LINEDOWN:
                        first += line;
                        break;

                case SB_TOP:
                        first = 0;
                        break;

                case SB_BOTTOM:
                        first = maxstart;
                        break;

                case SB_THUMBPOSITION:
                        first = nPos;
                        break;
                }


        // Ensure range is valid

        if (first < 0)
                first = 0;

        if (first > maxstart)
                first = maxstart;




        // Update if changed

        if (first != pAPP->PltFirstSample) {

                SetScrollPos( SB_HORZ, first, TRUE );

                pAPP->PltFirstSample = first;

                Invalidate();

                }


}

void CViewWindow::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) {

        TRACE0("CViewWindow::OnVScroll\n");

        int first, maxstart;

        first    = pAPP->PltFirstTrace;
        maxstart = pAPP->NchannelsPlot - pAPP->PltNdisplay;

        switch (nSBCode) {

                case SB_PAGEUP:
                        first -= pAPP->PltNdisplay;
                        break;

                case SB_PAGEDOWN:
                        first += pAPP->PltNdisplay;
                        break;

                case SB_LINEUP:
                        first--;
                        break;

                case SB_LINEDOWN:
                        first++;
                        break;

                case SB_TOP:
                        first = 0;
                        break;

                case SB_BOTTOM:
                        first = maxstart;
                        break;

                case SB_THUMBPOSITION:
                        first = nPos;
                        break;
                }


        // Ensure range is valid

        if (first < 0)
                first = 0;

        if (first > maxstart)
                first = maxstart;



        // Update if changed

        if (first != pAPP->PltFirstTrace) {

                SetScrollPos( SB_VERT, first, TRUE );

                pAPP->PltFirstTrace = first;
                pAPP->ResetPaint        = TRUE;

                Invalidate();

                }


}

void CViewWindow::OnPaint() {

        TRACE0("CViewWindow::OnPaint\n");

        // No paint when minimized

        if ( IsIconic() )
                return;


        // Delay paint when dragging for a move or resize

        if ( IsDragging ) {
                PaintDeferred = TRUE;
                return;
                }



        // Redraw plot

        CPaintDC dc(this); // device context for painting

        DrawBackground( &dc );

        DrawTitles( &dc );

        DrawScales( &dc );

        DrawAxes( &dc );




        // Redraw traces

        pAPP->ResetPaint = TRUE;

        if (pAPP->IsAllocated)
                DrawTraces( &dc );



        // Do not call CFrameWnd::OnPaint() for painting messages
}

LRESULT CViewWindow::OnEnterSizeMove( WPARAM wParam, LPARAM lParam ) {

        TRACE0("CViewWindow::OnEnterSizeMove\n");

        // Prevent any painting while dragging for move/resize

        IsDragging = TRUE;
        return LRESULT(0);
}


LRESULT CViewWindow::OnExitSizeMove( WPARAM wParam, LPARAM lParam ) {

        TRACE0("CViewWindow::OnExitSizeMove\n");

        // Drag for move/resize is done, so allow painting and
        // force a paint if any have been skipped during the drag.

        IsDragging = FALSE;
        if (PaintDeferred) {
                PaintDeferred = FALSE;
                Invalidate(FALSE);
        }
        return LRESULT(0);
}


void CViewWindow::SetClipLimits( int height, int traceHalf ) {

        TRACE0("CViewWindow::SetClipLimits\n");

        int i;

        if (pAPP->PltClipping)
                for ( i = 0 ; i < pAPP->MaxTotalPlotChannel ; i++ ) {
                        pYclipmin[i] = pYaxis[i] - traceHalf;
                        pYclipmax[i] = pYaxis[i] + traceHalf;
                        }

        else
                for ( i = 0 ; i < pAPP->MaxTotalPlotChannel ; i++ ) {
                        pYclipmin[i] = 0;
                        pYclipmax[i] = height-1;
                        }
}


void CViewWindow::DrawBackground( CDC *pDC ) {

        TRACE0("CViewWindow::DrawBackground\n");

        int height, width;

        GetSize( &height, &width );

        pDC->FillSolidRect( 0, 0, width+1, height+1, ColorBack );
}


void CViewWindow::DrawTitles( CDC *pDC ) {

        TRACE0("CViewWindow::DrawTitles\n");

        int i, ch2, itrace;


        // clear title area


        // draw titles

        pDC->SelectObject( CurrFont );
        pDC->SetTextColor( ColorTitle );
        pDC->SetBkMode(TRANSPARENT);

        ch2    = cyChar/2;
        itrace = pAPP->PltFirstTrace;

        for ( i = 0 ; i < pAPP->PltNdisplay ; i++  ) {

                pDC->TextOut( HMARGIN, pYaxis[i]-ch2,
                             &(pAPP->pPlotTitle[itrace*MAXTITLE]) );
                itrace++;
                }

}


void CViewWindow::DrawScales( CDC *pDC ) {

        TRACE0("CViewWindow::DrawScales\n");

        int i, j, k, xpos, nthick, nax, yax[2], sign[2];
        int tic, xinc, ticsm, xincsm, nsmall;


        pDC->SelectObject( PenAxis );


        // Draw extra top and bottom axes with tics

        yax[0] = YScaleTop;     sign[0] =  1;
        yax[1] = YScaleBottom;  sign[1] = -1;
        nax=2;

        nthick   = 3;
        nsmall   = 4;

        xinc     = 100;
        xincsm   = xinc / (nsmall+1);

        tic      = VTICK;
        ticsm    = tic  / 2;


        for ( i = 0 ; i < nax ; i++ ) {

                // Draw line

                pDC->MoveTo( TraceStart, yax[i] );
                pDC->LineTo( TraceEnd+1, yax[i] );


                // Draw tics

                xpos = TraceStart;
                while ( xpos < TraceEnd+1 ) {

                        // Large tic

                        for ( j = 0 ; j < nthick ; j++ ) {

                                pDC->MoveTo( xpos+j, yax[i] );
                                pDC->LineTo( xpos+j, yax[i]+sign[i]*tic );

                                } // end for j < nthick

                        // Small tics

                        for ( k = 0 ; k < nsmall ; k++ ) {

                                xpos += xincsm;

                                for ( j = 0 ; j < nthick ; j++ ) {

                                        pDC->MoveTo( xpos+j, yax[i] );
                                        pDC->LineTo( xpos+j, yax[i]+sign[i]*ticsm );

                                        } // end for j < nthick

                                } // end for k < nsmall

                        xpos += xincsm;

                        } // end while

        } // end for i > nax

}


void CViewWindow::DrawAxes( CDC *pDC ) {

        TRACE0("CViewWindow::DrawAxes\n");

        int i;


        pDC->SelectObject( PenAxis );


        // Draw horizontal trace axis lines

        for ( i = 0 ; i < pAPP->PltNdisplay ; i++ ) {

                pDC->MoveTo( TraceStart, pYaxis[i] );
                pDC->LineTo( TraceEnd+1, pYaxis[i] );

                }
}


void CViewWindow::DrawTraces( CDC *pDC ) {

        TRACE0("CViewWindow::DrawTraces\n");

        int npoints, j, k, kend, height, width;



        // Correct number of points based on TraceWidth and max plot points

        kend = pAPP->PltFirstSample + TraceWidth;

        if (kend > pAPP->NsamplesPlot)
                kend = pAPP->NsamplesPlot;

        npoints = kend - pAPP->PltFirstSample;




        // Fill plot arrays from data arrays with clipping

        FillTraces( npoints );




        // Blankout and draw new traces

        k = 0;

        if (pAPP->ResetPaint == TRUE) {

                pAPP->ResetPaint = FALSE;

                GetSize( &height, &width );

                pDC->FillSolidRect(TraceStart,0,TraceWidth,height,ColorBack);

                DrawScales( pDC );

                DrawAxes( pDC );

                pDC->SelectObject( PenTrace );

                for ( j = 0 ; j < pAPP->PltNdisplay ; j++ ) {

                        pDC->Polyline( &(pPtArray[k]), npoints );

                        k += npoints;

                        }
                }



        // Or erase old and draw new traces

        else {

                for ( j = 0 ; j < pAPP->PltNdisplay ; j++ ) {

                        pDC->SelectObject( PenBack );
                        pDC->Polyline( &(pOldArray[k]), npoints );


                        pDC->SelectObject( PenAxis );
                        pDC->MoveTo( TraceStart, pYaxis[j] );
                        pDC->LineTo( TraceEnd+1, pYaxis[j] );


                        pDC->SelectObject( PenTrace );
                        pDC->Polyline( &(pPtArray[k]), npoints );

                        k += npoints;

                        }

                DrawScales( pDC );

                }

}

void CViewWindow::FillTraces( int npoints ) {

        TRACE0("CViewWindow::FillTraces\n");

        int kstart, jstart, jend, jtrace, tracehalf;
        int i, j, k, kdata, joff, xpos, xval;
        int ycurr, yprev, yval, yscaled, yax, yclipmin, yclipmax;



        // Prepare point array with all points (some new and some old)

        k      = 0;
        jtrace = pAPP->PltFirstTrace;
        kstart = pAPP->PltFirstSample;

        if (kstart > pAPP->NsamplesPlot)
                kstart -= pAPP->NsamplesPlot;

        tracehalf = TraceHeight / 2;

        for ( j = 0 ; j < pAPP->PltNdisplay ; j++ ) {

                xpos     = TraceStart;
                yprev    = 0;
                yax      = pYaxis[j];
                yclipmin = pYclipmin[j];
                yclipmax = pYclipmax[j];

                joff     = pAPP->pDCoffset[jtrace];
                jstart   = jtrace*pAPP->NsamplesPlot;
                jend     = jstart + pAPP->NsamplesPlot;
                kdata    = jstart + kstart;



                for ( i = 0 ; i < npoints ; i++ ) {

                    // Fill old trace points

                    pOldArray[k].x = pPtArray[k].x;
                    pOldArray[k].y = pPtArray[k].y;

                    // Process analog channels

                    // Note: GpsCounter channel should never be turned on,
                    // since we prohibit it when setting up the plot array.

                    if ( (pAPP->pPlotChType[jtrace] == CH_TYPE_ANALOG) ||
                         (pAPP->pPlotChType[jtrace] == CH_TYPE_GPSCOUNTER) ) {

                        // Remove DC offset

                        yval = (pAPP->pDemuxArray[kdata] - joff);


                        // Scale, rounding towards zero

                        if (yval >= 0)
                                yscaled = yax - (yval >> pAPP->PltShiftBy);
                        else
                                yscaled = yax + (-yval >> pAPP->PltShiftBy);


                        // Set x value

                        xval = xpos;

                        } // end analog and gps counter channels


                    // Process digital and GPS mark channels

                    else { // CH_TYPE_DIGITAL || CH_TYPE_GPSMARK

                            // Set digital y value to zero or half height

                            if (pAPP->pDemuxArray[kdata] == 0) {
                                    ycurr   = 0;
                                    yscaled = yax;
                                    }
                            else {
                                    ycurr   = 1;
                                    yscaled = yax - tracehalf;
                                    }


                            // Set or correct x values to avoid sloping lines

                            if (ycurr == yprev) {      // flat section
                                    xval = xpos;
                                    }
                            else if (ycurr > yprev) {  // step up
                                    xval = xpos;
                                    if (i != 0)
                                            pPtArray[k-1].x = xpos;
                                    }
                            else { // ycurr < yprev    // step down
                                    xval = xpos-1;
                                    }

                            yprev = ycurr;

                        } // end digital and GPS mark channels



                    // Clip

                    if (yscaled < yclipmin)
                            yscaled = yclipmin;

                    else if (yscaled > yclipmax)
                            yscaled = yclipmax;



                    // Fill new trace points

                    pPtArray[k].x = xval;
                    pPtArray[k].y = yscaled;


                    // Move to next point

                    xpos++;
                    k++;
                    kdata++;
                    if (kdata >= jend)
                            kdata = jstart;

                    } // end for i < npoints

                jtrace++;

                } // end for j < PltNdisplay

}


void CViewWindow::OnDestroy() {

        TRACE0("CViewWindow::OnDestroy\n");


        // Clean up pens

        DeletePens();



        // Clean up last font

        DeleteFont();



        // Clean up plot arrays

        DeletePlotArrays();



        // Clean up vertical plot position buffers

        DeleteVerticalBuffers();





        CFrameWnd::OnDestroy();
}

void CViewWindow::OnMouseMove( UINT nFlags, CPoint point ) {

        CString PaneString;
        char    *d;
        int     i, x, t, y, traceHalf;

        traceHalf = TraceHeight / 2;

        // Determine which trace we are closest to

        t = pAPP->PltNdisplay - 1;

        for ( i = 0 ; i < pAPP->PltNdisplay ; i++ ) {
                if ( point.y < pYaxis[i] - traceHalf ) {
                        t = i - 1;
                        break;
                        }
                }

        if ( t < 0 )
                t = 0;


        // Determine x, y, trace

        x  = point.x - TraceStart;
        if ( x < 0 )
                x = 0;
        else if ( x > TraceWidth )
                x = TraceWidth;

        d  = ">";
        y  = pYaxis[t] - point.y;
        if (y < 0) {
                d = "<";
                y = -y;
                }

        t += pAPP->PltFirstTrace;

        PaneString.Format( "( %04d, %03d %1s %-2d )", x, y, d, t );

        m_wndStatusBar.SetPaneText( PANE_COORDS, (LPCTSTR)PaneString, TRUE );

}

void CViewWindow::OnOpenFileDialog() {

        TRACE0("CViewWindow::OnOpenFileDialog\n");

        CString szFilter, tmpDataName, tmpHdrName, tmpOpenDir;
        int     tmpFmt;
        char path[_MAX_PATH], drive[_MAX_DRIVE], dir[_MAX_DIR], ext[_MAX_EXT];


        // General organization of this function is:
        //
        //   Bring up file dialog for selecting data file
        //      (choices not saved yet)
        //   Determine data format based on suffix
        //   If using .out files
        //      Bring up file dialog for selecting hdr file
        //      Save Hdr choice
        //   Now save data choice and update info for display



        // Create data file dialog and set initial directory

        szFilter  =  "Dat Files (*.dat)|*.dat";
        szFilter += "|Out Files (*.out)|*.out";
        szFilter += "|All Files (*.*)|*.*||";

        CFileDialog datadlg( TRUE, NULL, (LPCTSTR)pAPP->DataName,
                             OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_LONGNAMES,
                             szFilter );

        datadlg.m_ofn.lpstrInitialDir = (LPCTSTR)pAPP->OpenDir;
        datadlg.m_ofn.lpstrTitle = (LPCTSTR)"Open Data File";

        if (pAPP->DataFormat == FILEFMT_DAT)
                datadlg.m_ofn.nFilterIndex = 1;
        else if (pAPP->DataFormat == FILEFMT_OUT)
                datadlg.m_ofn.nFilterIndex = 2;
        else
                datadlg.m_ofn.nFilterIndex = 1; // default to .dat



        // Display file dialog

        if (datadlg.DoModal() != IDOK)
                return;


        // Retrieve the newly selected file name and its path

        tmpDataName = datadlg.GetPathName();
        _splitpath( tmpDataName, drive, dir, NULL, ext );
        _makepath( path, drive, dir, NULL, NULL );
        tmpOpenDir = path;


        // Validate data file

        if ( !SrFileReadable( tmpDataName ) ) {
                pAPP->ErrString.Format( "File %s must exist and be readable.",
                                        tmpDataName );
                pAPP->ShowError();
                return;
                }



        // Set data format based on file extension

        if ( _stricmp( ext, ".OUT" ) == 0 )
                tmpFmt = FILEFMT_OUT;
        else if ( _stricmp( ext, ".DAT" ) == 0 )
                tmpFmt = FILEFMT_DAT;
        else
                tmpFmt = FILEFMT_OTHER;



        // Make sure file type is consistent with file name
        // (ie 00000000.out is not valid when looking for .dat files)

        if ((tmpFmt == FILEFMT_DAT && datadlg.m_ofn.nFilterIndex == 2) ||
            (tmpFmt == FILEFMT_OUT && datadlg.m_ofn.nFilterIndex == 1) ) {
                if (datadlg.m_ofn.nFilterIndex == 1)
                        pAPP->ErrString.Format( "File type of .DAT and " );
                else //  (datadlg.m_ofn.nFilterIndex == 2)
                        pAPP->ErrString.Format( "File type of .OUT and " );
                pAPP->ErrString += "file name of \n";
                pAPP->ErrString += tmpDataName;
                pAPP->ErrString += " are inconsistent.\nPlease try again.\n";
                pAPP->ShowError();
                return;
                }


        // Out files also need a separate header

        if (tmpFmt == FILEFMT_OUT) {

                // Create hdr file dialog and set initial directory

                if ( pAPP->HdrName.IsEmpty() )
                        pAPP->HdrName = "SCOPE.HDR";

                szFilter = "Header Files (*.hdr)|*.hdr|All Files (*.*)|*.*||";

                CFileDialog hdrdlg( TRUE, NULL, (LPCTSTR)pAPP->HdrName,
                                    OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_LONGNAMES,
                                    szFilter );

                hdrdlg.m_ofn.lpstrInitialDir = (LPCTSTR)tmpOpenDir;
                hdrdlg.m_ofn.lpstrTitle = (LPCTSTR)"Open Header File (Required for .out file processing)";



                // Display file dialog

                if (hdrdlg.DoModal() != IDOK)
                        return;



                // Retrieve the newly selected hdr file name and its path

                tmpHdrName = hdrdlg.GetPathName();
                _splitpath( tmpHdrName, drive, dir, NULL, NULL );
                _makepath( path, drive, dir, NULL, NULL );
                tmpOpenDir = path;


                // Validate hdr file

                if ( !SrFileReadable( tmpHdrName ) ) {
                        pAPP->ErrString.Format( "File %s must exist and be readable.",
                                                tmpHdrName );
                        pAPP->ShowError();
                        return;
                        }
                }



        // Save data file info now that Hdr dialog didn't return with cancel

        pAPP->OpenDir = tmpOpenDir;

        if ( !pAPP->SetFileInfo( tmpFmt, tmpDataName, tmpHdrName ) )
                pAPP->ShowError();



        // Update for possible changes to display

        CClientDC dc(this);
        ComputeMaxTitleLength( &dc );
        if ( ! ComputePlotPositions() ) {
                pAPP->ErrString = "Failed to compute plot positions\n";
                pAPP->ShowError( );
                return;
                }
        SetScrollBar( SB_BOTH );
        Invalidate();


        // Start from beginning of data

        pAPP->DataCurrentPt = 0;
        UpdateData();



        controlkey = FALSE;
}

int CViewWindow::UpdateData() {

        TRACE0("CViewWindow::UpdateData\n");

        int     i, j, FileSeq, FileRec, FilePt, Nread, Nremain, Nstart, Nvalid;
        int     DataEndPt, EndSeq, EndRec, EndPt;
        CString CurrFullName;


        // Determine location of current data.  Find both the location for the
        // start of the first buffer and the start of the last buffer to be
        // displayed.

        DataEndPt = pAPP->DataCurrentPt + (pAPP->Nplotbuffers-1) * pAPP->DataPtsPerBuff;

        pAPP->GetFilePosForPt( pAPP->DataCurrentPt, &FileSeq, &FileRec, &FilePt );
        pAPP->GetFilePosForPt( DataEndPt, &EndSeq, &EndRec, &EndPt );


        // Verify which of the sequential files required to display
        // this data exist.

        if (pAPP->DataDoSeq) {

                for ( i = FileSeq ; i < EndSeq + 1 ; i += pAPP->DataSeqInc ) {

                        if (pAPP->DataSeqType == SEQTYPE_DEC)
                                CurrFullName.Format( "%s%08d%s", pAPP->DataPath,
                                                     i, pAPP->DataExt );
                        else // (pAPP->DataSeqType == SEQTYPE_HEX)
                                CurrFullName.Format( "%s%08X%s", pAPP->DataPath,
                                                     i, pAPP->DataExt );

                        if ( !SrFileReadable( CurrFullName ) )
                                break;

                        } // end for i

                EndSeq = i - pAPP->DataSeqInc;         // Set to last existing file

                if (EndSeq < FileSeq) { // We don't have even the first file
                        sprintf( pAPP->szTemp,
                                 "File %s\ndoes not exist or can not be read.\nCheck sequential naming mode.",
                                 (LPCTSTR)CurrFullName );
                        pAPP->ShowMsg( pAPP->szTemp, "Scroll Message" );
                        return( 0);
                        }

                } // end if DataDoSeq


        //  Set display file name

        if (pAPP->DataDoSeq) {
                if (pAPP->DataSeqType == SEQTYPE_DEC)
                        pAPP->DataCurrName.Format( "%08d%s", FileSeq, pAPP->DataExt );
                else // (pAPP->DataSeqType == SEQTYPE_HEX)
                        pAPP->DataCurrName.Format( "%08X%s", FileSeq, pAPP->DataExt );
                }
        else
                pAPP->DataCurrName = pAPP->DataName;


        // Read and demux current data.  Note: The loop is to compensate
        // for the times when more than one file must be read to get the
        // requested amount data.  NOTE: Nremain and Nstart here are
        // numbers of points per one channel.

        Nremain = pAPP->NsamplesPlot;
        Nstart  = 0;
        while (Nremain > 0) {
                Nread = pAPP->ReadDataFromFile( FileSeq, FileRec, FilePt, Nremain, Nstart );
                if (Nread == 0) // no data read
                        break;
                Nremain -= Nread;
                Nstart  += Nread;
                if (!pAPP->DataDoSeq) // Don't keep reading if not doing seq
                        break;
                FileSeq += pAPP->DataSeqInc;
                FileRec = 0;
                FilePt  = 0;
                if (EndSeq < FileSeq ) // Stop if no more files exist
                        break;
                }

        // If we didn't get all the data we asked for, just pad the rest
        // of the points out with the last known value for that channel.
        // Note that Nstart must be changed so it gives the start point
        // based on all channels and not just one.  Nvalid is the last
        // valid point for channel 0.

        if (Nremain > 0) {
                Nstart *= pAPP->Nchannels;
                Nvalid  = Nstart - pAPP->Nchannels;
                for ( i = 0 ; i < Nremain ; i++ )
                        for ( j = 0 ; j < pAPP->Nchannels ; j++ )
                                pAPP->pDataArray[Nstart++] = pAPP->pDataArray[Nvalid+j];
                }
        pAPP->DemuxData();



        // Draw traces

        CClientDC dc(this);
        DrawTraces( &dc );
        SetWindowTitle();
        SetPaneString();

        return( 1 );
}

void CViewWindow::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) {

        TRACE0("CViewWindow::OnKeyUp\n");


        // Keep track of control key

        if (nChar == VK_CONTROL)

                controlkey = FALSE;




        // Default handling

        CFrameWnd::OnKeyUp(nChar, nRepCnt, nFlags);
}

void CViewWindow::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) {

        TRACE0("CViewWindow::OnKeyDown\n");

        int scancode;




        // Keep track of control key

        if (nChar == VK_CONTROL)

                controlkey = TRUE;



        // Translate scan codes into a reliable virtkey value

        scancode = nFlags & 0x00FF;

        if ( (scancode == 0x0C) || (scancode == 0x33) ) // -_ or ,< keys
                nChar = VK_SUBTRACT;

        if ( (scancode == 0x0D) || (scancode == 0x34) ) // =+ or .> keys
                nChar = VK_ADD;




        switch (nChar) {

            // Increase amplitude (decrease shift)

            case 0xBB:          // =+ key
            case 0xBE:          // .> key
            case VK_ADD:        // numpad + key
            case VK_UP:         // up arrow key

                pAPP->PltShiftBy -= nRepCnt;

                if (pAPP->PltShiftBy < pAPP->MinAmpShift)
                        pAPP->PltShiftBy = pAPP->MinAmpShift;

                UpdateTitleTraces();

                break;




            // Decrease amplitude (increase shift)

            case 0xBD:          // -_ key
            case 0xBC:          // ,< key
            case VK_SUBTRACT:   // numpad - key
            case VK_DOWN:       // down arrow key

                pAPP->PltShiftBy += nRepCnt;

                if (pAPP->PltShiftBy > pAPP->MaxAmpShift-1)
                        pAPP->PltShiftBy = pAPP->MaxAmpShift-1;

                UpdateTitleTraces();

                break;





            // Process arrow keys for scrolling

            case VK_LEFT:
                OnShowPrevBuffer();
                break;

            case VK_RIGHT:
                OnShowNextBuffer();
                break;


// This was old use of arrow keys, now the up and down arrows are
// for changing amplitude and the left and right arrows are for
// scrolling between data files
//
//            case VK_UP:
//                SendMessage(WM_VSCROLL, MAKELONG(SB_LINEUP, 0), 0L);
//                break;
//            case VK_DOWN:
//                SendMessage(WM_VSCROLL, MAKELONG(SB_LINEDOWN, 0), 0L);
//                break;
//            case VK_LEFT:
//                SendMessage(WM_HSCROLL, MAKELONG(SB_LINEUP, 0), 0L);
//                break;
//            case VK_RIGHT:
//                SendMessage(WM_HSCROLL, MAKELONG(SB_LINEDOWN, 0), 0L);
//                break;



            // Process page up/down keys for scrolling

            case VK_PRIOR:    //PAGEUP key

                if (controlkey)
                        SendMessage(WM_HSCROLL, MAKELONG(SB_PAGEUP, 0), 0L);
                else
                        SendMessage(WM_VSCROLL, MAKELONG(SB_PAGEUP, 0), 0L);
                break;

            case VK_NEXT:     // PAGEDOWN key

                if (controlkey)
                        SendMessage(WM_HSCROLL, MAKELONG(SB_PAGEDOWN, 0), 0L);
                else
                        SendMessage(WM_VSCROLL, MAKELONG(SB_PAGEDOWN, 0), 0L);
                break;




            // Process home/end keys for scrolling

            case VK_HOME:

                if (controlkey)
                        SendMessage(WM_VSCROLL, MAKELONG(SB_TOP, 0), 0L);
                else
                        SendMessage(WM_HSCROLL, MAKELONG(SB_TOP, 0), 0L);
                break;

            case VK_END:

                if (controlkey)
                        SendMessage(WM_VSCROLL, MAKELONG(SB_BOTTOM, 0), 0L);
                else
                        SendMessage(WM_HSCROLL, MAKELONG(SB_BOTTOM, 0), 0L);
                break;



         }




        // Default handling

        CFrameWnd::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CViewWindow::OnClose() {

        TRACE0("CViewWindow::OnClose\n");

        CFrameWnd::OnClose();
}

void CViewWindow::OnDisplayDialog() {

        TRACE0("CViewWindow::OnDisplayDialog\n");

        int         oldColorScheme, oldNdisplay, oldClipping;
        CDisplayDlg dlg(this);

        oldColorScheme = pAPP->PltColorScheme;
        oldNdisplay    = pAPP->PltNdisplay;
        oldClipping    = pAPP->PltClipping;

        if (dlg.DoModal() == IDOK) {

                if (oldColorScheme != pAPP->PltColorScheme)
                        UpdateColorScheme();
                if (oldNdisplay != pAPP->PltNdisplay) {
                        pAPP->ComputeBufferSizes();
                        if (!ComputePlotPositions())
                                return;
                        }
                if (oldClipping != pAPP->PltClipping) {
                        if (!ComputePlotPositions())
                                return;
                        }

                SetScrollBar( SB_BOTH );

                Invalidate();                    // cause repaint
                SetWindowTitle();
                SetPaneString();
                }

        controlkey = FALSE;
}


void CViewWindow::OnSeqNameDialog() {

        TRACE0("CViewWindow::OnSeqNameDialog\n");

        int oldSeqType;

        oldSeqType = pAPP->DataSeqType;

        CSeqNameDlg dlg(this);

        if (dlg.DoModal() == IDOK) {

                // Update seq home number if type changed

                if (pAPP->DataSeqType != oldSeqType) {
                        if (oldSeqType == SEQTYPE_DEC)
                                sprintf( pAPP->szTemp, "%ld", pAPP->DataSeqHome );
                        else // (oldSeqType == SEQTYPE_HEX)
                                sprintf( pAPP->szTemp, "%lx", pAPP->DataSeqHome );

                        if (pAPP->DataSeqType == SEQTYPE_DEC)
                                sscanf( pAPP->szTemp, "%ld", &pAPP->DataSeqHome );
                        else // (pAPP->DataSeqType == SEQTYPE_HEX)
                                sscanf( pAPP->szTemp, "%lx", &pAPP->DataSeqHome );
                        }

                SetWindowTitle();
                SetPaneString();
                }

        controlkey = FALSE;
}


void CViewWindow::OnSetClip() {

        TRACE0("CViewWindow::OnSetClip\n");

        int height, width;

        pAPP->PltClipping = !pAPP->PltClipping;

        GetSize( &height, &width );

        SetClipLimits( height, (pYaxis[1]-pYaxis[0])/2 );

//      Used to call UpdateTitleTraces, but that didn't do
//      the trace draw when running.  This caused bad redraw
//      behavior when clipping was changed via the pulldown
//      menu and held longer than one buffer.

        SetWindowTitle( );
        SetPaneString();
        if (pAPP->IsAllocated) {
                CClientDC dc(this);
                DrawTraces( &dc );
                }
}

void CViewWindow::OnSetDC() {

        TRACE0("CViewWindow::OnSetDC\n");

        pAPP->ResetDC = TRUE;

        MessageBeep(MB_ICONASTERISK);


       // One beep is enough for Win2K/XP

#if defined( SROS_WIN9X ) || defined( SROS_WIN98 )
        MessageBeep(MB_ICONASTERISK);
        MessageBeep(MB_ICONASTERISK);
#endif
}

void CViewWindow::OnSetAutoDC() {

        TRACE0("CViewWindow::OnSetAutoDC\n");

        pAPP->PltAutoDC = !pAPP->PltAutoDC;
        pAPP->ResetDC = TRUE;

        SetWindowTitle();
        SetPaneString();
}

void CViewWindow::OnShowNextBuffer() {

        TRACE0("CViewWindow::OnShowNextBuffer\n");

        // Increment to next data buffer position

        pAPP->DataCurrentPt += pAPP->DataPtsPerBuff;


        // Enforce limits when working with single file

        if (!pAPP->DataDoSeq) {
                if (pAPP->DataCurrentPt > pAPP->DataMaxPt) {
                        pAPP->DataCurrentPt = pAPP->DataMaxPt;
                        pAPP->ErrString = "End of data reached\n";
                        pAPP->ShowError( );
                        }
                }

        // Restore last pt value if we can't show the new one

        if ( !UpdateData() )
                pAPP->DataCurrentPt -= pAPP->DataPtsPerBuff;
}

void CViewWindow::OnShowPrevBuffer() {

        TRACE0("CViewWindow::OnShowPrevBuffer\n");

        // Increment to previous data buffer position

        pAPP->DataCurrentPt -= pAPP->DataPtsPerBuff;


        // Enforce limits when working with single file

        if (!pAPP->DataDoSeq) {
                if (pAPP->DataCurrentPt < pAPP->DataMinPt) {
                        pAPP->DataCurrentPt = pAPP->DataMinPt;
                        pAPP->ErrString = "Start of data reached\n";
                        pAPP->ShowError( );
                        }
                }

        // Restore last pt value if we can't show the new one

        if ( !UpdateData() )
                pAPP->DataCurrentPt += pAPP->DataPtsPerBuff;
}

void CViewWindow::OnShowGotoBuffer() {

        TRACE0("CViewWindow::OnShowGotoBuffer\n");

        int OldCurrentPt;


        OldCurrentPt = pAPP->DataCurrentPt;

        CGotoPtDlg dlg( this );

        if (dlg.DoModal() == IDOK) {

                // This set pAPP->DataCurrentPt to a new value
                // so go to selected data buffer position

                // Restore last pt value if we can't show the new one

                if ( !UpdateData() )
                        pAPP->DataCurrentPt = OldCurrentPt;
                }
}

void CViewWindow::OnSetStatusBar() {

        TRACE0("CViewWindow::OnSetStatusBar\n");

        pAPP->DoStatusBar = !pAPP->DoStatusBar;

        UpdateStatusBar();
}

void CViewWindow::UpdateStatusBar() {

        TRACE0("CViewWindow::UpdateStatusBar\n");

        CControlBar* pBar;

        // Update status bar

        pBar = GetControlBar( AFX_IDW_STATUS_BAR );
        if (pBar != NULL)
                ShowControlBar( pBar, pAPP->DoStatusBar, FALSE );


        if (!ComputePlotPositions())
                return;
        Invalidate();
}

void CViewWindow::UpdateTitleTraces() {

        TRACE0("CViewWindow::UpdateTitleTraces\n");

        // Update title

        SetWindowTitle();
        SetPaneString();


        // Update traces

        if (pAPP->IsAllocated) {
                CClientDC dc(this);
                DrawTraces( &dc );
                }
}


void CViewWindow::UpdateColorScheme() {

        TRACE0("CViewWindow::UpdateColorScheme\n");

        DeletePens();   // Remove old pens
        SetColors();    // Set colors according to current scheme
        CreatePens();   // Create new pens
}


void CViewWindow::SetColors() {

        TRACE0("CViewWindow::SetColors\n");

        switch (pAPP->PltColorScheme) {

        case CS_MONO:

                ColorBack  = RGB(255, 255, 255);        // White
                ColorAxis  = RGB(224, 224, 224);        // White Dim
                ColorTitle = RGB( 96,  96,  96);        // Gray Dark
                ColorTrace = RGB(  0,   0,   0);        // Black
                break;


        case CS_GREEN:

                ColorBack  = RGB(  0,   0,   0);        // Black
                ColorAxis  = RGB(128, 128, 128);        // Gray
                ColorTitle = RGB(160, 255, 160);        // Green Light
                ColorTrace = RGB(  0, 255,   0);        // Green
                break;


        case CS_BLUE:

                ColorBack  = RGB(  0,   0, 128);        // Blue Dark
                ColorAxis  = RGB(128, 128, 128);        // Gray
                ColorTitle = RGB(255, 255, 255);        // White
                ColorTrace = RGB(  0, 255, 255);        // Cyan
                break;


        case CS_BROWN:

                ColorBack  = RGB(128,   0,   0);        // Brown
                ColorAxis  = RGB(128, 128, 128);        // Gray
                ColorTitle = RGB(255, 192, 160);        // Light Orange
                ColorTrace = RGB(255, 255, 255);        // White
                break;


        default:
                ColorBack  = RGB(  0,   0,   0);        // Black
                ColorAxis  = RGB(128, 128, 128);        // Gray
                ColorTitle = RGB(  0, 128,   0);        // Green Dark
                ColorTrace = RGB(  0, 255,   0);        // Green
                break;

        }
}

void CViewWindow::CreatePens() {

        TRACE0("CViewWindow::CreatePens\n");

        PenBack  = new CPen( PS_SOLID, 1, ColorBack  );
        PenAxis  = new CPen( PS_SOLID, 1, ColorAxis  );
        PenTitle = new CPen( PS_SOLID, 1, ColorTitle );
        PenTrace = new CPen( PS_SOLID, 1, ColorTrace );
}

void CViewWindow::DeletePens() {

        TRACE0("CViewWindow::DeletePens\n");

        // Make sure pens are not selected

        CClientDC dc(this);
        dc.SelectObject(GetStockObject(BLACK_PEN));


        // Delete them (destructor calls DeleteObject)

        delete PenBack;   PenBack  = NULL;
        delete PenAxis;   PenAxis  = NULL;
        delete PenTitle;  PenTitle = NULL;
        delete PenTrace;  PenTrace = NULL;
}

void CViewWindow::CreateFont() {

        TRACE0("CViewWindow::CreateFont\n");

        if (CurrFont != NULL)
                DeleteFont();


        // Compensate for including Internal Leading here

        if (font.lfHeight > 0)
                font.lfHeight = -font.lfHeight;




        CurrFont = new CFont();
        CurrFont->CreateFontIndirect( &font );

        CurrFont->GetLogFont( &font );



        // Uncompensate for including Internal Leading here

        if (font.lfHeight < 0)
                font.lfHeight = -font.lfHeight;
}

void CViewWindow::DeleteFont() {

        TRACE0("CViewWindow::DeleteFont\n");

        // Make sure font is not selected

        CClientDC dc(this);
        dc.SelectObject(GetStockObject(SYSTEM_FIXED_FONT));


        // Delete it (destructor calls DeleteObject)

        delete CurrFont;        CurrFont = NULL;

}

void CViewWindow::DeletePlotArrays( ) {

        TRACE0("CViewWindow::DeletePlotArrays\n");

        if (pPtArray)
                { free(pPtArray);  pPtArray  = NULL; }

        if (pOldArray)
                { free(pOldArray); pOldArray = NULL; }
}

void CViewWindow::DeleteVerticalBuffers( ) {

        TRACE0("CViewWindow::DeleteVerticalBuffers\n");

        if (pYlast)
                { free(pYlast);    pYlast    = NULL; }

        if (pYaxis)
                { free(pYaxis);    pYaxis    = NULL; }

        if (pYclipmin)
                { free(pYclipmin); pYclipmin = NULL; }

        if (pYclipmax)
                { free(pYclipmax); pYclipmax = NULL; }
}

void CViewWindow::UpdateTextSize( CDC *pDC ) {

        TRACE0("CViewWindow::UpdateTextSize\n");

        TEXTMETRIC tm;


        // Update text dimensions

        pDC->SelectObject( CurrFont );
        pDC->GetTextMetrics( &tm );

        cxChar = tm.tmAveCharWidth;
        cyChar = tm.tmHeight + tm.tmExternalLeading;
}


void CViewWindow::OnContentsDialog() {

        TRACE0("CViewWindow::OnContentsDialog\n");

        CContentsDlg dlg(this);
        dlg.DoModal();

        controlkey = FALSE;
}

void CViewWindow::OnAboutDialog() {

        TRACE0("CViewWindow::OnAboutDialog\n");

        CAboutDlg dlg(this);
        dlg.DoModal();

        controlkey = FALSE;
}

void CViewWindow::OnDebug() {

        TRACE0("CViewWindow::OnDebug\n");

        CString msg;

        CClientDC dc(this);

        int lpy  = dc.GetDeviceCaps( LOGPIXELSY );
        int vres = dc.GetDeviceCaps( VERTRES );

        msg.Format("LOGPIXELSY = %d, VERTRES = %d\n",lpy,vres);
        AfxMessageBox(msg);
}


void CViewWindow::OnUpdateClipCheck(CCmdUI* pCmdUI) {

        TRACE0("CViewWindow::OnUpdateClipCheck\n");

        pCmdUI->SetCheck( pAPP->PltClipping );
}

void CViewWindow::OnUpdateAutoDCCheck(CCmdUI* pCmdUI) {

        TRACE0("CViewWindow::OnUpdateAutoDCCheck\n");

        pCmdUI->SetCheck( pAPP->PltAutoDC );
}

void CViewWindow::SetPaneString( ) {

        TRACE0("CViewWindow::SetPaneString\n");

        CString PaneString;


        // Sampling rate

        PaneString.Format( "%9.3f Hz", pAPP->AcqSPS );



        // Auto DC offset

        if (pAPP->PltAutoDC == TRUE )
                PaneString += "  AutoDC";


        // Clipping

        if (pAPP->PltClipping == TRUE )
                PaneString += "  Clip";


        // Sequential naming style

        if (!pAPP->DataDoSeq)
                PaneString += "     ";
        else if (pAPP->DataSeqType == SEQTYPE_HEX )
                PaneString += "  Hex";
        else // (pAPP->DataSeqType == SEQTYPE_DEC )
                PaneString += "  Dec";


        // Current file name and position

        if ( !pAPP->DataCurrName.IsEmpty() ) {
                sprintf( pAPP->szTemp, "  %s  Pt %d to %d",
                         pAPP->DataCurrName,
                         pAPP->DataCurrentPt,
                         (pAPP->DataCurrentPt+pAPP->NsamplesPlot-1) );
                PaneString += pAPP->szTemp;
                }


        // Now, finally write out the pane string we've built up

        m_wndStatusBar.SetPaneText( PANE_INFO, (LPCTSTR)PaneString, TRUE );
}

void CViewWindow::SetScrollBar( int nBar ) {

        TRACE0("CViewWindow::SetScrollBar\n");

        SCROLLINFO sif;

        sif.cbSize = sizeof(SCROLLINFO);
        sif.fMask  = SIF_ALL;


        if ((nBar == SB_VERT) || (nBar == SB_BOTH)) {

                sif.nMin  = 0;
                sif.nMax  = pAPP->NchannelsPlot - 1;
                sif.nPage = pAPP->PltNdisplay;
                sif.nPos  = pAPP->PltFirstTrace;

                SetScrollInfo( SB_VERT, &sif, TRUE );
                }


        if ((nBar == SB_HORZ) || (nBar == SB_BOTH)) {

                sif.nMin  = 0;
                sif.nMax  = pAPP->NsamplesPlot - 1;
                sif.nPage = TraceWidth;
                sif.nPos  = pAPP->PltFirstSample;

                SetScrollInfo( SB_HORZ, &sif, TRUE );
                }

}

void CViewWindow::SetWindowTitle() {

        TRACE0("CViewWindow::SetWindowTitle\n");

        CString title;
        CString tmpstr;


        title = pAPP->AppTitle;


        // Showing bits

        tmpstr.Format( " - Showing %d bits",
                       pAPP->MaxAmpShift - pAPP->PltShiftBy );
        title += tmpstr;


        // File naming scheme

        if (!pAPP->DataDoSeq)
                title += " - Single file name";

        else if (pAPP->DataSeqType == SEQTYPE_HEX )
                title += " - Sequential hex file names";

        else // (pAPP->DataSeqType == SEQTYPE_DEC )
                title += " - Sequential decimal file names";

        SetWindowText( title );
}

void CViewWindow::GetSize( int *height, int *width ) {

        TRACE0("CViewWindow::GetSize\n");

        RECT r, rs;

        GetClientRect(&r);

        if (m_wndStatusBar.IsWindowVisible())
                m_wndStatusBar.GetClientRect( &rs );
        else
                rs.bottom = 0;


        *height = r.bottom - rs.bottom;
        *width  = r.right;
}

int CViewWindow::ChangeFontSize( int newsize ) {

        TRACE0("CViewWindow::ChangeFontSize\n");


        // Enforce reasonable sizes

        if (newsize < minPoint)
                newsize = minPoint;
        if (newsize > maxPoint)
                newsize = maxPoint;



        // Test for no change

        if (font.lfHeight == newsize)
                return( 0 );



        // Create font with new size

        font.lfWidth  = 0;
        font.lfHeight = newsize;


// Leave font the same no matter what size we're asking for
//
//        if (newsize < midPoint)
//                SrStringCopy(font.lfFaceName,"Small Fonts", LF_FACESIZE);
//        else
//                SrStringCopy(font.lfFaceName,"CourierNew", LF_FACESIZE); // was Arial


        CreateFont();




        // Refigure text size and title length

        CClientDC dc(this);

        UpdateTextSize( &dc );

        ComputeMaxTitleLength( &dc );


        return( 1 );
}

int CViewWindow::ComputeMaxTitleLength( CDC *pDC ) {

        TRACE0("CViewWindow::ComputeMaxTitleLength\n");

        int       i, ncount;
        CSize     sz(0,0);
        LPCTSTR   pstr;

        MaxTitleLength = 0;

        pDC->SelectObject( CurrFont );


        for ( i = 0 ; i < pAPP->MaxTotalPlotChannel ; i++ ) {

                pstr   = &(pAPP->pPlotTitle[i*MAXTITLE]);
                ncount = strlen( pstr );

                sz     = pDC->GetTextExtent( pstr, ncount );

                if (MaxTitleLength < sz.cx)
                        MaxTitleLength = sz.cx;
                }

        return( MaxTitleLength );
}

int CViewWindow::ComputePlotPositions( ) {

        TRACE0("CViewWindow::ComputePlotPositions\n");

        int    totalHeight, traceHalf, evenHeight, halfExcess;
        int    height, width, sbextra, maxFirstSample;
        int    i, j, xpos, yax;
        POINT *pArray;

        SCROLLINFO sif;


        // Always check the return value of this function and return
        // immediately if it fails.  This is because failure means that
        // the allocate plot buffer function has failed and the destroy
        // window function may have been called so the entire CViewWindow
        // is deleted.  You can't access any class variables at that point,
        // so all functions should just unwind with returns.




        // Get size of window

        GetSize( &height, &width );



        // Always compute totalHeight assuming that horizontal scroll bar
        // is visible to avoid race condition where size without bar
        // increases font which reduces width so that bar is needed, but
        // size with bar has smaller font that increases width so bar
        // is not needed, etc

        GetScrollInfo( SB_HORZ, &sif );

        if ((int)sif.nPage >= sif.nMax)
                sbextra = GetSystemMetrics( SM_CYHSCROLL );
        else
                sbextra = 0;



        // Compute trace height

        totalHeight = height - 2 * VMARGIN - 2 * VTICK - sbextra;
        TraceHeight = (int)((float)totalHeight / pAPP->PltNdisplay);


        ChangeFontSize( TraceHeight/2 );  // was TraceHeight


        // Compute scale positions

        YScaleTop    = 0      + VMARGIN;
        YScaleBottom = height - VMARGIN;


        // Compute trace width

        TraceStart = HMARGIN + MaxTitleLength + HMARGIN;
        TraceEnd   = width - HMARGIN;
        TraceWidth = TraceEnd - TraceStart;

        if (TraceWidth < 1)
                TraceWidth = 1;



        // Compute trace height positions

        traceHalf   = TraceHeight / 2;
        evenHeight  = TraceHeight * pAPP->PltNdisplay;
        halfExcess  = (totalHeight - evenHeight + sbextra) / 2;

        pYaxis[0] = VMARGIN + VTICK + halfExcess + traceHalf;
        pYlast[0] = pYaxis[0];

        for ( i = 1 ; i < pAPP->MaxTotalPlotChannel ; i++ ) {
                pYaxis[i] = pYaxis[i - 1] + TraceHeight;
                pYlast[i] = pYaxis[i];
                }


        // Update clipping

        SetClipLimits( height, traceHalf );




        // If needed, reallocate plot buffers

        if (!AllocatePlotBuffers())
                return( 0 );



        // Reset horizontal scroll bar range

        maxFirstSample = pAPP->NsamplesPlot - TraceWidth - 1;
        if (maxFirstSample < 0)
                maxFirstSample = 0;
        if (pAPP->PltFirstSample > maxFirstSample)
                pAPP->PltFirstSample = maxFirstSample;

        SetScrollBar( SB_HORZ );



        // Fill up polyline positions

        pAPP->ResetPaint = TRUE;

        pArray = pPtArray;

        for ( j = 0 ; j < pAPP->NchannelsPlot; j++ ) {

                xpos = TraceStart;
                yax  = pYaxis[j];

                for ( i = 0 ; i < pAPP->NsamplesPlot ; i++ ) {

                        pArray->x = xpos++;
                        pArray->y = yax;

                        pArray++;
                        }
                }

        // Need to replot

        Invalidate( FALSE );

        return( 1 );
}


int CViewWindow::AllocatePlotBuffers() {

        TRACE0("CViewWindow::AllocatePlotBuffers\n");


        // Only reallocate if need to increase size

        if (LastPtsPlot < pAPP->NbufferptsPlot) {

                LastPtsPlot = pAPP->NbufferptsPlot;


                // Clean up any previous plot arrays

                DeletePlotArrays( );


                // Allocate current arrays

                pPtArray  = (POINT*) malloc(pAPP->NbufferptsPlot * sizeof(POINT));
                pOldArray = (POINT*) malloc(pAPP->NbufferptsPlot * sizeof(POINT));

                if (!pPtArray || !pOldArray) {
                        pAPP->ErrString = "Can not allocate plot buffers\n";
                        pAPP->ExitError();
                        return( 0 );
                        }
                }


        // Fail with NULL arrays even if haven't reallocated
        if (!pPtArray || !pOldArray)
                return( 0 );

        return( 1 );
}


void CViewWindow::InitDialogPositions( void ) {

        TRACE0("CViewWindow::InitDialogPositions\n");


        // These dialogs are identical for all A/D systems

        DisplayPosTop   = DLG_POS_INIT;
        DisplayPosLeft  = DLG_POS_INIT;

        ContentsPosTop  = DLG_POS_INIT;
        ContentsPosLeft = DLG_POS_INIT;

        TopicPosTop     = DLG_POS_INIT;
        TopicPosLeft    = DLG_POS_INIT;
        TopicPosBottom  = DLG_POS_INIT;
        TopicPosRight   = DLG_POS_INIT;

}

void CViewWindow::GetDialogPosition( RECT *r, int nID ) {

        TRACE0("CViewWindow::GetDialogPosition\n");

        int height, width;


        // Keep original size

        height = r->bottom - r->top;
        width  = r->right  - r->left;


        // Get saved dialog positions

        switch ( nID ) {


                // These dialogs are identical for all A/D systems

                case IDD_DISPLAY:
                        r->top    = DisplayPosTop;
                        r->left   = DisplayPosLeft;
                        r->bottom = r->top  + height;
                        r->right  = r->left + width;
                        break;

                case IDD_HELPCONTENTS:
                        r->top    = ContentsPosTop;
                        r->left   = ContentsPosLeft;
                        r->bottom = r->top  + height;
                        r->right  = r->left + width;
                        break;

                case IDD_HELPTOPIC:
                        r->top    = TopicPosTop;
                        r->left   = TopicPosLeft;
                        r->bottom = TopicPosBottom;
                        r->right  = TopicPosRight;
                        break;


                default:
                        r->top    = DLG_POS_INIT;
                        r->left   = DLG_POS_INIT;
                        r->bottom = DLG_POS_INIT;
                        r->right  = DLG_POS_INIT;
                        break;
                }
}


void CViewWindow::SaveDialogPosition( RECT *r, int nID ) {

        TRACE0("CViewWindow::SaveDialogPosition\n");


        // Save dialog positions

        switch ( nID ) {

                // These dialogs are identical for all A/D systems

                case IDD_DISPLAY:
                        DisplayPosTop   = r->top;
                        DisplayPosLeft  = r->left;
                        break;

                case IDD_HELPCONTENTS:
                        ContentsPosTop  = r->top;
                        ContentsPosLeft = r->left;
                        break;

                case IDD_HELPTOPIC:
                        TopicPosTop     = r->top;
                        TopicPosLeft    = r->left;
                        TopicPosBottom  = r->bottom;
                        TopicPosRight   = r->right;
                        break;

                }
}





//***********************************************************************
//
// CLASS CViewDialog
//
// This class, derived from CDialog, contains a number of validation and
// convenience functions available to all the view dialogs.  Basically,
// it is just an extension of the CDialog class
//

BEGIN_MESSAGE_MAP(CViewDialog, CDialog)
        //{{AFX_MSG_MAP(CViewDialog)
        ON_WM_ACTIVATEAPP()
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()




CViewDialog::CViewDialog(UINT nIdTemplate, CWnd* pParent /*=NULL*/)
             : CDialog(nIdTemplate, pParent) {

        TRACE0("CViewDialog::CViewDialog\n");

        DialogId   = nIdTemplate;
        CancelId   = IDCANCEL;
        DoValidate = TRUE;
}

BOOL CViewDialog::OnInitDialog( int OkButtonId ) {

        TRACE0("CViewDialog::OnInitDialog\n");

        CDialog::OnInitDialog();


        // Return to saved position, if any

        RestorePosition();


        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CViewDialog::OnOK() {

        TRACE0("CViewDialog::OnOK\n");

        RememberPosition();

        CDialog::OnOK();
}

void CViewDialog::OnCancel() {

        TRACE0("CViewDialog::OnCancel\n");

        RememberPosition();

        CDialog::OnCancel();

        DoValidate = FALSE;
}

void CViewDialog::OnActivateApp( BOOL bActive, HTASK hTask ) {

        TRACE0("CViewDialog::OnActivateApp\n");

        CDialog::OnActivateApp( bActive, hTask );

        DoValidate = bActive;
}

LONG CViewDialog::OnEditLostFocus( UINT nID, LONG xxx ) {

        TRACE1("CViewDialog::OnEditLostFocus (%u)\n", nID);

        MessageBeep( 0 );

        return( 1 );
}

int CViewDialog::CheckFocus() {

        TRACE0("CViewDialog::CheckFocus\n");

        CWnd *pWnd;
        int   iret;

        // Call this function from inside button click routines.
        // Even though the click routine is being executed, the
        // previous control may still have the focus.  If it was
        // an edit control, it needs to go through the "losing
        // focus" validation function.

        pWnd = GetFocus();

        if (pWnd)
                iret = OnEditLostFocus(pWnd->GetDlgCtrlID(), 0L );
        else
                iret = 1;


        return( iret );
}

int CViewDialog::ValidateAsInt( int nID, int minval, int maxval, int *theval, char *txt ) {

        TRACE0("CViewDialog::ValidateAsInt\n");

        BOOL good;
        int  oldval, newval;


        oldval = *theval;

        newval = (int)GetDlgItemInt(nID, &good);

        if ((good) && (newval >= minval) && (newval <= maxval) ) {
                *theval = newval;
                return( TRUE );
                }
        else {
                pAPP->ErrString.Format("%s must be an integer between %d and %d", (LPCTSTR)txt, minval, maxval);
                pAPP->ShowError( );
                SetDlgItemInt(nID, oldval);
                GotoDlgCtrl( GetDlgItem(nID) );
                return( FALSE );
                }
}

int CViewDialog::ValidateAsLong( int nID, long minval, long maxval, long *theval, char *txt ) {

        TRACE0("CViewDialog::ValidateAsLong\n");

        BOOL good;
        long  oldval, newval;


        oldval = *theval;

        newval = (long)GetDlgItemInt(nID, &good);

        if ((good) && (newval >= minval) && (newval <= maxval) ) {
                *theval = newval;
                return( TRUE );
                }
        else {
                pAPP->ErrString.Format("%s must be a long integer between %d and %d", (LPCTSTR)txt, minval, maxval);
                pAPP->ShowError( );
                SetDlgItemInt(nID, (int)oldval);
                GotoDlgCtrl( GetDlgItem(nID) );
                return( FALSE );
                }
}

int CViewDialog::ValidateAsHex( int nID, int minval, int maxval, int *theval, char *txt ) {

        TRACE0("CViewDialog::ValidateAsHex\n");

        CString str;
        BOOL    good;
        int     oldval, newval;


        oldval = *theval;

        GetDlgItemText( nID, str );

        newval = strtol( str, NULL, 0 );

        // Overflow or underflow in conversion to int

        if ( (newval == LONG_MAX) || (newval == LONG_MIN) )
                good = FALSE;

        // Found zero or can't convert

        else if ( newval == 0 ) {
                str.TrimLeft();
                if ( (str.Compare("0") == 0) || (str.Compare("0x0") == 0) )
                        good = TRUE;
                else
                        good = FALSE;
                }

        // All is good

        else
                good = TRUE;


        if ((good) && (newval >= minval) && (newval <= maxval) ) {
                *theval = newval;
                return( TRUE );
                }
        else {
                pAPP->ErrString.Format("%s must be an integer between %d and %d", (LPCTSTR)txt, minval, maxval);
                pAPP->ShowError( );

                SetDlgItemHex( nID, oldval );
                GotoDlgCtrl( GetDlgItem(nID) );
                return( FALSE );
                }
}


int CViewDialog::ValidateAsFloat( int nID, float minval, float maxval, float *theval, char *txt ) {

        TRACE0("CViewDialog::ValidateAsFloat\n");

        CString btntxt;
        BOOL    good;
        float   oldval, newval;


        oldval = *theval;

        GetDlgItemText(nID, btntxt);
        newval = (float)strtod(btntxt,NULL);
        if (newval != 0.0)
                good = TRUE;
        else
                good = FALSE;

        if ((good) && (newval >= minval) && (newval <= maxval) ) {
                *theval = newval;
                return( TRUE );
                }
        else {
                pAPP->ErrString.Format("%s must be a float between %f and %f", (LPCTSTR)txt, minval, maxval);
                pAPP->ShowError( );

                btntxt.Format("%f", oldval);
                SetDlgItemText(nID, btntxt);
                GotoDlgCtrl( GetDlgItem(nID) );
                return( FALSE );
                }
}


int CViewDialog::SetDlgItemHex( int nID, int val ) {

        TRACE0("CViewDialog::SetDlgItemHex\n");

        CString txt;
        txt.Format("0x%X", val);
        SetDlgItemText(nID, txt);

        return(TRUE);
}


int CViewDialog::SetDlgItemFloat( int nID, float val ) {

        TRACE0("CViewDialog::SetDlgItemFloat\n");

        CString txt;

        txt.Format("%.3f", val);
        SetDlgItemText( nID, txt );

        return(TRUE);
}


float CViewDialog::GetDlgItemFloat(int nID, BOOL *lpTrans ) {

        CString txt;
        float   newval;

        GetDlgItemText(nID, txt);
        newval = (float)strtod(txt,NULL);


        // Fill error flag if desired

        if (lpTrans != NULL) {
                if (newval == 0.0)
                        *lpTrans = FALSE;
                else
                        *lpTrans = TRUE;
                }

        return( newval );
}


BOOL CViewDialog::SetTextReadOnly( int nID ) {

        TRACE0("CViewDialog::SetTextReadOnly\n");

        CEdit *pTxt = (CEdit*)GetDlgItem( nID );
        if (pTxt)
                return( pTxt->SetReadOnly( TRUE ) );
        else
                return( FALSE );
}

void CViewDialog::RestorePosition() {

        TRACE0("CViewDialog::RestorePosition\n");

        CViewWindow *pWnd = (CViewWindow *)m_pParentWnd;

        if (!pWnd)
                return;


        // Get current positions

        RECT r;

        GetWindowRect( &r );



        // Update with saved positions

        pWnd->GetDialogPosition( &r, DialogId );



        // If no uninitialized positions, then move dialog

        if ((r.top    != DLG_POS_INIT) &&
            (r.left   != DLG_POS_INIT) &&
            (r.bottom != DLG_POS_INIT) &&
            (r.right  != DLG_POS_INIT) )

                MoveWindow( &r );
}

void CViewDialog::RememberPosition() {

        TRACE0("CViewDialog::RememberPosition\n");


        CViewWindow *pWnd = (CViewWindow *)m_pParentWnd;

        if (!pWnd)
                return;



        // Get current position and have main window save it.

        RECT r;

        GetWindowRect( &r );

        pWnd->SaveDialogPosition( &r, DialogId );
}




//***********************************************************************
//
// CLASS CAboutDlg
//
// This class, derived from CDialog, displays version information about
// the current program.
//

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
        //{{AFX_MSG_MAP(CAboutDlg)
        ON_BN_CLICKED(IDC_BTN_X_OK,  OnOK)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()




CAboutDlg::CAboutDlg(CWnd* pParent /*=NULL*/)
          : CDialog(CAboutDlg::IDD, pParent) {

        TRACE0("CAboutDlg::CAboutDlg\n");
}

BOOL CAboutDlg::OnInitDialog() {

        TRACE0("CAboutDlg::OnInitDialog\n");

        CDialog::OnInitDialog();


        // Set dialog title based on app title

        sprintf( pAPP->szTemp, "About %s", pAPP->AppTitle );

        SetWindowText( pAPP->szTemp );



        // Set version text based on app title

        CStatic *pWnd = (CStatic *) GetDlgItem( IDC_LAB_X_VERSION );

        if (pWnd) {

                sprintf( pAPP->szTemp, "%s Rev %s",
                         pAPP->AppTitle, pAPP->AppRev );

                pWnd->SetWindowText( pAPP->szTemp );

                }


        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CAboutDlg::OnOK() {

        TRACE0("CAboutDlg::OnOK\n");

        CDialog::OnOK();
}





//***********************************************************************
//
// CLASS CMsgDlg
//
// This class, derived from CDialog, displays a message with no beep like
// AfxMessageBox.
//

BEGIN_MESSAGE_MAP(CMsgDlg, CDialog)
        //{{AFX_MSG_MAP(CMsgDlg)
        ON_BN_CLICKED(IDC_BTN_M_OK,  OnOK)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()




CMsgDlg::CMsgDlg(CWnd* pParent /*=NULL*/)
          : CDialog(CMsgDlg::IDD, pParent) {

        TRACE0("CMsgDlg::CMsgDlg\n");

        m_MsgText   = "View Message";
        m_TitleText = "Message for View";

}

CMsgDlg::CMsgDlg(const char *Message, const char *Title, CWnd* pParent /*=NULL*/)
          : CDialog(CMsgDlg::IDD, pParent) {

        TRACE0("CMsgDlg::CMsgDlg\n");

        if (Message)
                m_MsgText = Message;
        if (Title)
                m_TitleText = Title;

}

BOOL CMsgDlg::OnInitDialog() {

        TRACE0("CMsgDlg::OnInitDialog\n");

        CDialog::OnInitDialog();


        // Set dialog title

        SetWindowText( (LPCTSTR)m_TitleText );



        // Set static message text

        CStatic *pWnd = (CStatic *) GetDlgItem( IDC_LAB_M_MESSAGE );

        if (pWnd)
                pWnd->SetWindowText( (LPCTSTR)m_MsgText );


        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CMsgDlg::OnOK() {

        TRACE0("CMsgDlg::OnOK\n");

        CDialog::OnOK();
}





//***********************************************************************
//
// CLASS CDisplayDlg
//
// This class, derived from CViewDialog, presents and controls the Display
// dialog.  This dialog allows the user to specify various display options
// such as the number of traces to be displayed, the scaling applied
// to those traces, and the overall color scheme.

BEGIN_MESSAGE_MAP(CDisplayDlg, CViewDialog)
        //{{AFX_MSG_MAP(CDisplayDlg)
        ON_MESSAGE(WM_EDITLOSTFOCUS,           OnEditLostFocus)
        ON_BN_CLICKED(IDC_BTN_D_OK,            OnOK)
        ON_BN_CLICKED(IDC_BTN_D_CANCEL,        OnCancel)
        ON_BN_CLICKED(IDC_BTN_D_HELP,          OnHelp)
        ON_EN_KILLFOCUS(IDC_TXT_D_SHIFTBY,     OnKillfocusTxtShiftby)
        ON_EN_KILLFOCUS(IDC_TXT_D_NDISPLAY,    OnKillfocusTxtNdisplay)
        ON_EN_KILLFOCUS(IDC_TXT_D_FIRSTSAMPLE, OnKillfocusTxtFirstsample)
        ON_EN_KILLFOCUS(IDC_TXT_D_FIRSTTRACE,  OnKillfocusTxtFirsttrace)
        ON_BN_CLICKED(IDC_CHK_D_CLIPPING,      OnChkClipping)
        ON_BN_CLICKED(IDC_CHK_D_AUTODC,        OnChkAutoDC)
        ON_BN_CLICKED(IDC_OPT_D_COLORMONO,     OnOptColorScheme)
        ON_BN_CLICKED(IDC_OPT_D_COLORGREEN,    OnOptColorScheme)
        ON_BN_CLICKED(IDC_OPT_D_COLORBLUE,     OnOptColorScheme)
        ON_BN_CLICKED(IDC_OPT_D_COLORBROWN,    OnOptColorScheme)
        //}}AFX_MSG_MAP
//      ON_CONTROL_RANGE(BN_CLICKED, IDC_OPT_D_COLORMONO, IDC_OPT_D_COLORBROWN, OnOptColorScheme)
END_MESSAGE_MAP()




CDisplayDlg::CDisplayDlg(CWnd* pParent /*=NULL*/)
           : CViewDialog(CDisplayDlg::IDD, pParent) {

        //{{AFX_DATA_INIT(CDisplayDlg)
        m_ShiftBy     = 0;
        m_Ndisplay    = 0;
        m_FirstSample = 0;
        m_FirstTrace  = 0;
        m_ColorScheme = 0;
        m_TraceWidth  = 0;
        m_Clipping    = FALSE;
        m_AutoDC      = FALSE;
        //}}AFX_DATA_INIT
}


void CDisplayDlg::DoDataExchange(CDataExchange* pDX) {

        CViewDialog::DoDataExchange(pDX);

        //{{AFX_DATA_MAP(CDisplayDlg)
        DDX_Control(pDX, IDC_TXT_D_SHIFTBY,     m_TXT_ShiftBy);
        DDX_Control(pDX, IDC_TXT_D_NDISPLAY,    m_TXT_Ndisplay);
        DDX_Control(pDX, IDC_TXT_D_FIRSTSAMPLE, m_TXT_FirstSample);
        DDX_Control(pDX, IDC_TXT_D_FIRSTTRACE,  m_TXT_FirstTrace);
        DDX_Control(pDX, IDC_LAB_D_PLOTBITS,    m_LAB_PlotBits);
        DDX_Control(pDX, IDC_LAB_D_NSCAN,       m_LAB_Nscan);
        DDX_Control(pDX, IDC_LAB_D_LASTSAMPLE,  m_LAB_LastSample);
        DDX_Control(pDX, IDC_CHK_D_CLIPPING,    m_CHK_Clipping);
        DDX_Control(pDX, IDC_CHK_D_AUTODC,      m_CHK_AutoDC);
        DDX_Control(pDX, IDC_OPT_D_COLORMONO,   m_OPT_ColorMono);
        DDX_Control(pDX, IDC_OPT_D_COLORGREEN,  m_OPT_ColorGreen);
        DDX_Control(pDX, IDC_OPT_D_COLORBLUE,   m_OPT_ColorBlue);
        DDX_Control(pDX, IDC_OPT_D_COLORBROWN,  m_OPT_ColorBrown);
        DDX_Text(pDX, IDC_TXT_D_SHIFTBY,     m_ShiftBy);
        DDX_Text(pDX, IDC_TXT_D_NDISPLAY,    m_Ndisplay);
        DDX_Text(pDX, IDC_TXT_D_FIRSTSAMPLE, m_FirstSample);
        DDX_Text(pDX, IDC_TXT_D_FIRSTTRACE,  m_FirstTrace);
        //}}AFX_DATA_MAP
}


BOOL CDisplayDlg::OnInitDialog() {

        // Initialize all member variables from application globals

        m_ShiftBy     = pAPP->PltShiftBy;
        m_Ndisplay    = pAPP->PltNdisplay;
        m_FirstSample = pAPP->PltFirstSample;
        m_FirstTrace  = pAPP->PltFirstTrace;
        m_ColorScheme = pAPP->PltColorScheme;
        m_Clipping    = pAPP->PltClipping;
        m_AutoDC      = pAPP->PltAutoDC;


        CViewWindow *pWnd = (CViewWindow *)m_pParentWnd;
        if (pWnd)
                m_TraceWidth = pWnd->TraceWidth;
        else
                m_TraceWidth = 500;




        // Initialize base class and initialize text boxes (via DDX)

        CViewDialog::OnInitDialog( 0 ); // use 0 not IDC_BTN_D_OK because
                                         // display dialog options are allowed
                                         // even in non-interactive mode


        // Initialize labels and radio buttons

        SetPlotBitsLabel();
        SetNscanLabel();
        SetLastSampleLabel();
        SetColorSchemeRadio();

        if (m_Clipping)
                m_CHK_Clipping.SetCheck(CHECKED);
        else
                m_CHK_Clipping.SetCheck(UNCHECKED);


        if (m_AutoDC)
                m_CHK_AutoDC.SetCheck(CHECKED);
        else
                m_CHK_AutoDC.SetCheck(UNCHECKED);



        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CDisplayDlg::SetPlotBitsLabel( ) {

        CString str;
        int nscan;

        nscan = pAPP->MaxAmpShift - m_ShiftBy;
        str.Format("Plot %d Bits", nscan );
        CStatic *Label = (CStatic*)GetDlgItem(IDC_LAB_D_PLOTBITS);
        Label->SetWindowText( str );
}

void CDisplayDlg::SetNscanLabel( ) {
        CString str;
        str.Format("Scan %d Traces", pAPP->NchannelsPlot);
        m_LAB_Nscan.SetWindowText( str );
}

void CDisplayDlg::SetLastSampleLabel( ) {

        sprintf(pAPP->szTemp, "Last is %d", (m_FirstSample + m_TraceWidth));
        m_LAB_LastSample.SetWindowText( pAPP->szTemp );
}

void CDisplayDlg::SetColorSchemeRadio( ) {

        TRACE0("CDisplayDlg::SetColorSchemeRadio\n");

        int ibtn;

        ibtn = IDC_OPT_D_COLORMONO + m_ColorScheme;
        CheckRadioButton( IDC_OPT_D_COLORMONO, IDC_OPT_D_COLORBROWN, ibtn );
}

void CDisplayDlg::OnCancel() {

        TRACE0("CDisplayDlg::OnCancel\n");

        CViewDialog::OnCancel();
}

void CDisplayDlg::OnOK() {

        TRACE0("CDisplayDlg::OnOK\n");

        if (!CheckFocus())
                return;


        CViewDialog::OnOK();


        // Update globals

        pAPP->PltShiftBy     = m_ShiftBy;
        pAPP->PltNdisplay    = m_Ndisplay;
        pAPP->PltFirstSample = m_FirstSample;
        pAPP->PltFirstTrace  = m_FirstTrace;
        pAPP->PltColorScheme = m_ColorScheme;
        pAPP->PltClipping    = m_Clipping;
        pAPP->PltAutoDC      = m_AutoDC;
}

void CDisplayDlg::OnHelp() {

        TRACE0("CDisplayDlg::OnHelp\n");

        if (!CheckFocus())
                return;


        pAPP->ShowHelp(IDR_HELP_DISP_TITLE, IDR_HELP_DISP_TEXT, this);
}

LONG CDisplayDlg::OnEditLostFocus( UINT nID, LONG xxx ) {

        TRACE1("CDisplayDlg::OnEditLostFocus (%u)\n", nID);

        int  maxFirst;
        BOOL good;


        if (!DoValidate) {
                return( 1 );
                }


        // Don't allow validation when already doing validation

        DoValidate = FALSE;

        // Validate various edit controls

        switch ( nID ) {

                case IDC_TXT_D_SHIFTBY:

                        good = ValidateAsInt( IDC_TXT_D_SHIFTBY,
                                                pAPP->MinAmpShift,
                                                pAPP->MaxAmpShift-1,
                                                &m_ShiftBy,
                                                "Shift by" );
                        if (good)
                                SetPlotBitsLabel();

                        break;



                case IDC_TXT_D_NDISPLAY:

                        good = ValidateAsInt( IDC_TXT_D_NDISPLAY,
                                                1,
                                                pAPP->NchannelsPlot,
                                                &m_Ndisplay,
                                                "Number to display" );

                        if (good) {
//                              AdjustFirstTrace();
                                maxFirst = pAPP->NchannelsPlot - m_Ndisplay;
                                if (m_FirstTrace > maxFirst) {
                                        m_FirstTrace = maxFirst;
                                        SetDlgItemInt(IDC_TXT_D_FIRSTTRACE,
                                                        m_FirstTrace);
                                        }
                                }

                        break;



                case IDC_TXT_D_FIRSTSAMPLE:

                        maxFirst = pAPP->NsamplesPlot - m_TraceWidth - 1;
                        if (maxFirst < 0)
                                maxFirst = 0;

                        good = ValidateAsInt( IDC_TXT_D_FIRSTSAMPLE,
                                                0,
                                                maxFirst,
                                                &m_FirstSample,
                                                "First sample" );

                        if (good)
                                SetLastSampleLabel();

                        break;




                case IDC_TXT_D_FIRSTTRACE:

                        maxFirst = pAPP->NchannelsPlot - m_Ndisplay;

                        good = ValidateAsInt( IDC_TXT_D_FIRSTTRACE,
                                                0,
                                                maxFirst,
                                                &m_FirstTrace,
                                                "First trace" );

                        break;

                default:
                        good = TRUE;
                        break;

                }


        // Re-allow validation

        DoValidate = TRUE;


        if (good)
                return( 1 );
        else
                return( 0 );
}


void CDisplayDlg::OnKillfocusTxtShiftby() {

        TRACE0("CDisplayDlg::OnKillfocusTxtShiftby\n");

        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_D_SHIFTBY, 0 );
}

void CDisplayDlg::OnKillfocusTxtNdisplay() {

        TRACE0("CDisplayDlg::OnKillfocusTxtNdisplay\n");

        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_D_NDISPLAY, 0 );
}


void CDisplayDlg::OnKillfocusTxtFirstsample() {

        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_D_FIRSTSAMPLE, 0 );
}

void CDisplayDlg::OnKillfocusTxtFirsttrace() {

        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_D_FIRSTTRACE, 0 );
}

void CDisplayDlg::OnChkClipping(  ) {

        m_Clipping = !m_Clipping;
}

void CDisplayDlg::OnChkAutoDC(  ) {

        m_AutoDC = !m_AutoDC;
}

void CDisplayDlg::OnOptColorScheme(  ) {

        int ibtn;

        ibtn = GetCheckedRadioButton( IDC_OPT_D_COLORMONO, IDC_OPT_D_COLORBROWN );
        m_ColorScheme = ibtn - IDC_OPT_D_COLORMONO;
}





//***********************************************************************
//
// CLASS CSeqNameDlg
//
// This class, derived from CViewDialog, presents and controls the Sequential
// Naming dialog.  This dialog allows the user to specify the sequential file
// name rule to use in determining the next file to display.

BEGIN_MESSAGE_MAP(CSeqNameDlg, CViewDialog)
        //{{AFX_MSG_MAP(CSeqNameDlg)
        ON_BN_CLICKED(IDC_BTN_Q_OK,        OnOK)
        ON_BN_CLICKED(IDC_BTN_Q_CANCEL,    OnCancel)
        ON_BN_CLICKED(IDC_BTN_Q_HELP,      OnHelp)
        ON_BN_CLICKED(IDC_OPT_Q_SEQDEC,    OnOptSeqNaming)
        ON_BN_CLICKED(IDC_OPT_Q_SEQHEX,    OnOptSeqNaming)
        //}}AFX_MSG_MAP
//      ON_CONTROL_RANGE(BN_CLICKED, IDC_OPT_Q_SEQDEC, IDC_OPT_Q_SEQHEX, OnOptSeqNaming)
END_MESSAGE_MAP()




CSeqNameDlg::CSeqNameDlg(CWnd* pParent /*=NULL*/)
           : CViewDialog(CSeqNameDlg::IDD, pParent) {

        //{{AFX_DATA_INIT(CSeqNameDlg)
        m_SeqNaming = 0;
        //}}AFX_DATA_INIT
}


void CSeqNameDlg::DoDataExchange(CDataExchange* pDX) {

        CViewDialog::DoDataExchange(pDX);

        //{{AFX_DATA_MAP(CSeqNameDlg)
        //}}AFX_DATA_MAP
}


BOOL CSeqNameDlg::OnInitDialog() {

        // Initialize all member variables from application globals

        m_SeqNaming = pAPP->DataSeqType;
        if (m_SeqNaming == SEQTYPE_NONE)
                m_SeqNaming = SEQTYPE_DEC;


        // Initialize base class (via DDX)

        CViewDialog::OnInitDialog( 0 ); // use 0 not IDC_BTN_Q_OK because
                                         // seq name dialog options are allowed
                                         // even in non-interactive mode


        // Initialize radio buttons

        SetSeqNamingRadio();


        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CSeqNameDlg::SetSeqNamingRadio( ) {

        TRACE0("CSeqNameDlg::SetSeqNamingRadio\n");

        int ibtn;

        if (m_SeqNaming == SEQTYPE_DEC)
                ibtn = IDC_OPT_Q_SEQDEC;

        else // (m_SeqNaming == SEQTYPE_HEX)
                ibtn = IDC_OPT_Q_SEQHEX;

        CheckRadioButton( IDC_OPT_Q_SEQDEC, IDC_OPT_Q_SEQHEX, ibtn );
}

void CSeqNameDlg::OnCancel() {

        TRACE0("CSeqNameDlg::OnCancel\n");

        CViewDialog::OnCancel();
}

void CSeqNameDlg::OnOK() {

        TRACE0("CSeqNameDlg::OnOK\n");

        if (!CheckFocus())
                return;


        CViewDialog::OnOK();


        // Update globals

        pAPP->DataSeqType = m_SeqNaming;
}

void CSeqNameDlg::OnHelp() {

        TRACE0("CSeqNameDlg::OnHelp\n");

        if (!CheckFocus())
                return;


        pAPP->ShowHelp(IDR_HELP_SEQN_TITLE, IDR_HELP_SEQN_TEXT, this);
}

void CSeqNameDlg::OnOptSeqNaming(  ) {

        int ibtn;

        ibtn = GetCheckedRadioButton( IDC_OPT_Q_SEQDEC, IDC_OPT_Q_SEQHEX );

        if (ibtn == IDC_OPT_Q_SEQDEC)
                m_SeqNaming = SEQTYPE_DEC;

        else // (ibtn == IDC_OPT_Q_SEQHEX)
                m_SeqNaming = SEQTYPE_HEX;
}





//***********************************************************************
//
// CLASS CGotoPtDlg
//
// This class, derived from CViewDialog, presents and controls the GotoPt
// dialog.  This dialog allows the user to specify a new start point
// for viewing data.

BEGIN_MESSAGE_MAP(CGotoPtDlg, CViewDialog)
        //{{AFX_MSG_MAP(CGotoPtDlg)
        ON_MESSAGE(WM_EDITLOSTFOCUS,      OnEditLostFocus)
        ON_BN_CLICKED(IDC_BTN_G_OK,       OnOK)
        ON_BN_CLICKED(IDC_BTN_G_CANCEL,   OnCancel)
        ON_BN_CLICKED(IDC_BTN_G_HELP,     OnHelp)
        ON_EN_KILLFOCUS(IDC_TXT_G_POINT,  OnKillfocusPoint)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()




CGotoPtDlg::CGotoPtDlg(CWnd* pParent /*=NULL*/)
              : CViewDialog(CGotoPtDlg::IDD, pParent) {

        TRACE0("CGotoPtDlg::CGotoPtDlg\n");

        //{{AFX_DATA_INIT(CGotoPtDlg)
        m_NewPt = 0;
        //}}AFX_DATA_INIT
}


void CGotoPtDlg::DoDataExchange(CDataExchange* pDX) {

        CViewDialog::DoDataExchange(pDX);

        //{{AFX_DATA_MAP(CGotoPtDlg)
        DDX_Control(pDX, IDC_TXT_G_POINT, m_TXT_NewPt);
        DDX_Text(pDX, IDC_TXT_G_POINT, m_NewPt);
        //}}AFX_DATA_MAP
}


BOOL CGotoPtDlg::OnInitDialog() {

       // Initialize all member variables from application globals

         m_NewPt = pAPP->DataCurrentPt;


        // Initialize base class and initialize text boxes (via DDX)

        CViewDialog::OnInitDialog( IDC_BTN_G_OK );


        return TRUE;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CGotoPtDlg::OnCancel() {

        TRACE0("CGotoPtDlg::OnCancel\n");

        CViewDialog::OnCancel();
}

void CGotoPtDlg::OnOK() {

        TRACE0("CGotoPtDlg::OnOK\n");

        if (!CheckFocus())
                return;

        CViewDialog::OnOK();


        // Update globals

        pAPP->DataCurrentPt = m_NewPt;
}

void CGotoPtDlg::OnHelp() {

        TRACE0("CGotoPtDlg::OnHelp\n");

        if (!CheckFocus())
                return;

        pAPP->ShowHelp(IDR_HELP_GOTO_TITLE, IDR_HELP_GOTO_TEXT, this);
}

LONG CGotoPtDlg::OnEditLostFocus( UINT nID, LONG xxx ) {

        TRACE1("CGotoPtDlg::OnEditLostFocus (%u)\n", nID);

        BOOL good;


        if (!DoValidate) {
                return( 1 );
                }


        // Don't allow validation when already doing validation

        DoValidate = FALSE;

        // Validate various edit controls

        switch ( nID ) {

                case IDC_TXT_G_POINT:

                        good = ValidateAsInt( IDC_TXT_G_POINT,
                                              -999999999,  // was pAPP->DataMinPt
                                               999999999,  // was pAPP->DataMaxPt,
                                              &m_NewPt,
                                              "New start pt" );
                        if (good) {
                                CorrectToBufferEdge();
                                break;
                                }

                default:
                        good = TRUE;
                        break;

                }


        // Re-allow validation

        DoValidate = TRUE;


        if (good)
                return( 1 );
        else
                return( 0 );
}

void CGotoPtDlg::OnKillfocusPoint() {

        TRACE0("CGotoPtDlg::OnKillfocusPoint\n");

        PostMessage( WM_EDITLOSTFOCUS, IDC_TXT_G_POINT, 0 );
}

void CGotoPtDlg::CorrectToBufferEdge() {

        TRACE0("CGotoPtDlg::CorrectToBufferEdge\n");

        int Nbuff;


        // Enforce limits when working with single file

        if (!pAPP->DataDoSeq) {
                if (m_NewPt < 0) {
                        m_NewPt = 0;
                        pAPP->ErrString.Format( "For single file, minimum start pt is 0\n" );
                        pAPP->ShowError();
                        }
                if (m_NewPt > pAPP->DataMaxPt) {
                        m_NewPt = pAPP->DataMaxPt;
                        pAPP->ErrString.Format("For single file, maximum start pt is %d\n",
                                                                         pAPP->DataMaxPt);
                        pAPP->ShowError();
                        }
                }




        // Keep point an integral buffer edge.

        if (pAPP->DataPtsPerBuff != 0) {
                Nbuff = (int)(m_NewPt / pAPP->DataPtsPerBuff);
                if (m_NewPt < 0)
                        Nbuff--;
                m_NewPt = Nbuff * pAPP->DataPtsPerBuff;
                }

        SetDlgItemInt( IDC_TXT_G_POINT, m_NewPt );
}



//***********************************************************************
//
// CLASS CContentsDlg
//
// This class, derived from CViewDialog, presents and controls the Help
// Contents dialog.  This dialog lists the topics for which help text is
// available and allows the user to select and view this text.

BEGIN_MESSAGE_MAP(CContentsDlg, CViewDialog)
        //{{AFX_MSG_MAP(CContentsDlg)
        ON_BN_CLICKED(IDC_BTN_H_DISPLAY,   OnBtnDisplay)
        ON_BN_CLICKED(IDC_BTN_H_HELP,      OnHelp)
        ON_BN_CLICKED(IDC_BTN_H_CLOSE,     OnCancel)
        ON_LBN_DBLCLK(IDC_LST_H_TOPICS,    OnDblclkTopics)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()


CContentsDlg::CContentsDlg(CWnd* pParent /*=NULL*/)
            : CViewDialog(CContentsDlg::IDD, pParent) {

}


void CContentsDlg::DoDataExchange(CDataExchange* pDX) {

        CViewDialog::DoDataExchange(pDX);

        //{{AFX_DATA_MAP(CContentsDlg)
        DDX_Control(pDX, IDC_LST_H_TOPICS, m_LST_topics);
        //}}AFX_DATA_MAP
}


void CContentsDlg::OnBtnDisplay() {

        int   pos, titleID, textID;
        DWORD item;


        titleID = IDR_HELP_UNKNOWN_TITLE;
        textID  = IDR_HELP_UNKNOWN_TEXT;

        pos = m_LST_topics.GetCurSel();

        if (pos != LB_ERR) {

                item    = m_LST_topics.GetItemData(pos);
                titleID = LOWORD(item);
                textID  = HIWORD(item);

                }

        pAPP->ShowHelp( titleID, textID, &m_LST_topics );
}

void CContentsDlg::OnHelp() {

        pAPP->ShowHelp( IDR_HELP_HELP_TITLE, IDR_HELP_HELP_TEXT, this );
}

void CContentsDlg::OnCancel() {

        CViewDialog::OnCancel();
}

BOOL CContentsDlg::OnInitDialog() {

        CViewDialog::OnInitDialog( 0 );

        m_LST_topics.ResetContent();

        pAPP->LoadHelpTitles( &m_LST_topics );

        m_LST_topics.SetCurSel(0);


        m_LST_topics.SetFocus();

        return FALSE; // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}

void CContentsDlg::OnDblclkTopics() {

        OnBtnDisplay();
}






//***********************************************************************
//
// CLASS CTopicDlg
//
// This class, derived from CViewDialog, presents and controls the Help
// Topic dialog.  This dialog displays help text for the selected topic.
// This may have been selected directly by the user from the Help
// Contents dialog or it may have been selected automatically by the
// program based on context.

BEGIN_MESSAGE_MAP(CTopicDlg, CViewDialog)
        //{{AFX_MSG_MAP(CTopicDlg)
        ON_WM_SIZE()
        ON_EN_SETFOCUS(IDC_TXT_H_TOPICTEXT, OnSetfocusTopic)
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()


CTopicDlg::CTopicDlg(CWnd* pParent /*=NULL*/)
          : CViewDialog(CTopicDlg::IDD, pParent) {

        TRACE0("CTopicDlg::CTopicDlg\n");

        //{{AFX_DATA_INIT(CTopicDlg)
        //}}AFX_DATA_INIT

        m_TitleID = IDR_HELP_UNKNOWN_TITLE;
        m_TextID  = IDR_HELP_UNKNOWN_TEXT;
}

CTopicDlg::CTopicDlg(int TitleID, int TextID, CWnd* pParent /*=NULL*/)
          : CViewDialog(CTopicDlg::IDD, pParent) {

        TRACE0("CTopicDlg::CTopicDlg\n");

        m_TitleID = TitleID;
        m_TextID  = TextID;
}

void CTopicDlg::DoDataExchange(CDataExchange* pDX) {

        CViewDialog::DoDataExchange(pDX);

        //{{AFX_DATA_MAP(CTopicDlg)
        DDX_Control(pDX, IDC_TXT_H_TOPICTEXT, m_TXT_Topic);
        //}}AFX_DATA_MAP
}


BOOL CTopicDlg::OnInitDialog() {

        int iret;

        TRACE0("CTopicDlg::OnInitDialog\n");

        // Initialize all member variables from application globals


        CViewDialog::OnInitDialog( 0 );



        // Set left and right margin borders in pixels

        m_TXT_Topic.SetMargins( 15, 20 );



        // Set title

        htitle = (LPCSTR)::LockResource(
                         ::LoadResource( NULL,
                         ::FindResource( NULL,
                            (LPCTSTR)m_TitleID, RT_RCDATA)) );
        SetWindowText( htitle );




        // Set text

        htext = (LPCSTR)::LockResource(
                        ::LoadResource( NULL,
                        ::FindResource( NULL,
                            (LPCTSTR)m_TextID, RT_RCDATA)) );
        m_TXT_Topic.SetWindowText( htext );


        CWnd *pWnd = GetDlgItem( IDC_TXT_H_TOPICTEXT );
        if (pWnd) {
                pWnd->SetFocus( );
                iret = FALSE;
                }
        else
                iret = TRUE;


        TRACE0("CTopicDlg::OnInitDialog END\n");

        return iret;  // return TRUE unless you set the focus to a control
                      // EXCEPTION: OCX Property Pages should return FALSE
}



void CTopicDlg::OnSize(UINT nType, int cx, int cy) {

        TRACE0("CTopicDlg::OnSize\n");

        CViewDialog::OnSize( nType, cx, cy );

        m_TXT_Topic.MoveWindow( 0, 0, cx, cy );
}

void CTopicDlg::OnSetfocusTopic() {

        TRACE0("CTopicDlg::OnSetfocusTopic\n");

        m_TXT_Topic.SetSel( 0, 0, FALSE );
}
